/*
JMP Clinical Analytical Process

Name: Adverse Events Distribution
Authors: Russ Wolfinger and Kelci Miclaus, SAS Institute Inc.
Support:  Kelci Miclaus, SAS Institute Inc.
History:
   kemicl JMPCv5.0   Large Re-org for new medical monitor dashboards with counts and percents
   kemicl JMPCv5.1   LoadData work and Clinical variable specification blocks

Description: 
   This process compares distributions of demographic variables across treatment arms via a one-way 
   ANOVA or contingency analysis.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\DomainDistribution.sas"}$
$INCB: {{"EventsO","CrossoverPeriods","AETermValues","TreatmentEmergentEvents"},{"ADSLDistributionOptions"}}$
$INCD: {"AE",{""}}$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CMP: OOUT$ $UE: AE.AESER$ $NOTES: Used in AE Stacking and Data Filter to categorize serious events.$ 
$CMP: OOUT$ $UE: AE.AESEV or AE.AETOXGR$ $NOTES: Used in AE Stacking and Data Filter to categorize events by severity/toxicity.$ 
$CMP: OOUT$ $UE: AE.AEREL$ $NOTES: Used in AE Stacking and Data Filter to categorize events by causality.$ 
$CMP: OOUT$ $UE: AE.AEOUT$ $NOTES: Used in AE Stacking and Data Filter to categorize events by their outcome.$ 
$CMP: OOUT$ $UE: AE.AEACN$ $NOTES: Used in AE Stacking and Data Filter to categorize events by action taken.$ 
$CMP: OOUT$ $UE: AE.EPOCH$ $NOTES: Variable will be available in Event Stacking, Data Filter, and Distributions.$
$CVS_STOP$
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath AETermLevel AEGroupLevel TrtVar TrtUse IncSF PopulationSet DomFiltVarLst
DesignByVars SubjectFilter WhereStmt WhereForReports AEWhereStmt AESerOnly TEAEOnly CalculateRelativeRisks 
CalculateRates RateType RateMultiplier ComputeDuration DurationVar AEIncludeFlag ScreenControlValue
PropThresh TreatmentPropThresh IgnoreTEAEFlags TrtOffset IRTrtOffset  SupportOngoing TableSortOrder
OutPath ServerOutPath CountMultiple Distribution_OutData MacroPath DistributionScript  AEDeltaDrop  MergeSuppDom objReferenceName  EXENDTCFl;

*ProcessBody;

%macro AEDistribution;

   *initialization and argument checking;
   %let ProcessName = AEDistribution; 
   %include "&MacroPath./UtilityMacros.sas" / nosource;
   %include "&MacroPath./CDISCParms.sas";
   %include "&MacroPath./DomainDistribution.sas";
   %if &exiterror %then %goto exit;

   %if ^%length(&SupportOngoing) %then %let SupportOngoing = NO;
   %let textlabel = %nrbquote(%LocaleGet(event_Text));
   %if (%upcase(&CalculateRates) = YES and %upcase(&ComputeDuration) = %nrbquote(DATE) and  %upcase(&RateType) = %nrbquote(PYS)) %then %do;

		%if %sysfunc(findw(&ADSLVARS, RFSTDTC)) %then %let adslst = RFSTDTC;
		%else %let adslst =;
		%if %sysfunc(findw(&ADSLVARS, RFENDTC)) %then %let adslen = RFENDTC;
		%else %let adslen =;

		%if (^%length(&adslst) or ^%length(&adslen)) and %upcase(&SupportOngoing) = NO %then %do;
			%if ^%length(&adslst) %then %let adslst = RFSTDTC;
			%if ^%length(&adslen) %then %let adslen = RFENDTC;
			%put ERROR: The following required variables for calculating Time at Risk as Patient Years on Study are missing: &adslst  &adslen;
			%put ERROR: Please select Patient Years of Exposure for Time at Risk for Incidence Rate calculations.;
			%let exiterror = 1;
			%goto exit;
		%end;
		%else %if ^%length(&adslst) %then %do;
			%put ERROR: The following required variables for calculating Time at Risk as Patient Years on Study are missing: RFSTDTC;
			%put ERROR: Please select Patient Years of Exposure for Time at Risk for Incidence Rate calculations.;
			%let exiterror = 1;
			%goto exit;
		%end;		
	%end;
   *** Prepare ADSL ***;
   %PREPADSL;
   %if &exiterror %then %goto exit;   
 
   *** Get Last Modified Date for Ongoing if calculating Rates */
   %global LastModAEDays;   
   %if %upcase(&CalculateRates) = YES and %upcase(&SupportOngoing) = YES %then %do;  
	   %let _lib = %scan(&in_ae, 1, ".");
	   %let _inDompath=%sysfunc(pathname(&_lib));
	   %let _inDompath=&_inDompath.\;
	   %if ^%sysfunc(exist(&in_ae)) %then %let _ext = xpt; %else %let _ext = sas7bdat;;
	   %if %upcase(&_lib) = ADAM %then %let _domname = ADAE.&_ext; %else %let _domname = AE.&_ext;;
	   %FileAttribs("&_inDomPath.&_domname");
	   
	   data null;
	    mod_dtm = input(symget("FileModifyDT"),datetime20.);
	    mod_dtm = mod_dtm/60/60/24;
	    call symputx("LastModAEDays",mod_dtm);
	  run;
   %end;
   %else %let LastModAEDays = 0;


   *** Load in Domain data to WORK ***; 
   %AESerOnlyToAEWhereStmt;
   %LOADDATA(domlist=AE,checkForZeroObs=1, _TempWhereStmt =&AEWhereStmt,_DeltaFilter=&AEDeltaDrop);   
   %WHEREDELTA(deltastate = AEDeltaDrop, wherestate = AEWhereStmt, deltaflag = AERFlg);
   

   *** Get Variable Based on Drop Down Label ***;
   %let CatName = &AETermLevel;
   %let GroupVar = &AEGroupLevel;
   /*Variable for Count Plot*/
   %let TmName = &CatName;
      
   *** Check if Catname null ***;
   %if ^%sysfunc(findw(&AEVARS, &CatName)) %then %do;
      %put [MS_S_0017];
      %put ERROR: Selected AE term variable has all missing values or is in SUPPAE.; 
      %put ERROR: Please select an alternate variable or select merge supplemental domain in the dialog.;
      %let exiterror = 1;
      %goto exit;
   %end;
      
   *** Check if Group null ***;
   %let TmGroup = &GroupVar;
   %if ^%sysfunc(findw(&AEVARS, &GroupVar)) %then %do;
      %put NOTE: Selected AE grouping variable has all missing values. Grouping variable will not be used.; 
      %put NOTE: Select an alternate variable or select merge supplemental domain in the dialog.;
      %let GroupVar = ;
      %let TmGroup = ;
   %end;
   %else %do;
      *** Ensure that the Grouping Variable is at least the level of Term Variable ***;
      %COMPAREMEDDRA(term = &CatName, group = &GroupVar);
      %if &exiterror %then %goto exit;
      %if &CatName = &GroupVar %then %let TmGroup = ;
   %end;
   
   %if (&NtrtPeriods > 1 and %sysfunc(findw(&AEVARS, %upcase(AESTDTC))) and (%upcase(&TrtUse)=ACTUAL or %upcase(&TrtUse) =PLANNED)) %then %let CalculateRates = No;

   %global calcincidencerates TEAEText MissingPatients;
   %let MissingPatients = 0;
   %let calcincidencerates = 0;
   %if %upcase(&CalculateRates) = YES %then %do;
   	*** Subset to Events in Patient Risk Years ***;
   	%if ^%length(&IRTrtOffset) %then %let IRTrtOffset = 0;
   	%if %upcase(&ComputeDuration) = %nrbquote(DATE) %then %do;
   		/* Create event flag for inclusion and determine dates to use for patient years */
		%if %upcase(&RateType) = %nrbquote(PYE) %then %do;
			%TREATMENTEMERGENT(dataset = AE, domain = AE, date = AESTDTC, offset = &IRTrtOffset, IgnoreFlag = NO, ExEndFlag = NO, ignoreRFdates = YES, useRFdates = NO);
		%end;
		%else %do;
			%TREATMENTEMERGENT(dataset = AE, domain = AE, date = AESTDTC, offset = &IRTrtOffset, IgnoreFlag = NO, ExEndFlag = NO, ignoreRFdates = NO, useRFdates = YES);
		%end;
		data ae;
			set ae;
			where _OTFLAG = 1;
		run;

		%if ^%length(&TEAEVARFLAGe) and %upcase(&SupportOngoing) = NO  %then %do;
			%put ERROR: Required treatment end date variables missing for determining events on Treatment.;
			%put ERROR: TRTEDTM, TRTEDT, RFXENDTC, EXENDTC, are not available.;
			%let exiterror = 1;
		%end;
		
		%let UseLastModTot = 0;
		%let init_inadsl = &in_adsl;		
		%if %upcase(&SupportOngoing) = YES and ^%length(&TEAEVARFLAGe) %then %do;
			%let UseLastModTot = 1;
			data tempinadsl; 
				set &in_adsl;
				TRTEDT = &LastModAEDays;
			run;
			%let TEAEVarFLAGe = TRTEDT;
			%let in_adsl = tempinadsl;
		%end;
      	 /* Calculate patient years to merge in later to data used in demographic output */
   		%include "&MacroPath./TRTDURDY.sas";
   		%if &UseLastModTot %then %do;
   			%let in_adsl = &init_inadsl;
   			%let TEAEVarFLAGe = Last Modified Date of Adverse Event dataset;
   		%end;
   		
   		%let PDurStart =  %nrbquote(%LocaleGet(PatientDurStartDate_Text));
   		/* Patient Duration Start Date */
   		%let PDurEnd = %nrbquote(%LocaleGet(PatientDurEndDate_Text));
   		/* Patient Duration End Date */
   		
		data patientDaysDT;
		 set patientDaysDT;
			/* kemicl support ongoing subject with no end date*/
			%if &UseLastModTot %then %do;
					EndDateImputed = 1;
			%end;
			%else %if &LastModAEDays > 0 %then %do;
				if missing(EndDY) then do;
					EndDY = &LastModAEDays;
					EndDateImputed = 1;
					Offset = 0;					
					_fracST_ = StartDY - floor(StartDY);
					_fracEN_ = EndDY - floor(EndDY);	
					/* kemicl if either start or end is time based, do not add one to account for day rounding */
					if (_fracST_>0 or _fracEN_>0) then PatientDays = (EndDY+Offset) - StartDY;
					else PatientDays = (EndDY+Offset) - StartDY +1;				
				end;
				else do;
					EndDateImputed = 0;
				end;				
			%end;	
			StartDY = StartDY*60*60*24;
			format StartDY E8601DT19.;
			label StartDY = "&PDurStart";
			EndDY = EndDY*60*60*24;
			format EndDY E8601DT19.;			
			label EndDY = "&PDurEnd";
			drop endday startday endtime starttime d9partial: _fracST_ _fracEN_ %if &UseLastModTot %then TRTEDT; ;
		run;   		
   		/* check for patients missing start or end dates */
   		data patientDaysDT missingdates(keep=USUBJID);
   			set patientDaysDT;
   			if missing(EndDY) or missing(StartDY) then output missingdates;
   			else output patientDaysDT;
   		run;
   		%nobsvars(missingdates);
   		%let MissingPatients = &nobs;
   		
   			/* kemicl if doing incidence rates remove subjects and their aes with no data */
   		%if &nobs %then %do;
   			data ae;
					merge ae(in=a) missingdates(in=b);
					by USUBJID;
					if a and not b;
   			run;
   		%end;
		%end;
		%else %do;
		/*kemicl: else do for has flags and variables already goes here. 
			- For AE flag, will need to check if C or N type and then subset records to Y or YES or 1.
			- for Duration variable, keep it with USUBJID from adsl and create PatientDaysDT data set and create patientDays variable
			from the specd duration variable.  
			*/
			data _null_;
				set ae(keep = &AEIncludeFlag);
				call symputx("aefltype",vtype(&AEIncludeFlag));
				stop;
			run;
			
			data ae;
				set ae;
				%if &aefltype = "N" %then %do;
					where &AEIncludeFlag = 1;
				%end;
				%else %do;
					where kupcase(&AEIncludeFlag) in ("Y","YES",%unquote(%LocaleGetAll(Y_Text, _case = u)),%unquote(%LocaleGetAll(Yes_Text, _case = u)));
				%end;
			run;
			
			data patientDaysDT;
				set &in_adsl (keep = USUBJID &DurationVar);
				PatientDays = &DurationVar;
			run;
			%let TEAEVARFLAGs = &AEIncludeFLAG;
			%let TEAEVARFLAGe = &DurationVar;
		%end;
		
		%nobsvars(ae);
		%if &nobs = 0 %then %do;
			%put [MS_S_0000];
			%put ERROR: There are no adverse events after filtering to only include records occurring within time at risk period for incidence rates.;
			%let exiterror = 1;
			%goto exit;
		%end;    		
		%let calcincidencerates = 1;
		%if %upcase(&RateType) = %nrbquote(PYE) %then %do;
			%let trterm = %nrbquote(%LocaleGet(Ontreatmenteventsandincidencerates_Text));   				
		%end;
		%else %do;
			%let trterm = %nrbquote(%LocaleGet(Onstudyeventsandincidencerates_Text));  
		%end;		
		 %let trsent = %nrbquote(%LocaleGet(XdeterminedusingYandZ_Text));
		 %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARFLAGs, &TEAEVARFLAGe));		
   %end;
   %else %do;
		*** Subset to Event Type ***;
		%include "&MacroPath./TEAEOTFU.sas";
	%end;
   %if &exiterror %then %goto exit; 
	/* For report output */
	%let WhereForReports=&WhereStmt;
	%let DomainWhereStmt = %nrbquote(&AEWhereStmt);
 	
   /* S1125285 kemicl check for allempty values of term in filtered data */
   data valcheck;
      set AE;
      where not missing(&TmName);
   run;
   
   %nobsvars(valcheck);
   %if &nobs = 0 %then %do;
      %put [MS_S_0026];
      %put ERROR: All values of %upcase(&TmName) are missing after filtering data based on dialog specifications.;
      %let exiterror = 1;
      %goto exit;
   %end;    
   /* end S1125285 */
   
   /*get treatment groups and control level for calculation of relative risks*/
   %if %upcase(&CalculateRelativeRisks) = YES %then %do;
      %let doTreatComp = 1;
      %let NeedTwoLevels = 0;
      %let NeedBinary = 0;
      * determine treatment levels;
      %include "&MacroPath./TreatmentLevels.sas" / nosource;
      %if &exiterror %then %goto exit;
      %put The value &&tlev&plev will be used as the control for treatment comparison incidence; 
      %if &nlev = 1 %then %let doTreatComp = 0; 
   %end;
   %else %do;
      %let doTreatComp = 0;
   %end;
   
	*** determine presence of MULTIPLE values and look to SUPP variables ***;
	%let AERELMULT = 0;
	%let nAERelVals = 0;
	%let AEACNMULT = 0;
	%let nAEAcnVals = 0;
	%let AERELlist = AEREL;
	%let AEACNlist = AEACN;
	%if (%sysfunc(findw(&suppdomains, %upcase(SUPPAE))) and &MergeSuppDom)  %then %do;
		%let trMultiple = %nrbquote(%LocaleGet(Multiple_Text));
		%if %sysfunc(findw(&AEVARS, AEREL)) %then %do;
			proc sql noprint;         
				select distinct upcase(aerel) into :aerelvals separated by '~'
				from AE;
			quit;
			%if %sysfunc(findw(&aerelvals,%upcase(&trMultiple), ~)) %then %let AERELMULT = 1;
			%let nAERelVals = %sysfunc(countw(&aerelvals,~));
		%end;
		%if %sysfunc(findw(&AEVARS, AEACN)) %then %do;
			proc sql noprint;         
				select distinct upcase(aeacn)
					into :aeacnvals separated by '~'
					from AE;
			quit;
			%if %sysfunc(findw(&aeacnvals,%upcase(&trMultiple), ~)) %then %let AEACNMULT = 1;
			%let nAEAcnVals = %sysfunc(countw(&aeacnvals,~));
		%end;   

		%if &AERELMULT %then %do;
			%let AERELlist = ;
			%let i =1;
			%do %while(%length(%scan(&AEVARS,&i)));
			  %let currv = %scan(&AEVARS,&i);
			  %if %upcase(%substr(&currv,1,5)) = AEREL or  %upcase(%substr(&currv,1,3)) = REL  %then %do;
					 %if &nAERelVals=1 and %upcase(&currv) = AEREL %then %let currv = ;
					 %let AERELlist = &AERELlist &currv;
			  %end;
			 %let i=%eval(&i+1);
			%end;
		%end;

		%if &AEACNMULT %then %do;
			%let AEACNlist = ;
			%let i =1;	
			%do %while(%length(%scan(&AEVARS,&i)));
			  %let currv = %scan(&AEVARS,&i);
			  %if %upcase(%substr(&currv,1,5)) = AEACN or  %upcase(%substr(&currv,1,3)) = ACN  %then %do;
					 %if &nAEAcnVals=1 and %upcase(&currv) = AEACN %then %let currv = ;
					 %let AEACNlist = &AEACNlist &currv;
			  %end;
			 %let i=%eval(&i+1);
			%end;
		%end;
	%end;

   %global AEevars;
   %let AEvarlist = AESER  AESEV AETOXGR AEOUT &AERELlist &AEACNlist;
   %let AEevars = ;
   %let i = 1;
   %do %while(%length(%scan(&AEvarlist,&i)));
      %let currv = %scan(&AEvarlist,&i);
      %if %sysfunc(findw(&AEVARS, &currv)) > 0 %then %do;
     		 %let AEevars = &AEevars &currv;
      %end;
      %let i=%eval(&i+1);
   %end;

   %if (%length(&DomFiltVarLst) & &AEACNMULT) %then %do;
   	%if %sysfunc(findw(&DomFiltVarLst,AEACN)) %then %do;
   		%let DomFiltVarLst = %sysfunc(tranwrd(&DomFiltVarLst, AEACN, &AEACNlist));
   	%end;
   %end;
   
  %if (%length(&DomFiltVarLst) & &AERELMULT) %then %do;
   	%if %sysfunc(findw(&DomFiltVarLst,AEREL)) %then %do;
   		%let DomFiltVarLst = %sysfunc(tranwrd(&DomFiltVarLst, AEREL, &AERELlist));
   	%end;
   %end;   
   
   %DomainDistribution(ae,fillmissingVars = &TmName &TmGroup ,countplotVars=&TmName, stackVars=&AEevars, distributionVars= &AEevars &TmGroup , customFilterVars =&DomFiltVarLst, doTreatmentRRs=&doTreatComp, doRates = &calcincidencerates);
   %if &exiterror=1 %then %goto exit;

   %exit:;
   %ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AEDistribution;

* run the macro;
%AEDistribution;


