/*
JMP Clinical Analytical Process

Name: CM Distribution
Authors: Russ Wolfinger and Kelci Miclaus, SAS Institute Inc.
Support: Kelci Miclaus
History:
   kemicl JMPCv5.0   Large Re-org for new medical monitor dashboards with counts and percents
   kemicl JMPCv5.1   LoadData work and Clinical variable specification blocks

Description: 
   This process compares distributions of demographic variables across treatment arms via a one-way 
   ANOVA or contingency analysis.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\DomainDistribution.sas"}$
$INCB: {{"InterventionsO","CrossoverPeriods","TreatmentEmergentInterventions"},{"ADSLDistributionOptions"}}$
$INCD: {{""},{""}}$
$CMP: RSYS.Term Variable$ $UE: INTERVENTIONS.xxDECOD or INTERVENTIONS.xxTRT$ 
   $NOTES: At least one variable that provides the intervention term value must be present. For EX EXTRT required.$ 
$CMP: CPV.{"Term Level","Reported Name"}$ $UE: INTERVENTIONS.xxTRT$ $NOTES:  Counts and frequencies of values are summarized.$
$CMP: CPV.{"Term Level","Standardized Name"}$ $UE: INTERVENTIONS.xxDECOD$ $NOTES: Counts and frequencies of values are summarized.$ 
$CMP: OOUT$ $UE: INTERVENTIONS.xxDOSE$ $NOTES: Used in Data Filter and shown in distributions.$ 
$CMP: OOUT$ $UE: INTERVENTIONS.xxCAT$ $NOTES: Used in Data Filter and shown in distributions.$ 
$CMP: OOUT$ $UE: INTERVENTIONS.EPOCH$ $NOTES: Variable will be available in Intervention Stacking, Data Filter, and Distributions.$
$CVS_STOP$
*/

*ProcessVariables;
*options mprint mlogic symbolgen;
%global StudyName InterventionsDomain InPath AInPath InterventionsTermLevel InterventionsGroupLevel ConMedOnly
TrtVar TrtUse IncSF PopulationSet DesignByVars PropThresh  CalculateRelativeRisks ScreenControlValue  
SubjectFilter WhereStmt WhereForReports InterventionsWhereStmt CountMultiple  MergeSuppDom
OutPath ServerOutPath Distribution_OutData MacroPath DistributionScript IntDeltaDrop TableSortOrder
DomFiltVarLst ITEAEOnly objReferenceName IgnoreTEAEFlags EXENDTCFl;

*ProcessBody;
%macro InterventionsDistribution;

*initialization, load macros and argument checking;
%let ProcessName = InterventionsDistribution; 

%if ^%length(%trim(&InterventionsDomain)) %then %do;
   %put [MS_S_0130];
   %put ERROR: No interventions domain selected.  You may need to add one or more to your study.;
   %let exiterror = 1;
   %goto exit;
%end;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./DomainDistribution.sas";
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
 
*** Prepare ADSL ***;
%PREPADSL;
%if &exiterror %then %goto exit;

 	*** Load in Domain data to WORK ***; 
%let domain = %lowcase(%unquote(&InterventionsDomain)); 	
%let trNew = %nrbquote(%LocaleGet(New_Text));
%let trStable = %nrbquote(%LocaleGet(Stable_Text));
%LOADDATA(domlist=&InterventionsDomain, checkForZeroObs=1, _TempWhereStmt =&InterventionsWhereStmt,_DeltaFilter=&IntDeltaDrop);
%WHEREDELTA(deltastate = IntDeltaDrop, wherestate = InterventionsWhereStmt, deltaflag = &domain.RFlg);  

   /* For report output */
%let WhereForReports=&WhereStmt;
%let DomainWhereStmt = %nrbquote(&InterventionsWhereStmt);
	*** Subset to Intervention Type ***;
%include "&MacroPath./TEAEOTFUInt.sas";	
%TEAEInt(dom = %upcase(&domain));   

%let textlabel = %nrbquote(%LocaleGet(intervention_Text));
%let TmName = &InterventionsTermLevel;
%let TmGroup = &InterventionsGroupLevel;

%if %sysfunc(findw(&&&InterventionsDomain.VARS, &TmName)) = 0 %then %do;
	%put [MS_S_0084];
	 %put ERROR: Selected Term Level &TmName has all missing values or is in supplemental data.; 
	%put ERROR: Please select an alternate variable or select merge supplemental domain in the dialog.;
	%let exiterror=1;
	%goto exit;
%end;

*** Check if Group null ***;
%let TmGroup = &InterventionsGroupLevel;
%if ^%sysfunc(findw(&&&InterventionsDomain.VARS, &TmGroup)) %then %do;
	%put NOTE: Selected Group Level has all missing values or is in supplemental data. Grouping variable will not be used.; 
	%put NOTE: Please Select an alternate variable or select merge supplemental domain in the dialog.;
	%let TmGroup = ;
%end;

/*kemicl add in extra grouping vars to the distributions */
%let GroupsVarList = &TMGroup;
/*Check for other groupings that we should include in distributions */
%let PossibleGrpVars =&InterventionsDomain.CAT &InterventionsDomain.SCAT &InterventionsDomain.BODSYS  &InterventionsDomain.CLAS ;
%let j = 1;
%do %while (%length(%scan(&PossibleGrpVars,&j)));
	%let currGP = %scan(&PossibleGrpVars,&j);
	%if (%sysfunc(findw(&&&InterventionsDomain.VARS, &currGP)) > 0 and %sysfunc(findw(&GroupsVarList, &currGP)) = 0 ) %then %let GroupsVarList = &GroupsVarList &currGP;
	%let j = %eval(&j+1);
%end;

/* S1125285 kemicl check for all empty values of term in filtered data */
data valcheck;
   set &domain;
   where not missing(&TmName);
run;

%nobsvars(valcheck);
%if &nobs = 0 %then %do;
   %put [MS_S_0085];
   %put ERROR: All values of %upcase(&TmName) are missing after filtering data based on dialog specifications.;
   %let exiterror = 1;
   %goto exit;
%end;    
/* end S1125285 */
   
%let SLDOSEVar = 0;
%let SLCATVar = 0;
%if %sysfunc(findw(&&&domain.VARS, %upcase(&domain)DOSE)) > 0 %then %do;
   %let SLDOSEVar = 1;
   %let DoseLabel = &domain.DOSE;
   %CheckLabel(&domain,DoseLabel);
%end;
%if %sysfunc(findw(&&&Domain.VARS, %upcase(&Domain)CAT)) > 0 %then %do;
   %let SLCATvar = 1;
   %let CatLabel = &domain.CAT;
   %CheckLabel(&domain,CatLabel);
%end;

/*get treatment groups and control level for calculation of relative risks*/
%if %upcase(&CalculateRelativeRisks) = YES %then %do;
	%let doTreatComp = 1;
	%let NeedTwoLevels = 0;
	%let NeedBinary = 0;
	* determine treatment levels;
	%include "&MacroPath./TreatmentLevels.sas" / nosource;
	%if &exiterror %then %goto exit;
	%put The value &&tlev&plev will be used as the control for treatment comparison incidence; 
	%if &nlev = 1 %then %let doTreatComp = 0; 
%end;
%else %do;
	%let doTreatComp = 0;
%end;

%DomainDistribution(&domain, fillmissingVars = &TmName &TmGroup, countplotVars = &TmName, distributionVars=&GroupsVarList,customFilterVars =&DomFiltVarLst,  doTreatmentRRs=&doTreatComp);

%if &exiterror=1 %then %goto exit;

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend InterventionsDistribution;

* run the macro;
%InterventionsDistribution;