/*
JMP Clinical Analytical Process

Name: Mortality Kaplan Meier
Authors: Geoff Mann and Russ Wolfinger, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.

History:
   tzuchu 09Oct14 added FDA Overall Survival Analysis Pannel v1.0

Description: 
   This process creates Kaplan-Meier survival curves and associated statistics, grouped
   by treatment arm.
*/

/*
-----------------------------------------------------------
Copyright (C) 2009 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/
*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"LoadDataO","LoadDataO","LoadDataForDM","LoadDataO","LoadDataO"},{"GeneralOptions"}}$
$INCD: {{"LB","DS","DM","MH","CM"},{"LB"}}$
$CMP: RSYS$ $UE: DS.DSDECOD$
$CMP: RSYS$ $UE: DM.RFSTDTC$
$CMP: RSYS$ $UE: DM.RFENDTC$
$CMP: CPV.{"End Date Variable to Use for Censoring","Subject Reference End Date"}$ $UE: DM.RFENDTC$
$CMP: CPV.{"End Date Variable to Use for Censoring","Date of Last Study Treatment"}$ $UE: DM.RFXENDTC$
$CMP: CPV.{"End Date Variable to Use for Censoring","Date of End of Participation"}$ $UE: DM.RFPENDTC$
$CMP: OSYS$ $UE: DS.DSCAT$ $NOTES: If available, subset to DS records where = DISPOSITION EVENT.$
$CMP: OOUT$ $UE: DM.SEX$
$CMP: OOUT$ $UE: DM.RACE$
$CMP: OOUT$ $UE: DM.COUNTRY$
$CMP: OOUT$ $UE: DM.SITEID$
$CMP: OOUT$ $UE: LB.LBTESTCD$
$CMP: OOUT$ $UE: LB.LBTEST$
$CMP: OOUT$ $UE: LB.LBSTNRLO$
$CMP: OOUT$ $UE: LB.LBSTNRHI$
$CMP: OOUT$ $UE: CM.CMDECOD$
$CMP: OOUT$ $UE: CM.CMTRT$
$CMP: OOUT$ $UE: MH.MHDECOD$
$CMP: OOUT$ $UE: MH.MHTERM$
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName InPath AInPath TrtUse TrtVar CensorDateVar IncSF PopulationSet DeathThreshold
DesignByVars SubjectFilter WhereStmt WhereForReports OutPath ServerOutPath MacroPath 
AgeGroupFlag AgeGroup1 AgeGroup2 AgeGroup3 AgeGroup4 AgeGroup5 LB_High_Multiplier LB_Low_Multiplier
MortalityKaplanMeier_OutData MortalityKaplanMeierScript BeginTime EndTime TimeScale
MergeSuppDom ScreenControlValue MortalityKaplanMeier_OutReport MortalityKaplanMeier_Body MortalityKaplanMeier_Contents MortalityKaplanMeier_Frame
HR_Flag_AgeGroup HR_Flag_Gender HR_Flag_Race HR_Flag_Ethnicity HR_Flag_Country HR_Flag_StudySite
HR_Flag_LBH HR_Flag_LBL HR_Flag_CM HR_Flag_MH shadedCI dieThreshold;

*ProcessBody;

%macro MortalityKaplanMeier;
*options mprint mlogic symbolgen;
%let WhereForReports=&WhereStmt;

*initialization and argument checking;
%let ProcessName = MortalityKaplanMeier; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./FindingsMacros.sas";
%if &exiterror %then %goto exit;

%if ^%sysfunc(findw(&domains, DS)) %then %do;
   %put [MS_S_0142];
   %put ERROR: The DS or ADDS data set is required for mortality information.;
   %let exiterror = 1;
   %goto exit;
%end;

* check more required domains;
%if ^%sysfunc(findw(&domains, DM)) and ^%length(&ADSLPRESENT) %then %do;
   %put [MS_S_0144];
   %put ERROR: The ADSL, DM, or ADDM data set is required for mortality information.;
   %let exiterror = 1;
   %goto exit;
%end;

*** Prepare ADSL ***;
%PREPADSL(_ConvertEndDateFlag=0);
%if &exiterror %then %goto exit;

%let dthdtcflag = %sysfunc(indexw(&ADSLVARS, DTHDTC, ' '));
%let dthflflag = %sysfunc(indexw(&ADSLVARS, DTHFL, ' '));

%if %sysfunc(findw(&domains, DS)) %then %do;
	%let dsFlag =1;
%end;
%let _DM_Flag_ = 1;
%let _LB_Flag_ = 1;
%let _Skip_LB_H = 0;
%let _Skip_LB_L = 0;
%if ^%sysfunc(findw(&domains, LB)) %then %do;
   %let _LB_Flag_ = 0;
   %let _Skip_LB_H = 1;
   %let _Skip_LB_L = 1;
%end;

%let _CM_Flag_ = 1;
%let _Skip_CM = 0;
%if ^%sysfunc(findw(&domains, CM)) %then %do;
   %let _CM_Flag_ = 0;
   %let _Skip_CM = 1;
%end;

%let _MH_Flag_ = 1;
%let _Skip_MH = 0;
%if ^%sysfunc(findw(&domains, MH)) %then %do;
   %let _MH_Flag_ = 0;
   %let _Skip_MH = 1;
%end;

* check subgroup choices;
%if %upcase(&HR_Flag_LBH) = NO %then %do;	
   %let _Skip_LB_H = 1;
%end;
%if %upcase(&HR_Flag_LBL) = NO %then %do;	
   %let _Skip_LB_L = 1;
%end;

%if %upcase(&HR_Flag_CM) = NO %then %do;	
   %let _Skip_CM = 1;
%end;
%if %upcase(&HR_Flag_MH) = NO %then %do;	
   %let _Skip_MH = 1;
%end;

%if &CensorDateVar = RFENDTC %then %do;
	%let CensorDateValue = Subject Reference End Date;
%end;
%else %if &CensorDateVar = RFXENDTC %then %do;
        %if %sysfunc(findw(%upcase(&ADSLVARS), TRTEDT)) %then %let CensorDateVar = TRTEDT;
	%let CensorDateValue =Date of Last Study Treatment;
%end;
%else %if &CensorDateVar = RFPENDTC %then %do;
        %if %sysfunc(findw(%upcase(&ADSLVARS), EOSDT)) %then %let CensorDateVar = EOSDT;
	%let CensorDateValue =Date of End of Participation;
%end;
%else %do;
	%let CensorDateValue =Last Disposition Record Date;
%end; 

%if &shadedCI =Yes %then %let shadedCI=1;
%else %let shadedCI=0;

* Check age grouping;
%if %upcase(&AgeGroupFlag)=YES %then %do;
	%let Aflag = 0;
   %if %length(%trim(&AgeGroup1)) %then %let FAG1 = 1;
   %else %let FAG1 = 0;
   %if %length(%trim(&AgeGroup2)) %then %let FAG2 = 1;
   %else %let FAG2 = 0;
   %if %length(%trim(&AgeGroup3)) %then %let FAG3 = 1;
   %else %let FAG3 = 0;
   %if %length(%trim(&AgeGroup4)) %then %let FAG4 = 1;
   %else %let FAG4 = 0;
   %if %length(%trim(&AgeGroup5)) %then %let FAG5 = 1;
   %else %let FAG5 = 0;
   %if %eval(&FAG1+&FAG2+&FAG3+&FAG4+&FAG5) < 2 %then %do;
      %ERROR: You must specify at least 2 Age groups.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %let Aflag = %sysevalf(&FAG1+&FAG2+&FAG3+&FAG4+&FAG5);
%end;
%else %do;
	%let AgeGroup1 = All;
	%let FAG1 = 1;
	%let FAG2 = 0;
	%let FAG3 = 0;
	%let FAG4 = 0;
	%let FAG5 = 0;
%end;

*** check if there is any overlapping between any age groups ***;
%if %upcase(&AgeGroupFlag)=YES %then %do;
	%do i =1 %to &Aflag;
		data age_sub&i (keep=USUBJID);
			set &in_adsl;
			if &&AgeGroup&i;
		run;
		proc sort data =age_sub&i;
			by USUBJID;
		run;
	%end;
	%let AflagL=%eval(&Aflag+1);
	%do i =1 %to &AflagL;
		%do j=%eval(&i+1) %to &Aflag;
			data suberr;
				merge age_sub&i(in=ina) age_sub&j(in=inb);
				by USUBJID;
				if ina & inb;
			run;
			%NObsVars(suberr);
			%put suberr=&nobs;
			%if &nobs %then %do;
				%put ERROR: There is an overlap between the age groups: &&AgeGroup&i and &&AgeGroup&j.;
				%put ERROR: Please redefine the age groups such that they are mutually exclusive with no overlap.;
				%let exiterror = 1;
   			%goto exit;
			%end;
		%end;
	%end;
%end;

* create DIED variable from DS and merge with adsl;
%let DeathLabel = %nrbquote(%LocaleGet(SubjectDied_Text));
%let trdeath = %upcase(%nrbquote(%LocaleGet(Death_Text)));
%let trtreatment = %upcase(%nrbquote(%LocaleGet(Treatment_Text)));
%let trscreening = %upcase(%nrbquote(%LocaleGet(Screening_Text)));
%let trdied = %upcase(%nrbquote(%LocaleGet(Died_Text)));
%let trdead = %upcase(%nrbquote(%LocaleGet(Dead_Text)));
%let trNotTreated = %nrbquote(%LocaleGet(NotTreated_Text));
%let trNotAssigned = %nrbquote(%LocaleGet(NotAssigned_Text));
* shall add this one to translation lookup table;
%let trDispositionEvent = %upcase(%nrbquote(%LocaleGet(DispositionEvent_Text)));
%let trComplete= %qupcase(%nrbquote(%LocaleGet(Complete_Finished_Text)));
%let trCompleted= %qupcase(%nrbquote(%LocaleGet(Completed_Text)));


* error check for end date variable to use for censoring;
%if %upcase(%substr(&CensorDateVar, 1, 2)) = DS %then %do;
	%if (^%index(&DMVars,RFSTDTC) and ^%index(&ADSLVars,TRTSDT)) | ^%index(&DSVars,&CensorDateVar) %then %do;
	   %put [MS_S_0145];
	   %put ERROR: Either RFSTDTC and TRTSDT are missing or &CensorDateVar is missing. These variables are required to determine the time-to-event variable for performing survival analysis.; 
	   %let exiterror= 1;
	   %goto exit;
	%end;
%end;
%else %do;
	%if (^%index(&DMVars,RFSTDTC) and ^%index(&ADSLVars,TRTSDT)) | ^%index(&ADSLVars,&CensorDateVar) %then %do;
	   %put [MS_S_0145];
	   %put ERROR: Either RFSTDTC and TRTSDT are missing or &CensorDateVar is missing. These variables are required to determine the time-to-event variable for performing survival analysis.; 
	   %let exiterror= 1;
	   %goto exit;
	%end;
%end;

* handeling DS dataset;
%LOADDATA(domlist=DS);

%PrepOutDS(MortalityKaplanMeier_OutData,OutDataName,adsl,_dkm);
data adsl;
	set &in_adsl;
	* if DTHDTC and DTHFL exist in DM table;
	%if &dthdtcflag %then %do;
		%calcStudyDay(DTHDTC,DTHDAYS);
	%end;
	* calculate censor cut off days;
	%if &CensorDateVar = RFENDTC %then %do;
		%calcStudyDay(RFENDTC,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = RFXENDTC %then %do;
		%calcStudyDay(RFXENDTC,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = TRTEDT %then %do;
		%calcStudyDay(TRTEDT,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = RFPENDTC %then %do;
		%calcStudyDay(RFPENDTC,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = EOSDT %then %do;
		%calcStudyDay(EOSDT,DCUTDAYS);
	%end;
run;
* if censor cutoff date is the last disposition record date, need to get the last DS date;
%if &CensorDateVar = DSSTDTC %then %do;
	* remove subjects with DS days <=1;
	data ds;
		set ds;
		if STDY >1 or upcase(DSDECOD) in ("DEATH","DIED","DEAD","&trdeath","&trdied","&trdead") or (STDY = 1 and upcase(DSCAT) in ("DISPOSITION EVENT","&trDispositionEvent"));
	run;

	proc sort data =ds;
		by USUBJID descending STDY;
	run;
	data ds;
		retain DCUTDAYS;
		set &in_ds;
		by USUBJID;
		if first.USUBJID then DCUTDAYS=STDY;
		else DCUTDAYS=DCUTDAYS;
	run;
%end;	
* get death for subjects in ds table;
data died;
	set ds;
	if upcase(DSDECOD) in ("DEATH","DIED","DEAD","&trdeath","&trdied","&trdead") then DIED = 1;
	else DIED = 0;
	keep USUBJID DSSTDTC STDY DIED %if &CensorDateVar = DSSTDTC %then DCUTDAYS;
run;

proc sort data=died;
	by USUBJID  descending DIED descending STDY;
run;
data died;
	set died;
	by USUBJID;
	if first.USUBJID;	
	rename STDY = DSSTDAYS;
run;

proc sql;
	create table adsl as
	select *
	from adsl d1
	left join died d2
	on d1.USUBJID=d2.USUBJID;
quit;
*get number of subjects who didnt die and who are missing the censor date value and delete them;
%let DCUTDmissflag = 0;
data DCUTDaymissing;
	set adsl;
	if missing(DCUTDAYS);
run;
%nobsvars(DCUTDaymissing);
%if &nObs > 0 %then %do;
	%let DCUTDmissflag =1;
	%let missingObs= &nObs;
%end;

* get time to mortality days and censor cut off days;
data OutLib.&OutDataName;
	set adsl;
	if missing(DIED) then DIED=0;
	if missing (DCUTDAYS) then delete;
	* if there are Subject Death Flag and time of death in DM table;
	%if &dthdtcflag %then %do;
		if DTHDAYS ^=. then do;
			* set the time to mortality to time of death in DM table;
			TimetoM = DTHDAYS;
			if DIED =. or DIED =0 then DIED =1;
		end;
		else do;
			* if death flag in DM is not Yes, but DSDECOD code is death in DS, then set time to mortality to time of DS event;
			if DIED = 1 then TimetoM =DSSTDAYS;
			else do;
				TimetoM =DCUTDAYS;
			end;
		end;
	%end;
	%else %do;
		if DIED =1 then do;	
			TimetoM = DSSTDAYS;
		end;
		else do;
			TimetoM = DCUTDAYS;
		end;
	%end;
	DCUTDAYS = DCUTDAYS + &dieThreshold;
	label DIED ="&DeathLabel";
	if kupcase(&TrtName) in (%unquote(%LocaleGetAll(NotTreated_Text, _case = u))) or  kupcase(&TrtName) = "NOTTRT"
      or kupcase(&TrtName) in (%unquote(%LocaleGetAll(NotAssigned_Text, _case = u))) or  kupcase(&TrtName) = "NOTASSGN" then delete;   
run;

* check vartype for Treatment to see if numeric;
%let dsid=%sysfunc(open(OutLib.&OutDataName));
%let TPos=%sysfunc(varnum(&dsid,&TrtName));
%global TType;
%let TType=%sysfunc(vartype(&dsid,&TPos)); 
%let rc1=%sysfunc(close(&dsid));  

* handling ScreenControlValue;
proc sort data=OutLib.&OutDataName(keep=&TrtName) out=tmp nodupkey;
   by &TrtName;
run;
%nobsvars(tmp);
%let nTrtGroup = &nObs;

%if %length(&ScreenControlValue) %then %do;
   data _null_;
      set tmp end=_e_;
      name = "trt_" || trim(left(_n_));
      call symput(name,trim(left(&TrtName)));
      if _e_ then call symput("_nTrt_",_n_);
   run;
   
	%let ScreenControlValue = %AddQuotes(&ScreenControlValue);
	%let TRLFlag = 0;
	%do i = 1 %to &_nTrt_;
		%if %nrbquote("%upcase(&&trt_&i)") = %nrbquote(%upcase(&ScreenControlValue)) %then %do;
			%let TRLFlag = 1;
			%let ScreenControlValue = "&&trt_&i";
		%end;
	%end;
	%*put ScreenControlValue=&ScreenControlValue;
	%*put TRLFlag=&TRLFlag;

   %if ^&TRLFlag %then %do;
      %put [MS_S_0146];
      %put ERROR: The specified Treatment Control Level, &ScreenControlValue, was not found.;
      %goto exit;
   %end;
%end;
%else %do;
   * handling "Screen Failure" as the last level of ARM;
   data tmp1;
      set tmp end=_e_;
      if _e_ then do;
         if kupcase(&TrtName) = "SCREEN FAILURE" | kupcase(&TrtName) = "SCRNFAL" |
            kupcase(&TrtName) in (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) then do;
            call symput("_SF_",&TrtNAme);
            call symput("_nSF_",_n_);
            call symput("_dSF_",1);
            delete;
         end;
         else do;
            call symput("_dSF_",0);
         end;
      end;
   run;
   
   * Check if there is a level named "Placebo" or "Pbo";
   %let TRLFlag = 0;
   data _null_;
        set tmp end=_e_;     
      if _e_ then call symput("LevelLast",trim(left(&TrtName)));
      if kupcase(&TrtName) = "PLACEBO" | kupcase(&TrtNAme) = "PBO" |
            kupcase(&TrtName) in (%unquote(%LocaleGetAll(Placebo_Text, _case = u))) then do;
         call symput("_ControlValue_",trim(left(&TrtName)));
         call symput("TRLFlag",1);
      end;
   run;
   %if ^&TRLFlag %then %do;
      %if &TType = N %then %let ScreenControlValue = &LevelLast;
      %else %let ScreenControlValue = "&LevelLast";
   %end;
   %else %do;
      %let ScreenControlValue = "&_ControlValue_";
   %end;
%end;

* re-ordering trtment levels;
data Trt_Num;
   set tmp;
   retain _c_ 0;
   if &TrtName = &ScreenControlValue then do;
      Trt_Num = &nTrtGroup;
   end;
   else do;
      _c_ + 1;
      Trt_Num = _c_;
   end;
   drop _c_;
run;

proc sort data=Trt_Num;
   by Trt_Num;
run;

data _null_;
   set Trt_Num;
   name = "ARM_" || trim(left(Trt_Num));
   call symput(name,&TrtName);
run;
   
data OutLib.&OutDataName;
   if 0 then set Trt_Num;
   if _n_ = 1 then do;
      declare hash h(dataset: "work.Trt_Num", hashexp: 6);
      h.defineKey("&TrtName");
      h.defineData('Trt_Num');
      h.defineDone();
   end;
   set OutLib.&OutDataName;
   if h.find() = 0;
run;
 
/*
* use max death day + 1 as censoring day for all living patients;
proc means data=OutLib.adsl_dkm noprint;
   var DTHDAYS;
   output out=dmax max=dmax;
run;

data _null_;
   set dmax;
   call symput('dmax',trim(left(dmax)));
run;

%put dmax = &dmax;
*/

* create time to event and censor variables;
%let trcensor = %nrbquote(%LocaleGet(Censor_Text));
%if %upcase(&TimeSCALE) = DAYS %then %do;
   %let trTimeSCALE = %nrbquote(%LocaleGet(Days_Text));
   %let trSingleTS = %nrbquote(%LocaleGet(Day_Text));
%end;
%else %if %upcase(&TimeSCALE) = WEEKS %then %do;
   %let trTimeSCALE = %nrbquote(%LocaleGet(Weeks_Text));
   %let trSingleTS = %nrbquote(%LocaleGet(Week_Text));
%end;
%else %do;
   %let trTimeSCALE = &TimeSCALE;
   %let trSingleTS = &TimeSCALE;
%end; 
%let trxsurv = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XSurviving_Text)), %nrbquote(&trTimeSCALE)));

data OutLib.&OutDataName;
   set OutLib.&OutDataName;
   if DIED=1 then do;
   	* compre time to death with censor cut off date;
   	* if time to death is within cut off date, then set censor to 0;
   	* if time to death is out of cut off date, then set censor to 1;
   	if TimetoM > DCUTDAYS then do;
   		CENSOR = 1;
   		SURV%upcase(&TimeSCALE) =DCUTDAYS;
   	end;
		else do;
			CENSOR = 0;
			SURV%upcase(&TimeSCALE) = TimetoM;
		end;
      /* SURVDAYS = &dmax + 1; */   
   end;
   else do;
   	CENSOR = 1;
   	SURV%upcase(&TimeSCALE) =DCUTDAYS;
   end;
   label CENSOR = "&trcensor";
   %if %upcase(&TimeSCALE) = WEEKS %then %do;
      SURV%upcase(&TimeSCALE) = SURV%upcase(&TimeSCALE) / 7;
   %end;
   %if %length(&EndTime) %then %do;
      if SURV%upcase(&TimeSCALE) > &EndTime then do;
         CENSOR = 1;
         SURV%upcase(&TimeSCALE) = &EndTime;
      end;
      %end;
   %if %length(&BeginTime) %then %do;
      if SURV%upcase(&TimeSCALE) >= &BeginTime;
      SURV%upcase(&TimeSCALE) = SURV%upcase(&TimeSCALE) - &BeginTime;
   %end;
   if SURV%upcase(&TimeSCALE) < 0 then SURV%upcase(&TimeSCALE)=0;
   * added Age Groups;
   %if %upcase(&AgeGroupFlag)=YES %then %do;
      %if &FAG1 %then %do;
         if (&AgeGroup1) then NewAgeGroup = 1; 
      %end;
      %if &FAG2 %then %do;
         if (&AgeGroup2) then NewAgeGroup = 2; 
      %end;
      %if &FAG3 %then %do;
         if (&AgeGroup3) then NewAgeGroup = 3; 
      %end;
      %if &FAG4 %then %do;
         if (&AgeGroup4) then NewAgeGroup = 4; 
      %end;
      %if &FAG5 %then %do;
         if (&AgeGroup5) then NewAgeGroup = 5; 
      %end;
   %end;
   %else %do;	
   	NewAgeGroup = 1;
   %end;
   label SURV%upcase(&TimeSCALE) = "&trxsurv";
   drop DSSTDTC DSSTDAYS;
run;
data OutLib.&OutDataName;
	set OutLib.&OutDataName;
	if missing(SURV%upcase(&TimeSCALE) ) or SURV%upcase(&TimeSCALE) =0 then do;
		CENSOR =1;
	end;
run;

/* Get last disposition event on/prior to Censor Date */
proc sort data =OutLib.&OutDataName(keep = USUBJID DCUTDAYS CENSOR DIED) out=censorinfo;
	by USUBJID;
run;
proc sort data =ds;
	by USUBJID;
run;
data getDSevent;
	merge censorinfo(in=ina) ds(keep = USUBJID DSDECOD STDY DSSTDTC %if %sysfunc(findw(%upcase(&DSVARS), DSSEQ)) %then %do; DSSEQ %end;);
	by USUBJID;
	if ina;
	if STDY <= DCUTDAYS;
	rename STDY = DSSTDAYS;
run;

proc sort data=getDSevent;
	by USUBJID descending DSSTDAYS descending DSSTDTC %if %sysfunc(findw(%upcase(&DSVARS), DSSEQ)) %then %do; descending DSSEQ %end;;
run;
data getDSevent;
	set getDSevent;
	by USUBJID;
	if CENSOR = 0 and DIED = 1 then DSDECOD = "&trdeath";
	if first.USUBJID;	
	label DSDECOD = "%nrbquote(%LocaleGet(LastDispEventPriorToCensorDate_Text))";
	keep USUBJID DSDECOD DSSTDTC DSSTDAYS;
run;
* merge back to result table so it can use the last DS event;
proc sql;
	create table OutLib.&OutDataName as
	select *
	from OutLib.&OutDataName d1
	left join getDSevent d2
	on d1.USUBJID=d2.USUBJID;
quit;	


%if &syserr>4 %then %do;
   %let exiterror = 1;
   %goto exit;
%end;

%let _nCensor_=0;
* S1157779 Check If data contain any death record;
data _null_;
   set OutLib.&OutDataName(keep=CENSOR) end=_e_;
   retain _nCensor_ 0;
   if CENSOR = 0 then _nCensor_ = _nCensor_ + 1;
   if _e_ then call symput("_nCensor_",_nCensor_);
run;

%let DeathResults = 1;
%if ^&_nCensor_ %then %do;
	%let DeathResults = 0;
   %goto nodeathexit;
%end;

%Nobsvars(OutLib.&OutDataName);
%if &nobs=0 and %length(&BeginTime) %then %do;
   %put [MS_S_0081];
   %put ERROR: You must enter an earlier Begin Time.;
   %let exiterror=1;
   %goto exit;
%end;

%let odata = OutLib.&OutDataName;
%include "&MacroPath./DesignByVars.sas";
* Overall Survival Analyses based on FDA Doc;
* Applying PROC PHREG and PROC LIFETEST;
* 1. Overall Survival Analysis; 
proc phreg data=outlib.&OutDataName;
   class Trt_Num(ref="&nTrtGroup");
   model SURV%upcase(&TimeSCALE)*censor(1) = Trt_Num;
   hazardratio Trt_Num / diff=ref;
   ods output HazardRatios=Part3_1; 
run;

proc lifetest data=outlib.&OutDataName;
   time SURV%upcase(&TimeSCALE)*censor(1);
   strata Trt_Num / test=logrank diff=Control("&nTrtGroup");
   ods output Quartiles=Part2_1 HomTests=Part3_2 SurvDiff=Part3_21;
run;


proc summary data=outlib.&OutDataName completetypes noprint;
   class Trt_Num;
   class DSDECOD;
   %if %length(%trim(&DesignByVars)) %then %do;
  	class &DesignByVars;
  	%end;
  	%if %length(%trim(&DesignByVars)) %then %do;
  		types Trt_Num*DSDECOD*&DesignByVars; 
  	%end;
  	%else %do;
   	types Trt_Num*DSDECOD; 
   %end;
   output out=OS;
run;

* patch for missing;
proc sort data=OS(keep=DSDECOD) out=tmp nodupkey;
   by DSDECOD;
run;

%do i = 1 %to &nTrtGroup;
   data tmp_&i;
      set tmp;
      Trt_Num = &i;
      _FREQ_ = 0;
   run;
   
   %if &i = 1 %then %do;
      data tmp_all;
         set tmp_1;
      run;
   %end;
   %else %do;
      data tmp_all;
         set tmp_all tmp_&i;
      run;
   %end;
%end;

data OS;
   merge tmp_all OS;
   by Trt_Num DSDECOD;
run;
%if %length(%trim(&DesignByVars)) %then %do;
	* get the levels of by group variable;
	proc sort data =OS out=dbtmp (keep=&DesignByVars) nodupkey;
		by &DesignByVars ;
	run;
	data _null_;
		set dbtmp end=last;
		name = "DBLevel_" || trim(left(_n_));
		call symput(name,trim(left(&DesignByVars)));
		if last then call symput("_ndbv_",trim(left(_n_)));
   run;

	%do j=1 %to &_ndbv_ ;
		data OSD&j;
			set OS;
			if &DesignByVars ="&&&DBLevel_&j";
		run;
		data outdataSub&j;
			set outlib.&OutDataName;
			if &DesignByVars ="&&&DBLevel_&j";
		run;
	%end;
%end;

%let trAge = %nrbquote(%LocaleGet(Age_Text));
%let trAgeGroup = %nrbquote(%LocaleGet(AgeGroup_Text));
%let trConcomitantMedication = %nrbquote(%LocaleGet(ConcomitantMedication_Text));
%let trCountry = %nrbquote(%LocaleGet(Country_Text));
%let trEthnicity = %nrbquote(%LocaleGet(Ethnicity_Text));
%let trGender = %nrbquote(%LocaleGet(Gender_Text));
%let trHazardRatio = %nrbquote(%LocaleGet(HazardRatio_Text));
%let trHazardRatioforXSubgroup = %nrbquote(%LocaleGet(HazardRatioforXSubgroup_Text));
%let trHighBaselineLabTest = %nrbquote(%LocaleGet(HighBaselineLabTest_Text));
%let trLabTest = %nrbquote(%LocaleGet(LabTest_Text));
%let trLogranktest = %nrbquote(%LocaleGet(Logranktest_Text));
%let trLowBaselineLabTest = %nrbquote(%LocaleGet(LowBaselineLabTest_Text));
%let trMedicalHistory = %nrbquote(%LocaleGet(MedicalHistory_Text));
%let trpvalue = %nrbquote(%LocaleGet(pvalue_Text));
%let trRace = %nrbquote(%LocaleGet(Race_Text));
%let trStudySite = %nrbquote(%LocaleGet(StudySite_Text));
%let trStudySiteIdentifier = %nrbquote(%LocaleGet(StudySiteIdentifier_Text));
		
%let trHazardRatioforAgeSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trAge)));
%let trHazardRatioforConMedSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trConcomitantMedication)));
%let trHazardRatioforCountrySG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trCountry)));
%let trHazardRatioforEthnicitySG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trEthnicity)));
%let trHazardRatioforGenderSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trGender)));
%let trHazardRatioforHighBLTSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trHighBaselineLabTest)));
%let trHazardRatioforLowBLTSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trLowBaselineLabTest)));
%let trHazardRatioforMedHistorySG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trMedicalHistory)));
%let trHazardRatioforRaceSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trRace)));
%let trHazardRatioforStudySiteSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trStudySite)));
		
%if %length(%trim(&DesignByVars)) %then %do;
%put _ndbv_=&_ndbv_ &DBVnL1;
	%do _j_=1 %to &_ndbv_;
		data OS;
			set OSD&_j_;
		run;
		proc sort data=outdataSub&_j_(keep=&TrtName Trt_Num) out=tmpsub nodupkey;
		   by Trt_Num;
		run;
		%nobsvars(tmp);
		%let nTrtGroupSub = &nObs;
		
		data _null_;
			set tmpsub end=last;
			if last then call symputx("nTrtGroupSubL",Trt_Num);
		run;
		%put nTrtGroupSub=&nTrtGroupSub nTrtGroupSubL=&nTrtGroupSubL;
		proc phreg data=outdataSub&_j_;
		   class Trt_Num(ref="&nTrtGroupSubL");
		   model SURV%upcase(&TimeSCALE)*censor(1) = Trt_Num;
		   hazardratio Trt_Num / diff=ref;
		   ods output HazardRatios=Part3_1; 
		run;
		
		proc lifetest data=outdataSub&_j_;
		   time SURV%upcase(&TimeSCALE)*censor(1);
		   strata Trt_Num / test=logrank diff=Control("&nTrtGroupSub");
		   ods output Quartiles=Part2_1 HomTests=Part3_2 SurvDiff=Part3_21;
		run;
		data _null_;
		   set OS end=_e_;
		   by Trt_Num;
		   retain _c_ _cc_ 0;
		   _g_ = Trt_Num;
		   if first.Trt_Num then do;
		      _c_ = _FREQ_;     
		      if kupcase(DSDECOD) in (
		        %unquote(%LocaleGetAll(Death_Text, _case = u)),
		        %unquote(%LocaleGetAll(Died_Text, _case = u)),
		        %unquote(%LocaleGetAll(Dead_Text, _case = u))) then _cc_ = 0;
		      else _cc_ = _FREQ_;
		   end;
		   else do;
		      _c_ + _FREQ_;
		      if kupcase(DSDECOD) not in (
		        %unquote(%LocaleGetAll(Death_Text, _case = u)),
		        %unquote(%LocaleGetAll(Died_Text, _case = u)),
		        %unquote(%LocaleGetAll(Dead_Text, _case = u))) then _cc_ + _FREQ_;
		   end;
		   if last.Trt_Num then do;
		      name = "Sum_" || trim(left(_g_));
		      call symputx(name,_c_);
		      name = "Sum_c_" || trim(left(_g_));
		      call symputx(name,_cc_);
		      *name ="ARM_" || trim(left(_g_));
		      *call symputx(name,&TrtName);
		   end;
		   *if _e_ then call symput("nTrtGroup",_g_);
		run;
		%do _p_ =1 %to &nTrtGroup;
			%let sum_&_j_.&_p_ = &&Sum_&_p_;
		%end;
		
		* construct data with "censored" record;
		data OS_d OS_c;
		   set OS;
		   by Trt_Num;
		   if kupcase(DSDECOD) in (
		     %unquote(%LocaleGetAll(Death_Text, _case = u)),
		     %unquote(%LocaleGetAll(Died_Text, _case = u)),
		     %unquote(%LocaleGetAll(Dead_Text, _case = u))) then do;
		      output OS_d;
		   end;
		   else do;
		      output OS_c;
		   end;
		run;
		
		%nObsVars(OS_d);
		%if &nObs %then %do;
		   data OS_d;
		      set OS_d;
		      _O_ = 1;
		   run;
		%end;
		%else %do;
		   data OS_d;
		      length DSDECOD $30;
		      do Trt_Num = 1 to &nTrtGroup;
		         DSDECOD = "&trdeath";
		         _FREQ_ = 0;
		         _O_ = 1;
		         output;
		      end;
		   run;
		%end;
		
		data OS_c; 
		   set OS_c;
		   by Trt_Num;
		   retain _O_;
		   if first.Trt_Num then _O_ = 3;
		   else _O_ = _O_ + 1;
		run;

		data OS_c2;
		   set OS;
		   by Trt_Num;
		   if first.Trt_Num;
		   _g_ = Trt_Num;
		   array _C_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_c_&i %end;);
		   DSDECOD = "%kupcase(%LocaleGet(Censored_Text))"; ***************************Translation*************************;
		   _FREQ_ = _C_(_g_);
		   _O_ = 2;
		run;
		
		data OS2;
		   set OS_d OS_c2 OS_c;
		   keep Trt_Num DSDECOD _FREQ_ _O_;
		run;
		
		proc sort data=OS2;
		   by Trt_Num _O_;
		run;
		
		data OS3;
		   set OS2;
		   by Trt_Num;
		   /*make count and percent character variables so we can merge with
		    parts 2 and 3*/
		   length Count Percentage $20 ;
		
		   array _T_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_&i %end;);
		   _g_ = Trt_Num;
		   if _T_(_g_) >0 then Percentage = put(_FREQ_/_T_(_g_)*100,8.1);
		   else Percentage = put (_T_(_g_),8.1) ;
		   Count = put(_FREQ_,8.0);
		   drop _T_:;
		run;
		
		data %do i = 1 %to &nTrtGroup; 
		      OS3_&i(rename=(Count=Count_&i Percentage=Percentage_&i))
		   %end;;
		   set OS3;
		   %do i = 1 %to &nTrtGroup;
		      if _g_ = &i then output OS3_&i;
		   %end;
		   drop _g_;
		run;
		
		data Part1;
		   merge %do i = 1 %to &nTrtGroup; OS3_&i %end;;
		   by _O_;
		   part = 1;
		   length col1 $50;
		   seq = _O_;
		   col1 = DSDECOD;
		   drop i _O_;      
		run;
		
		******************** PART 2 *************************;
		proc sort data=Part2_1; 
		   by Trt_Num percent;
		run;
		
		data Part2_2;
		   set Part2_1;
		   by Trt_Num;
		   length Count Percentage $20 LCL UCL $8;
		   array _T_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_&i %end;);
		   _g_ = Trt_Num;
		   if LowerLimit=. then LCL = "NE";
		   else LCL = strip(put(LowerLimit,8.1));
		   if UpperLimit=. then UCL = "NE";
		   else UCL = strip(put(UpperLimit,8.1));
		   Percentage = '(' || trim(LCL) ||', '|| trim(UCL) ||')';
		   if estimate=. then Count = "NE";
		   else Count = trim(left(put(estimate,8.1)));
		   drop _T_:;
		run;
		
		data %do i = 1 %to &nTrtGroup; 
		      p2_&i(rename=(Count=Count_&i Percentage=Percentage_&i))
		   %end;;
		   set Part2_2;
		   %do i = 1 %to &nTrtGroup;
		      if Trt_Num = &i then output p2_&i;
		   %end;
		   drop _g_;
		run;
		
		data Part2;
		   merge  %do i = 1 %to &nTrtGroup; p2_&i %end;;
		   by percent;
		   length col1 $50;
		   part=2;
		   if percent=25 then do;
		      col1 = "%nrbquote(%LocaleGet(FirstQuartile_Text)) (95% CI)";
		      seq=1;
		   end;
		   else if percent=50 then do;
		      col1 = "%nrbquote(%LocaleGet(Median_Text)) (95% CI)";
		      seq=2;
		   end;
		   else if percent=75 then do;
		      col1 = "%nrbquote(%LocaleGet(ThirdQuartile_Text)) (95% CI)";
		      seq=3;
		   end;
		   * adjust empty p2_x dataset;
		   %do i = 1 %to &nTrtGroup;
		      if trim(left(Count_&i))="" then do;
		         Count_&i = "NE";
		         Percentage_&i = "(NE, NE)";
		      end;
		   %end;
		   drop STRATUM Trt_Num Estimate Transform LowerLimit UpperLimit LCL UCL;
		run;
		
		%if &nTrtGroup > 1 %then %do;
		   ******************** PART 3 *************************;
		   data Part3_1;
		      set part3_1 end=_e_;
		      length col1 $50 %do i = 1 %to %eval(&nTrtGroup-1); Count_&i Percentage_&i %end; $20;
		      retain %do i = 1 %to %eval(&nTrtGroup-1); Count_&i Percentage_&i HazardRatio_&i WaldLower_&i WaldUpper_&i %end;;
		      %do i = 1 %to %eval(&nTrtGroup-1);
		         if index(Description,"&i") then do;
		            Count_&i = strip(put(HazardRatio,8.3));
		            Percentage_&i = '(' || strip(put(WaldLower,8.3)) ||', '|| strip(put(WaldUpper,8.3)) ||')';
		            HazardRatio_&i = HazardRatio;
		            WaldLower_&i = WaldLower;
		            WaldUpper_&i = WaldUpper;
		         end;
		      %end;
		      * handling empty case;
		      %do i = 1 %to %eval(&nTrtGroup-1);
		         if trim(left(Count_&i)) = "" then do;
		            Count_&i = "NE";
		            Percentage_&i = "(NE, NE)";
		            HazardRatio_&i = .;
		            WaldLower_&i = .;
		            WaldUpper_&i = .;
		         end;
		      %end;
		      if _e_;
		      col1 = "&trHazardRatio (95% CI)";
		      part = 3;
		      seq = 1;
		      drop Description HazardRatio WaldLower WaldUpper;
		   run;
		
		   %if &nTrtGroup > 2 %then %do;
		      data Part3_21;
		         set part3_21 end=_e_;
		         length col1 $50 %do i = 1 %to %eval(&nTrtGroup-1); Count_&i %end; $20;
		         retain %do i = 1 %to %eval(&nTrtGroup-1); Count_&i %end;;
		         %do i = 1 %to %eval(&nTrtGroup-1);
		            if Stratum1 = &i then do;
		               Count_&i = strip(put(Dunnett,8.3));
		            end;
		         %end;
		         %do i = 1 %to %eval(&nTrtGroup-1);
		            if trim(left(Count_&i)) = "" then Count_&i = "NE";
		         %end;
		         if _e_;
		         col1 = "&trLogranktest (%nrbquote(%LocaleFill(%LocaleGet(pvaluecomparedtoX_Text), %nrbquote(%substr(%nrbquote(&ScreenControlValue),2,%length(&ScreenControlValue)-2)))))";
		         part = 3;
		         seq = 2;
		      run;
		      
		      data Part3_2;
		         set Part3_2;
		         length col1 $50 Count_1 $20;
		         col1 = "&trLogranktest (&trpvalue)";
		         part = 3;
		         seq = 3;
		         Count_1 = strip(put(ProbChiSq,8.3));
		      run;
		      
		      data Part3_2;
		         set Part3_21 Part3_2;
		      run;
		   %end;
		   %else %do;  
		      data Part3_2;
		         set Part3_2;
		         length col1 $50 Count_1 $20;
		         col1 = "&trLogranktest (&trpvalue)";
		         part = 3;
		         seq = 2;
		         Count_1 = strip(put(ProbChiSq,8.3));
		      run;
		   %end;
		
		   data part3;
		      set Part3_1(in=in1) Part3_2(in=in2);
		      keep col1 Count_: Percentage_: part seq;
		      drop Description HazardRatio: WaldLower: WaldUpper:;
		   run;
		
		%end;
		%else %do;
		   * one ARM level;
		   %let _DM_Flag_ = 0;
		   %let _LB_Flag_ = 0;
		   %let _CM_Flag_ = 0;
		   %let _MH_Flag_ = 0;
		%end;
		
		%let MKM_OutData_A1&_j_ = ;
		%let MKM_OutData_A1 = ;
		%PrepOutDS(MKM_OutData_A1&_j_,OutDataName_A1&_j_,overallsurvival,_a1&_j_);
		%PrepOutDS(MKM_OutData_A1,OutDataName_A1,overallsurvival,_a1);
		
		data &&MKM_OutData_A1&_j_ %if &_j_ =1 %then &MKM_OutData_A1;;
		   set part1 part2 %if &nTrtGroup > 1 %then %do; part3 %end;;
		run;
		%if &_j_ >1 %then %do;
			data &MKM_OutData_A1;
				set &MKM_OutData_A1 &&MKM_OutData_A1&_j_;
			run;
		%end;
	%end;
%end;
%else %do;
	data _null_;
   	set OS end=_e_;
   	by Trt_Num;
   	retain _c_ _cc_ 0;
   	_g_ = Trt_Num;
   	if first.Trt_Num then do;
   	   _c_ = _FREQ_;     
   	   if kupcase(DSDECOD) in (
   	         %unquote(%LocaleGetAll(Death_Text, _case = u)),
   	         %unquote(%LocaleGetAll(Died_Text, _case = u)),
   	         %unquote(%LocaleGetAll(Dead_Text, _case = u))
   	      ) then _cc_ = 0;
   	   else _cc_ = _FREQ_;
   	end;
   	else do;
   	   _c_ + _FREQ_;
   	   if kupcase(DSDECOD) not in (
   	         %unquote(%LocaleGetAll(Death_Text, _case = u)),
   	         %unquote(%LocaleGetAll(Died_Text, _case = u)),
   	         %unquote(%LocaleGetAll(Dead_Text, _case = u))   	         
   	      ) then _cc_ + _FREQ_;
   	end;
   	if last.Trt_Num then do;
   	   name = "Sum_" || trim(left(_g_));
   	   call symputx(name,_c_);
   	   name = "Sum_c_" || trim(left(_g_));
   	   call symputx(name,_cc_);
   	   *name ="ARM_" || trim(left(_g_));
   	   *call symputx(name,&TrtName);
   	end;
   	*if _e_ then call symput("nTrtGroup",_g_);
	run;

	* construct data with "censored" record;
	data OS_d OS_c;
	   set OS;
	   by Trt_Num;
	   if kupcase(DSDECOD) in (
	      %unquote(%LocaleGetAll(Death_Text, _case = u)),
   	      %unquote(%LocaleGetAll(Died_Text, _case = u)),
   	      %unquote(%LocaleGetAll(Dead_Text, _case = u))
   	   ) then do;
	      output OS_d;
	   end;
	   else do;
	      output OS_c;
	   end;
	run;
	
	%nObsVars(OS_d);
	%if &nObs %then %do;
	   data OS_d;
	      set OS_d;
	      _O_ = 1;
	   run;
	%end;
	%else %do;
	   data OS_d;
	      length DSDECOD $30;
	      do Trt_Num = 1 to &nTrtGroup;
	         DSDECOD = "&trdeath";
	         _FREQ_ = 0;
	         _O_ = 1;
	         output;
	      end;
	   run;
	%end;

	data OS_c; 
	   set OS_c;
	   by Trt_Num;
	   retain _O_;
	   if first.Trt_Num then _O_ = 3;
	   else _O_ = _O_ + 1;
	run;
	
	data OS_c2;
	   set OS;
	   by Trt_Num;
	   if first.Trt_Num;
	   _g_ = Trt_Num;
	   array _C_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_c_&i %end;);
	   DSDECOD = "%kupcase(%LocaleGet(Censored_Text))"; ***************************Translation*************************;
	   _FREQ_ = _C_(_g_);
	   _O_ = 2;
	run;

	data OS2;
	   set OS_d OS_c2 OS_c;
	   keep Trt_Num DSDECOD _FREQ_ _O_;
	run;

	proc sort data=OS2;
	   by Trt_Num _O_;
	run;

	data OS3;
	   set OS2;
	   by Trt_Num;
	   /*make count and percent character variables so we can merge with
	    parts 2 and 3*/
	   length Count Percentage $20 ;
	
	   array _T_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_&i %end;);
	   _g_ = Trt_Num;
	   Percentage = put(_FREQ_/_T_(_g_)*100,8.1);
	   Count = put(_FREQ_,8.0);
	   drop _T_:;
	run;
	
	data %do i = 1 %to &nTrtGroup; 
	      OS3_&i(rename=(Count=Count_&i Percentage=Percentage_&i))
	   %end;;
	   set OS3;
	   %do i = 1 %to &nTrtGroup;
	      if _g_ = &i then output OS3_&i;
	   %end;
	   drop _g_;
	run;
	
	data Part1;
	   merge %do i = 1 %to &nTrtGroup; OS3_&i %end;;
	   by _O_;
	   part = 1;
	   length col1 $50;
	   seq = _O_;
	   col1 = DSDECOD;
	   drop i _O_;      
	run;
	
	******************** PART 2 *************************;
	proc sort data=Part2_1; 
	   by Trt_Num percent;
	run;
	
	data Part2_2;
	   set Part2_1;
	   by Trt_Num;
	   length Count Percentage $20 LCL UCL $8;
	   array _T_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_&i %end;);
	   _g_ = Trt_Num;
	   if LowerLimit=. then LCL = "NE";
	   else LCL = strip(put(LowerLimit,8.1));
	   if UpperLimit=. then UCL = "NE";
	   else UCL = strip(put(UpperLimit,8.1));
	   Percentage = '(' || trim(LCL) ||', '|| trim(UCL) ||')';
	   if estimate=. then Count = "NE";
	   else Count = trim(left(put(estimate,8.1)));
	   drop _T_:;
	run;
	
	data %do i = 1 %to &nTrtGroup; 
	      p2_&i(rename=(Count=Count_&i Percentage=Percentage_&i))
	   %end;;
	   set Part2_2;
	   %do i = 1 %to &nTrtGroup;
	      if Trt_Num = &i then output p2_&i;
	   %end;
	   drop _g_;
	run;
	
	data Part2;
	   merge  %do i = 1 %to &nTrtGroup; p2_&i %end;;
	   by percent;
	   length col1 $50;
	   part=2;
	   if percent=25 then do;
	      col1 = "%nrbquote(%LocaleGet(FirstQuartile_Text)) (95% CI)";
	      seq=1;
	   end;
	   else if percent=50 then do;
	      col1 = "%nrbquote(%LocaleGet(Median_Text)) (95% CI)";
	      seq=2;
	   end;
	   else if percent=75 then do;
	      col1 = "%nrbquote(%LocaleGet(ThirdQuartile_Text)) (95% CI)";
	      seq=3;
	   end;
	   * adjust empty p2_x dataset;
	   %do i = 1 %to &nTrtGroup;
	      if trim(left(Count_&i))="" then do;
	         Count_&i = "NE";
	         Percentage_&i = "(NE, NE)";
	      end;
	   %end;
	   drop STRATUM Trt_Num Estimate Transform LowerLimit UpperLimit LCL UCL;
	run;

	%if &nTrtGroup > 1 %then %do;
	   ******************** PART 3 *************************;
	   data Part3_1;
	      set part3_1 end=_e_;
	      length col1 $50 %do i = 1 %to %eval(&nTrtGroup-1); Count_&i Percentage_&i %end; $20;
	      retain %do i = 1 %to %eval(&nTrtGroup-1); Count_&i Percentage_&i HazardRatio_&i WaldLower_&i WaldUpper_&i %end;;
	      %do i = 1 %to %eval(&nTrtGroup-1);
	         if index(Description,"&i") then do;
	            Count_&i = strip(put(HazardRatio,8.3));
	            Percentage_&i = '(' || strip(put(WaldLower,8.3)) ||', '|| strip(put(WaldUpper,8.3)) ||')';
	            HazardRatio_&i = HazardRatio;
	            WaldLower_&i = WaldLower;
	            WaldUpper_&i = WaldUpper;
	         end;
	      %end;
	      * handling empty case;
	      %do i = 1 %to %eval(&nTrtGroup-1);
	         if trim(left(Count_&i)) = "" then do;
	            Count_&i = "NE";
	            Percentage_&i = "(NE, NE)";
	            HazardRatio_&i = .;
	            WaldLower_&i = .;
	            WaldUpper_&i = .;
	         end;
	      %end;
	      if _e_;
	      col1 = "&trHazardRatio (95% CI)";
	      part = 3;
	      seq = 1;
	      drop Description HazardRatio WaldLower WaldUpper;
	   run;
	
	   %if &nTrtGroup > 2 %then %do;
	      data Part3_21;
	         set part3_21 end=_e_;
	         length col1 $50 %do i = 1 %to %eval(&nTrtGroup-1); Count_&i %end; $20;
	         retain %do i = 1 %to %eval(&nTrtGroup-1); Count_&i %end;;
	         %do i = 1 %to %eval(&nTrtGroup-1);
	            if Stratum1 = &i then do;
	               Count_&i = strip(put(Dunnett,8.3));
	            end;
	         %end;
	         %do i = 1 %to %eval(&nTrtGroup-1);
	            if trim(left(Count_&i)) = "" then Count_&i = "NE";
	         %end;
	         if _e_;
	         col1 = "&trLogranktest (%nrbquote(%LocaleFill(%LocaleGet(pvaluecomparedtoX_Text), %nrbquote(%substr(%nrbquote(&ScreenControlValue),2,%length(&ScreenControlValue)-2)))))";
	         part = 3;
	         seq = 2;
	      run;
	      
	      data Part3_2;
	         set Part3_2;
	         length col1 $50 Count_1 $20;
	         col1 = "&trLogranktest (&trpvalue)";
	         part = 3;
	         seq = 3;
	         Count_1 = strip(put(ProbChiSq,8.3));
	      run;
	      
	      data Part3_2;
	         set Part3_21 Part3_2;
	      run;
	   %end;
	   %else %do;  
	      data Part3_2;
	         set Part3_2;
	         length col1 $50 Count_1 $20;
	         col1 = "&trLogranktest (&trpvalue)";
	         part = 3;
	         seq = 2;
	         Count_1 = strip(put(ProbChiSq,8.3));
	      run;
	   %end;
	
	   data part3;
	      set Part3_1(in=in1) Part3_2(in=in2);
	      keep col1 Count_: Percentage_: part seq;
	      drop Description HazardRatio: WaldLower: WaldUpper:;
	   run;
	
	%end;
	%else %do;
	   * one ARM level;
	   %let _DM_Flag_ = 0;
	   %let _LB_Flag_ = 0;
	   %let _CM_Flag_ = 0;
	   %let _MH_Flag_ = 0;
	%end;

	%let MKM_OutData_A1 = ;
	%PrepOutDS(MKM_OutData_A1,OutDataName_A1,overallsurvival,_a1);
	
	data &MKM_OutData_A1;
	   set part1 part2 %if &nTrtGroup > 1 %then %do; part3 %end;;
	run;
%end;

%let _OutputDatasets_ = &OutDataName..sas7bdat,&OutDataName_A1..sas7bdat;

%macro Table_A1;

%let trAnalysisX = %nrbquote(%LocaleGet(AnalysisX_Text));
%let trCount = %nrbquote(%LocaleGet(Count_Text));
%let trOverallSurvival = %nrbquote(%LocaleGet(OverallSurvival_Text));

%if %upcase(&SplitResults)=YES  %then %do;
	%do _j_=1 %to &_ndbv_;
		%let tmpbyvar = %scan(%upcase(&DBVList1),&_j_," ");
		proc report data = outlib.&&OutDataName_A1&_j_ LS=97  PS=54  SPLIT="#" CENTER nowindows missing;
      	column part seq col1 ( %do i=1 %to &nTrtGroup; ("%nrbquote(&&ARM_&i)" "N=&&&sum_&_j_.&i" (Count_&i Percentage_&i) ) %end; );
      	define part /order noprint;
      	define seq /order noprint;
      	define col1 / display "&trOverallSurvival" FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data style(column)=[width=2.5in]; 
            
      	%do i=1 %to &nTrtGroup;
      	   define Count_&i / display "&trCount" style(column)=[WIDTH=0.9in just=r];
      	   define Percentage_&i / display "%"  style(column)=[WIDTH=0.9in just=r];
      	%end;

      	compute col1;
      	   if (holdpart=1 and seq > 2) or (holdpart>1) then do;
      	      call define(_col_,'style','style=[asis=on pretext="  "]');
      	   end;
      	endcomp;

      	compute before part /style=[just=l background=gray];
      	   LENGTH text $ 50;                                                                
         IF part=1 THEN DO;                                                               
            text = '';                                                     
            num=0;                                                                       
         END; 
         ELSE IF part=2 THEN DO;                                                               
            text = "%nrbquote(%LocaleGet(KaplanMeierestimateofoverallsurvival_Text))";                                                     
            num=50;                                                                       
         END; 
         ELSE IF part=3 THEN DO;                                                               
            text = "%nrbquote(%LocaleGet(UnstratifiedAnalysis_Text))";                                                     
            num=50;                                                                       
         END;  
         ELSE DO;                                                                         
            text = "";                                                                    
            num=0;                                                                        
         END;                                                                             
         LINE text $VARYING. num ;       

         if part ^=. then holdpart=part;
 
      	ENDCOMP;                           

      	title1 height=17pt "%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(AnalysisX_Text)), 1)): %nrbquote(%LocaleGet(OverallSurvivalAnalysis_Text)) &dbv1=&&&DBLevel_&_j_: Censor Date Determined using &CensorDateVar";
		run;
	%end;
%end;
%else %do;
proc report data = outlib.&OutDataName_A1 LS=97  PS=54  SPLIT="#" CENTER nowindows missing;
      column part seq col1 ( %do i=1 %to &nTrtGroup; ("%nrbquote(&&ARM_&i)" "N=&&Sum_&i" (Count_&i Percentage_&i) ) %end; );
      define part /order noprint;
      define seq /order noprint;
      define col1 / display "&trOverallSurvival" FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data style(column)=[width=2.5in]; 
            
      %do i=1 %to &nTrtGroup;
         define Count_&i / display "&trCount" style(column)=[WIDTH=0.9in just=r];
         define Percentage_&i / display "%"  style(column)=[WIDTH=0.9in just=r];
      %end;

      compute col1;
         if (holdpart=1 and seq > 2) or (holdpart>1) then do;
            call define(_col_,'style','style=[asis=on pretext="  "]');
         end;
      endcomp;

      compute before part /style=[just=l background=gray];
         LENGTH text $ 50;                                                                
         IF part=1 THEN DO;                                                               
            text = '';                                                     
            num=0;                                                                       
         END; 
         ELSE IF part=2 THEN DO;                                                               
            text = "%nrbquote(%LocaleGet(KaplanMeierestimateofoverallsurvival_Text))";                                                     
            num=50;                                                                       
         END; 
         ELSE IF part=3 THEN DO;                                                               
            text = "%nrbquote(%LocaleGet(UnstratifiedAnalysis_Text))";                                                     
            num=50;                                                                       
         END;  
         ELSE DO;                                                                         
            text = "";                                                                    
            num=0;                                                                        
         END;                                                                             
         LINE text $VARYING. num ;       

         if part ^=. then holdpart=part;
 
      ENDCOMP;                           

      title1 height=17pt "%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(AnalysisX_Text)), 1)): %nrbquote(%LocaleGet(OverallSurvivalAnalysis_Text)): %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(CensorDateDeterminedUsingX_Text)), &CensorDateVar))";
run;
%end;

%mend Table_A1;

/* generate tables;
options orientation=landscape;
ods rtf file="%sysfunc(pathname(Outlib))&_slash_.OverallSurvivalTables_&_nThePro_..rtf";
%Table_A1;
ods rtf close;

%let exiterror = 1;
%goto exit;
*/

* macros for sub-analysis;
%macro DM_SG_HR(_InDt,_gVar,_OutDt);
   %put Process for &_gVar in dataset &_InDt..;
   %let gVarType = ;
   %global HR_NoGo;
   %let HR_NoGo = 0;
   %VarAttributes(&_InDt,&_gVar,_Type=gVarType);
   %if &gVarType = N %then %let wStatement = %str(Where=( (&_gVar ^= .) & (SURV%upcase(&TimeSCALE) ^= .) ));
   %else %let wStatement = %str(Where=( (&_gVar ^= " ") & (SURV%upcase(&TimeSCALE) ^= .) ));

   * generate data for subgroup table;
   proc sort data=&_InDt(&wStatement) out=DM_SG;
      by &_gVar;
   run;

   * screen out data without reference level within each by-var to avoid ERROR;
   proc sort data=DM_SG(where=(&TrtName=&ScreenControlValue)) out=tmp(keep=&_gVar) nodupkey;
      by &_gVar;
   run;
   
   data DM_SG;
      if 0 then set tmp;
      if _n_ = 1 then do;
         declare hash h(dataset: "work.tmp", hashexp: 6);
         h.defineKey("&_gVar");
         h.defineDone();
      end;
      set DM_SG;
      if h.find() = 0;
   run;
   
   
   
   *S1393880 avoid processing single treatment for a subgroup;
   proc sort data=DM_SG out=tmp nodupkey;
   	by &TrtName;
   run;
   
   %let _ntrt_ = 0;
   data _null_;
   	set tmp end=_e_;
   	if _e_ then call symput("_ntrt_",_n_);
   run;
   %if &_ntrt_ < 2 %then %do;
      %let HR_NoGo = 1;
      %goto exit_DM_SG_HR;
   %end;
   
   
   
   
   
   * screen out data with only control level;
   data DM_SG;
      set DM_SG;
      retain _c_ 0;
      by &_gVar;
      if first.&_gVar then _c_ = 1;
      else _c_+1;
      if last.&_gVar then do;
         if _c_ = 1 then delete;
      end;
   run;
   
   %nObsVars(DM_SG);
   %if ^&nObs %then %do;
      %let HR_NoGo = 1;
      %goto exit_DM_SG_HR;
   %end;
   
   proc phreg data=DM_SG ;
      class Trt_Num(ref="&nTrtGroup");
      by &_gVar;
      model SURV%upcase(&TimeSCALE)*CENSOR(1) = Trt_Num;
      hazardratio Trt_Num / diff=ref;
      ods output HazardRatios=HR_12; 
   run;

   proc summary data=DM_SG completetypes noprint;
      *format DSDECOD $code.;
      class Trt_Num &_gVar CENSOR;
      types Trt_Num*&_gVar*CENSOR; 
      output out=HR_11;
   run;

   data %do i = 1 %to &nTrtGroup;
         DM_HR_1_&i
      %end;;
      set HR_11;
      %do i = 1 %to &nTrtGroup;
         if Trt_Num = &i then output DM_HR_1_&i;
      %end;
   run;

   %do i = 1 %to &nTrtGroup;
      %let j = %eval(&i*2);
      %let k = %eval(&j-1);
      data tmp1;
         set DM_HR_1_&i;
         if CENSOR = 0;
         rename _FREQ_ = Count_&k;
         drop Censor _TYPE_;
      run;
      proc summary data=DM_HR_1_&i completetypes noprint;
         class &_gVar;
         var _FREQ_;
         types &_gVar;
         output out=tmp2(drop=_TYPE_ _FREQ_) sum=Count_&j;
      run;
      data tmp3;
         merge tmp1 tmp2;
         by &_gVar;
      run;
      %if &i = 1 %then %do;
         data DM_HR_1;
            set tmp3;
            if Count_&j = . then Count_&j = 0;
            if Count_&k = . then Count_&k = 0;
         run;
      %end;
      %else %do;
         data DM_HR_1;
            merge DM_HR_1 tmp3;
            by &_gVar;
            if Count_&j = . then Count_&j = 0;
            if Count_&k = . then Count_&k = 0;
         run;
      %end;
   %end;
   
   data %do i = 1 %to %eval(&nTrtGroup-1); 
            HR_12_&i (rename=(HazardRatio=HazardRatio_&i WaldLower=WaldLower_&i WaldUpper=WaldUpper_&i))
        %end;;
      set HR_12;
      %do i = 1 %to %eval(&nTrtGroup-1); 
         if description = "Trt_Num &i vs &nTrtGroup" then output HR_12_&i;
      %end;
   run;
   
   data HR_12_All;
      merge %do i = 1 %to %eval(&nTrtGroup-1); HR_12_&i %end;;
      by &_gVar;
   run;

   %if %upcase(&_gVar) = NEWAGEGROUP %then %let _Sec_ = 1;
   %else %if %upcase(&_gVar) = SEX %then %let _Sec_ = 2;
   %else %if %upcase(&_gVar) = RACE %then %let _Sec_ = 3;
   %else %if %upcase(&_gVar) = ETHNIC %then %let _Sec_ = 4;
   %else %if %upcase(&_gVar) = COUNTRY %then %let _Sec_ = 5;
   %else %if %upcase(&_gVar) = SITEID %then %let _Sec_ = 6;
   %else %let _sec_ = 1; * one section case;
   
   data &_OutDt;
      length DM_Subgroup $32 %do i = 1 %to %eval(&nTrtGroup-1); HR_&i HR_LCL_&i HR_UCL_&i %end; $10;
      merge DM_HR_1 HR_12_All;
      by &_gVar;
      %if %upcase(&_gVar) = NEWAGEGROUP %then %do;
         %do i = 1 %to 5;
            if NewAgeGroup = &i then DM_Subgroup = "&&AgeGroup&i";
         %end;
      %end;
      %else %do;
         DM_Subgroup = trim(left(&_gVar));
      %end;
      _seq_ = _n_;
      _sec_ = &_Sec_;
      %let l = %eval(&nTrtGroup*2-1);
      %do i = 1 %to %eval(&nTrtGroup-1);
         %let k = %eval(&i*2-1);
         if (Count_&k = 0) | (Count_&k = .) | (Count_&l = 0) | (Count_&l = .) then do;
            HR_&i = "NE";
            HR_LCL_&i = "(NE,";
            HR_UCL_&i = "NE)";
         end;
         else do;
            if HazardRatio_&i = . then HR_&i = "NE";
            else HR_&i = strip(put(HazardRatio_&i,8.2));
            if WaldLower_&i = .  then HR_LCL_&i = "(NE,";
            else HR_LCL_&i = "(" || strip(put(WaldLower_&i,8.2)) || ",";
            if WaldUpper_&i = . then HR_UCL_&i = "NE)";
            else HR_UCL_&i = strip(put(WaldUpper_&i,8.2)) || ")";
         end;
         HR_CL_&i = trim(left(HR_LCL_&i)) || " " || trim(left(HR_UCL_&i));
      %end;
      drop Trt_Num Description &_gVar;
      *label HazardRatio="&trHazardRatio" WaldLower="Lower CI" WaldUpper="Upper CI";
   run;
   %exit_DM_SG_HR:; 
   %put HR_NoGo = &HR_NoGo;
%mend DM_SG_HR;

%macro Check_Empty_Var(_InDs,_Var);
   %let exiterror = 1;
   data _null_;
      set &_InDs(keep=&_Var);
      if length(trim(left(&_Var))) then do;
         call symputx('exiterror',0);
         stop;
      end;
   run; 
%mend Check_Empty_Var;

%macro Case_Missing(_OutDs,_sec);
   * handling missing case for a subgroup;
   data _null_;
      set outlib.&OutDataName_A1;
      if _n_ = 1;
      %do i = 1 %to &nTrtGroup;
         call symput("E_Count_&i",Count_&i+0);
      %end;
   run;
   data &_OutDs;
      set Part3_1(drop=Count_: Percentage_:);
      length %do i = 1 %to %eval(&nTrtGroup-1); HR_&i HR_LCL_&i HR_UCL_&i %end; $10;
      DM_Subgroup = "%nrbquote(%LocaleGet(Missing_Text))";
      _seq_ = 1;
      _sec_ = &_sec;
      %do i = 1 %to &nTrtGroup;
         %let j = %eval(&i*2);
         %let k = %eval(&j-1);
         Count_&k = &&E_Count_&i + 0;
         Count_&j = &&Sum_&i + 0;
      %end;    
      
      %let l = %eval(&nTrtGroup*2-1);
      %do i = 1 %to %eval(&nTrtGroup-1);
         %let k = %eval(&i*2-1);
         if (Count_&k = 0) | (Count_&k = .) | (Count_&l = 0) | (Count_&l = .) then do;
            HR_&i = "NE";
            HR_LCL_&i = "(NE,";
            HR_UCL_&i = "NE)";
         end;
         else do;
            if HazardRatio_&i = . then HR_&i = "NE";
            else HR_&i = strip(put(HazardRatio_&i,8.2));
            if WaldLower_&i = .  then HR_LCL_&i = "(NE,";
            else HR_LCL_&i = "(" || strip(put(WaldLower_&i,8.2)) || ",";
            if WaldUpper_&i = . then HR_UCL_&i = "NE)";
            else HR_UCL_&i = strip(put(WaldUpper_&i,8.2)) || ")";
         end;
         HR_CL_&i = trim(left(HR_LCL_&i)) || " " || trim(left(HR_UCL_&i));
      %end;
      *label HazardRatio="&trHazardRatio" WaldLower="Lower CI" WaldUpper="Upper CI";
      drop Description;
   run;
%mend;


%macro Table_A2(_ANum,_Ds);
   
   %let trAbnormalHighLabTests = %nrbquote(%LocaleGet(AbnormalHighLabTests_Text));
   %let trAbnormalLowLabTests = %nrbquote(%LocaleGet(AbnormalLowLabTests_Text));
   %let trAgeGroup = %nrbquote(%LocaleGet(AgeGroup_Text));
   %let trAnalysisX = %nrbquote(%LocaleGet(AnalysisX_Text));
   %let trConcomitantMedications = %nrbquote(%LocaleGet(ConcomitantMedications_Text));
   %let trConfidenceInterval = %nrbquote(%LocaleGet(ConfidenceInterval_Text));
   %let trCountry = %nrbquote(%LocaleGet(Country_Text));
   %let trDemographics = %nrbquote(%LocaleGet(Demographics_Text));
   %let trDemographicSubgroup = %nrbquote(%LocaleGet(DemographicSubgroup_Text));
   %let trEthnicity = %nrbquote(%LocaleGet(Ethnicity_Text));
   %let trEventCount = %nrbquote(%LocaleGet(EventCount_Text));
   %let trHazardRatio = %nrbquote(%LocaleGet(HazardRatio_Text));
   %let trHazardRatioEstsforX = %nrbquote(%LocaleGet(HazardRatioEstimatesforX_Text));
   %let trHazardRatioEstsforXatBL = %nrbquote(%LocaleGet(HazardRatioEstimatesforXatBaseline_Text));
   %let trLabTest = %nrbquote(%LocaleGet(LabTest_Text));
   %let trMedicationsHistory = %nrbquote(%LocaleGet(MedicationsHistory_Text));
   %let trRace = %nrbquote(%LocaleGet(Race_Text));
   %let trSex = %nrbquote(%LocaleGet(Sex_Text));
   %let trStudySiteIdentifier = %nrbquote(%LocaleGet(StudySiteIdentifier_Text));
   %let trSubgroupCount = %nrbquote(%LocaleGet(SubgroupCount_Text));
   %let trSubgroupofX = %nrbquote(%LocaleGet(SubgroupofX_Text));

   * generate reault table;
proc report data = &_Ds LS=97  PS=54  SPLIT="#" CENTER nowindows missing;
      column _sec_ _seq_ DM_Subgroup ( 
         %do i=1 %to &nTrtGroup; 
            %let j = %eval(2*&i);
            %let k = %eval(&j-1);
            ( "%nrbquote(&&ARM_&i)" "N=&&Sum_&i" (Count_&k Count_&j) ) 
         %end; 
         /*
         %do i=1 %to %eval(&nTrtGroup-1);
            ("&trHazardRatio (%trim(&&ARM_&i) vs %trim(&&ARM_&nTrtGroup))" ( HR_&i ("Confidence Interval" (HR_LCL_&i HR_UCL_&i) ))) 
         %end;
         */
         %do i=1 %to %eval(&nTrtGroup-1);
            ("%trim(%nrbquote(&&ARM_&i)) vs %trim(%nrbquote(&&ARM_&nTrtGroup))" "&trHazardRatio" ( HR_&i HR_CL_&i) ) 
         %end;
      );
      define _sec_ /order noprint;
      define _seq_ /order noprint;
      %if &_ANum = 2 %then %do;
         define DM_Subgroup / display "&trDemographicSubgroup" FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data style(column)=[width=1.5in]; 
      %end;
      %if &_ANum = 3 | &_ANum = 4 %then %do;
         define DM_Subgroup / display "&trLabTest" FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data style(column)=[width=1.5in]; 
      %end;
      %if &_ANum = 5 %then %do;
         define DM_Subgroup / display "&_CM_Label" FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data style(column)=[width=1.5in]; 
      %end;
      %if &_ANum = 6 %then %do;
         define DM_Subgroup / display "&_MH_Label" FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data style(column)=[width=1.5in]; 
      %end;
      
      %do i=1 %to &nTrtGroup;
         %let j = %eval(2*&i);
         %let k = %eval(&j-1);
         define Count_&k / display "&trEventCount" style(column)=[WIDTH=0.5in just=r];
         define Count_&j / display "&trSubgroupCount"  style(column)=[WIDTH=0.5in just=r];
      %end;

      %do i=1 %to %eval(&nTrtGroup-1);
         define HR_&i / display "&trHazardRatio" style(column)=[WIDTH=0.6in just=r];
         /*
         define HR_LCL_&i / display "Lower CL" style(column)=[WIDTH=0.5in just=r];
         define HR_UCL_&i / display "Upper CL" style(column)=[WIDTH=0.5in just=l];
         */
         define HR_CL_&i / display "&trConfidenceInterval" style(column)=[WIDTH=0.9in just=c];
      %end;
/*
      compute col1;
         if (holdpart=1 and seq > 2) or (holdpart>1) then do;
            call define(_col_,'style','style=[asis=on pretext="  "]');
         end;
      endcomp;
*/
      %if &_ANum = 2 %then %do;
      compute before _sec_ /style=[just=l background=gray];
         LENGTH text $ 50;                                                                
         IF _sec_=1 THEN DO;                                                               
            text = "&trAgeGroup";                                                                 
         END; 
         ELSE IF _sec_=2 THEN DO;                                                               
            text = "&trSex";                                                                 
         END; 
         ELSE IF _sec_=3 THEN DO;                                                               
            text = "&trRace";                                                                
         END;  
         ELSE IF _sec_=4 THEN DO;                                                               
            text = "&trEthnicity";                                                                
         END;  
         ELSE IF _sec_=5 THEN DO;                                                               
            text = "&trCountry";                                                                
         END;  
         ELSE IF _sec_=6 THEN DO;                                                               
            text = "&trStudySiteIdentifier";                                                                
         END;                                                                            
         LINE text $50. ;        
      endcomp; 
      %end;
 
      %if &_ANum = 2 %then %do;
         title1 height=17pt "%LocaleFill(%nrbquote(&trAnalysisX), &_BNum): %LocaleFill(%nrbquote(&trHazardRatioEstsforX), %LocaleFill(%nrbquote(&trSubgroupofX), %nrbquote(&trDemographics)))";
      %end;
      %if &_ANum = 3 %then %do;
   title1 height=17pt "%LocaleFill(%nrbquote(&trAnalysisX), &_BNum): %LocaleFill(%nrbquote(&trHazardRatioEstsforXatBL), %LocaleFill(%nrbquote(&trSubgroupofX), %nrbquote(&trAbnormalHighLabTests)))";
      %end;
      %if &_ANum = 4 %then %do;
   title1 height=17pt "%LocaleFill(%nrbquote(&trAnalysisX), &_BNum): %LocaleFill(%nrbquote(&trHazardRatioEstsforXatBL), %LocaleFill(%nrbquote(&trSubgroupofX), %nrbquote(&trAbnormalLowLabTests)))";
      %end;
      %if &_ANum = 5 %then %do;
   title1 height=17pt "%LocaleFill(%nrbquote(&trAnalysisX), &_BNum): %LocaleFill(%nrbquote(&trHazardRatioEstsforXatBL), %LocaleFill(%nrbquote(&trSubgroupofX), %nrbquote(&trConcomitantMedications)))";
      %end;
      %if &_ANum = 6 %then %do;
   title1 height=17pt "%LocaleFill(%nrbquote(&trAnalysisX), &_BNum): %LocaleFill(%nrbquote(&trHazardRatioEstsforXatBL), %LocaleFill(%nrbquote(&trSubgroupofX), %nrbquote(&trMedicationsHistory)))";
      %end;
run;
%mend Table_A2;




*** Analysis 2: Hazard Ratio Estimates for Subgroup of Demographics;
%if &_DM_Flag_ %then %do;
   * 1. Age Group;
   %if %upcase(&TrtName) ne AGE %then %do;
      %DM_SG_HR(outlib.&OutDataName,NewAgeGroup,DM_HR_Age);
      %let HR_NoGo_A2_1 = &HR_NoGo;
   %end;
   %else %do;
      %let HR_NoGo_A2_1 = 1;
   %end;
   
   * 2. Gender;
   %if %upcase(&TrtName) ne SEX %then %do;
      %DM_SG_HR(outlib.&OutDataName,SEX,DM_HR_Sex);
      %let HR_NoGo_A2_2 = &HR_NoGo;
   %end;
   %else %do;
      %let HR_NoGo_A2_2 = 1;
   %end;
   
   * 3. Race; 
   %if %upcase(&TrtName) ne RACE %then %do;
      %if ^%sysfunc(findw(%upcase(&DMVARS), RACE)) %then %do;
         * no RACE variable;
         %Case_Missing(DM_HR_Race,3);
         %let HR_NoGo_A2_3 = 0;
      %end;
      %else %do;
         %Check_Empty_Var(outlib.&OutDataName,RACE);
         %if &exiterror %then %do; 
            * empty RACE variable;
            %let exiterror = 0;
            %Case_Missing(DM_HR_Race,3);
         %let HR_NoGo_A2_3 = 0;
         %end;
         %else %do; 
            %DM_SG_HR(outlib.&OutDataName,RACE,DM_HR_Race);
            %let HR_NoGo_A2_3 = &HR_NoGo;
         %end;
      %end;
   %end;
   %else %do;
      %let HR_NoGo_A2_3 = 1;
   %end;
   
   * 4. Ethnic;
   %if %upcase(&TrtName) ne ETHNIC %then %do;
      %if ^%sysfunc(findw(%upcase(&DMVARS), ETHNIC)) %then %do;
         * no Ethnic variable;
         %Case_Missing(DM_HR_Ethnic,4);
         %let HR_NoGo_A2_4 = 0;
      %end;
      %else %do;
         %Check_Empty_Var(outlib.&OutDataName,ETHNIC);
         %if &exiterror %then %do;
            * empty ETHNIC variable;
            %let exiterror = 0;
            %Case_Missing(DM_HR_Ethnic,4);
            %let HR_NoGo_A2_4 = 0;
         %end;
         %else %do;
            %DM_SG_HR(outlib.&OutDataName,Ethnic,DM_HR_Ethnic);
            %let HR_NoGo_A2_4 = &HR_NoGo;
         %end;
      %end;
   %end;
   %else %do;
      %let HR_NoGo_A2_4 = 1;
   %end;
   
   * 5. Country; 
   %if %upcase(&TrtName) ne COUNTRY %then %do;
      %if ^%sysfunc(findw(%upcase(&DMVARS), COUNTRY)) %then %do;
         * no Country variable;
         %Case_Missing(DM_HR_Country,5);
         %let HR_NoGo_A2_5 = 0;
      %end;
      %else %do;
         %if &exiterror %then %do;
            * empty COUNTRY variable;
            %let exiterror = 0;
            %Case_Missing(DM_HR_Country,5);
            %let HR_NoGo_A2_5 = 0;
         %end;
         %else %do;
            %DM_SG_HR(outlib.&OutDataName,COUNTRY,DM_HR_Country);
            %let HR_NoGo_A2_5 = &HR_NoGo;
         %end;
      %end;
   %end;
   %else %do;
      %let HR_NoGo_A2_5 = 1;
   %end;
   
   * 6. Study Site; 
   %if %upcase(&TrtName) ne SITEID %then %do;
      %if ^%sysfunc(findw(%upcase(&DMVARS), SITEID)) %then %do;
         * no Ethnic variable;
         %Case_Missing(DM_HR_Site,6);
         %let HR_NoGo_A2_6 = 0;
      %end;
      %else %do;
         %if &exiterror %then %do;
            * empty SITEID variable;
            %let exiterror = 0;
            %Case_Missing(DM_HR_Site,6);
            %let HR_NoGo_A2_6 = 0;
         %end;
         %else %do;
            %DM_SG_HR(outlib.&OutDataName,SITEID,DM_HR_Site);
            %let HR_NoGo_A2_6 = &HR_NoGo;
         %end;
      %end;
   %end;
   %else %do;
      %let HR_NoGo_A2_6 = 1;
   %end;
   
   %if ^&HR_NoGo_A2_1 | ^&HR_NoGo_A2_2 | ^&HR_NoGo_A2_3 | 
      ^&HR_NoGo_A2_4 | ^&HR_NoGo_A2_5 | ^&HR_NoGo_A2_6 %then %let _Skip_A2 = 0;
   %else %let _Skip_A2 = 1;
   
   %if ^&_Skip_A2 %then %do;
      * Combine all subsets;
      data DM_HR_All;
         set 
            %if ^&HR_NoGo_A2_1 %then DM_HR_Age;
            %if ^&HR_NoGo_A2_2 %then DM_HR_Sex;
            %if ^&HR_NoGo_A2_3 %then DM_HR_Race;
            %if ^&HR_NoGo_A2_4 %then DM_HR_Ethnic;
            %if ^&HR_NoGo_A2_5 %then DM_HR_Country;
            %if ^&HR_NoGo_A2_6 %then DM_HR_Site;;
      run;
   
      %do i = 1 %to 6;
         %let MKM_OutData_A2_P&i = ;
         %PrepOutDS(MKM_OutData_A2_P&i,OutDataName_A2_P&i,overallsurvival,_a2&i);
      %end;
      
      * output data for drawing Forest Plot in JMP;
      data &MKM_OutData_A2_P1 &MKM_OutData_A2_P2 &MKM_OutData_A2_P3
         &MKM_OutData_A2_P4 &MKM_OutData_A2_P5 &MKM_OutData_A2_P6;
         set DM_HR_All;
         /*
         %do i = 1 %to %eval(&nTrtGroup-1);
            if HR_&i ne "NE" & WaldLower_&i ne . & WaldUpper_&i ne .;
         %end;
         */
         if _sec_ = 1 then output &MKM_OutData_A2_P1;
         else if _sec_ = 2 then output &MKM_OutData_A2_P2;
         else if _sec_ = 3 then output &MKM_OutData_A2_P3;
         else if _sec_ = 4 then output &MKM_OutData_A2_P4;
         else if _sec_ = 5 then output &MKM_OutData_A2_P5;
         else if _sec_ = 6 then output &MKM_OutData_A2_P6;
      run;
   
      %do i = 1 %to 6;  
         %if %sysfunc(exist(&&MKM_OutData_A2_P&i)) %then %do;
            %let _OutputDatasets_ = &_OutputDatasets_,&&OutDataName_A2_P&i...sas7bdat;
         %end;
      %end;
   %end;

%end;

*** Analysis 3 & 4: Hazard Ratio Estimates for Abnormal High/Low LB Test Subgroup at Baseline;
%if &_LB_Flag_ %then %do;

   %let BaselineTimeWindow = ;
   %let BaselineCalc = LAST;
   %let ResponseVar = LBSTRESN;
   
   %let FDeltaDrop = ;
   %let WhereTests = ;
   
   %CheckFindingsData(LB,_errchk = 0);
   %SplitBaselineTrial(LB,LB,TrialMeasurements=0);
   /*
   proc sort data=d1m;
      by USUBJID LBTESTCD;
   run;
   */
   proc sort data=lb(keep=USUBJID LBTESTCD LBTEST LBSTNRLO LBSTNRHI &GroupSortVars) out=lb2 nodupkey;
      by &GroupSortVars LBTESTCD USUBJID;
   run;
   %*put GroupSortVars=&GroupSortVars;
   data LB_H2 LB_L2;
      if 0 then set lb2;
      if _n_ = 1 then do;
         declare hash h(dataset: "lb2", hashexp: 6);
            h.defineKey("LBTESTCD","USUBJID");
            h.defineData(ALL:"YES");*("LBTEST","LBSTNRLO","LBSTNRHI");
            h.defineDone();
      end;
      set d1m; * output from SplitBaselineTrial;
      if h.find() = 0;
      if baseline < (LBSTNRLO/&LB_Low_Multiplier) then output LB_L2;
      else if baseline > (LBSTNRHI*&LB_High_Multiplier) then output LB_H2;
   run;
   
   proc sort data=LB_H2;
      by LBTEST;
   run;
   
   proc sort data=LB_L2;
      by LBTEST;
   run;
   
   * merge with data;
   data LB_H2;
      if 0 then set outlib.&OutDataName(keep=USUBJID Censor SURV%upcase(&TimeSCALE) &TrtName Trt_Num);
      if _n_ = 1 then do;
         declare hash h(dataset: "outlib.&OutDataName", hashexp: 6);
            h.defineKey("USUBJID");
            h.defineData("Censor","SURV%upcase(&TimeSCALE)","&TrtName","Trt_Num");
            h.defineDone();
      end;
      set LB_H2(keep=USUBJID LBTEST &GroupSortVars);
      if h.find() = 0;     
      if length(trim(left(LBTEST))) & SURV%upcase(&TimeSCALE) ne .;
   run;
   
   data LB_L2;
      if 0 then set outlib.&OutDataName(keep=USUBJID Censor SURV%upcase(&TimeSCALE) &TrtName Trt_Num);
      if _n_ = 1 then do;
         declare hash h(dataset: "outlib.&OutDataName", hashexp: 6);
            h.defineKey("USUBJID");
            h.defineData("Censor","SURV%upcase(&TimeSCALE)","&TrtName","Trt_Num");
            h.defineDone();
      end;
      set LB_L2(keep=USUBJID LBTEST &GroupSortVars);
      if h.find() = 0;
      if length(trim(left(LBTEST))) & SURV%upcase(&TimeSCALE) ne .;
   run;
   
   %DM_SG_HR(LB_H2,LBTEST,LB_HR_H);
   %if &HR_NoGo %then %do;
      %let _Skip_LB_H = 1;    
      %put NOTE: Skip the Hazard Ratio Estimat for Abnormal High LB Test Subgroup at Baseline because of not enough data for valid estimation.;
      %goto end_LB_H;
   %end; 
   
   * output data for drawing Forest Plot in JMP;
   %let MKM_OutData_A3=;
   %PrepOutDS(MKM_OutData_A3,OutDataName_A3,overallsurvival,_a3);
   data &MKM_OutData_A3;
      set LB_HR_H;
      /*
      %do i = 1 %to %eval(&nTrtGroup-1);
         if HR_&i ne "NE" & WaldLower_&i ne . & WaldUpper_&i ne .;
      %end;
      */
   run;
   
   %if %sysfunc(exist(&MKM_OutData_A3)) %then %do;
      %let _OutputDatasets_ = &_OutputDatasets_,&OutDataName_A3..sas7bdat;
   %end;
   %end_LB_H:;
      
   /*%let exiterror = 1; %goto exit;*/
   %DM_SG_HR(LB_L2,LBTEST,LB_HR_L);
   %if &HR_NoGo %then %do;
      %let _Skip_LB_L = 1;
      %put NOTE: Skip the Hazard Ratio Estimat for Abnormal Low LB Test Subgroup at Baseline because of not enough data for valid estimation.;
      %goto end_LB_L;
   %end; 
   
   * output data for drawing Forest Plot in JMP;
   %let MKM_OutData_A4=;
   %PrepOutDS(MKM_OutData_A4,OutDataName_A4,overallsurvival,_a4);
   data &MKM_OutData_A4;
      set LB_HR_L;
      /*
      %do i = 1 %to %eval(&nTrtGroup-1);
         if HR_&i ne "NE" & WaldLower_&i ne . & WaldUpper_&i ne .;
      %end;
      */
   run;
   
   %if %sysfunc(exist(&MKM_OutData_A4)) %then %do;
      %let _OutputDatasets_ = &_OutputDatasets_,&OutDataName_A4..sas7bdat;
   %end; 
   %end_LB_L:;
%end;


*** Analysis 5: Hazard Ratio Estimates for Concomitant Medication Subgroup at Baseline;
%if &_CM_Flag_ %then %do;
   %LOADDATA(domlist=CM);
   * extract data at baseline;
   data cm2;
      set &in_cm;
      if STDY <=1 & ENDY > 1;
   run;
   
   %if ^%sysfunc(findw(%upcase(&CMVARS), CMDECOD)) %then %do;
      %if ^%sysfunc(findw(%upcase(&CMVARS), CMTRT)) %then %do;
         %put NOTE: Skip the Hazard Ratio Estimat for Concomitant Medication Subgroup at Baseline because of the required variables CMDECOD/CMTRT are missing.;
         %let _Skip_CM = 1;
         %goto exit_CM;
      %end;
      %else %do;
         %let _CM_ByVar = CMTRT;
         %let _CM_label = Reported Name;
      %end;
   %end;
   %else %do;
      %let _CM_ByVar = CMDECOD;
      %let _CM_label = Standardized Medication Name;
   %end;
   
   proc sort data=cm2(keep=&_CM_ByVar USUBJID) nodupkey;
      by &_CM_ByVar USUBJID;
   run;

   *  extract Concomitant Medication Subgroup at Baseline from main data;
   data cm3;
      if 0 then set outlib.&OutDataName(keep=USUBJID Censor SURV%upcase(&TimeSCALE) &TrtName Trt_Num);
      if _n_ = 1 then do;
         declare hash h(dataset: "outlib.&OutDataName", hashexp: 6);
            h.defineKey("USUBJID");
            h.defineData("Censor","SURV%upcase(&TimeSCALE)","&TrtName","Trt_Num");
            h.defineDone();
      end;
      set cm2(keep=USUBJID &_CM_ByVar);
      if h.find() = 0;
      if length(trim(left(&_CM_ByVar))) & SURV%upcase(&TimeSCALE) ne .;
   run;
   
   %DM_SG_HR(cm3,&_CM_ByVar,CM_HR); 
   %if &HR_NoGo %then %do;
      %let _Skip_CM = 1;
      %put NOTE: Skip the Hazard Ratio Estimat for Concomitant Medication Subgroup at Baseline because of not enough data for valid estimation.;
      %goto exit_CM;
   %end; 
   
   * output data for drawing Forest Plot in JMP;
   %let MKM_OutData_A5=;
   %PrepOutDS(MKM_OutData_A5,OutDataName_A5,overallsurvival,_a5);
   data &MKM_OutData_A5;
      set CM_HR;
      /*
      %do i = 1 %to %eval(&nTrtGroup-1);
         if HR_&i ne "NE" & WaldLower_&i ne . & WaldUpper_&i ne .;
      %end;
      */
   run;
   
   %if %sysfunc(exist(&MKM_OutData_A5)) %then %do;
      %let _OutputDatasets_ = &_OutputDatasets_,&OutDataName_A5..sas7bdat;
   %end;
%end;
%exit_CM:;


*** Analysis 6: Hazard Ratio Estimates for Medication History Subgroup at Baseline;
%if &_MH_Flag_ %then %do;
   %LOADDATA(domlist=MH);
      
   * extract data at baseline;
   data mh2;
      set &in_mh;
      if STDY <=1;* & ENDY > 1; 
   run;

   %let _Skip_MH2 = 0;
   %nobsvars(mh2);
   %if ^&nobs %then %do;
      %put NOTE: Skip the Hazard Ratio Estimate for Medical History Subgroup at Baseline because of no data for valid estimation.;
      %let _Skip_MH = 1;
      %goto exit_MH;
   %end;
         
   %if ^%sysfunc(findw(%upcase(&MHVARS), MHDECOD)) %then %do;
      %if ^%sysfunc(findw(%upcase(&MHVARS), MHTERM)) %then %do;
         %put NOTE: Skip the Hazard Ratio Estimate for Medical Histroy Subgroup at Baseline because of the required variables CMDECOD/CMTRT are missing.;
         %let _Skip_MH = 1;
         %goto exit_MH;
      %end;
      %else %do;
         %let _MH_ByVar = MHTERM;
         %let _MH_Label = Reported Term; * y-axix name in hazard ratio plots;
      %end;
   %end;
   %else %do;
      %let _MH_ByVar = MHDECOD;
      %let _MH_Label = Dictionary-Derived Term;
   %end;
   
   proc sort data=mh2(keep=&_MH_ByVar USUBJID) nodupkey;
      by &_MH_ByVar USUBJID;
   run;

   *  extract Medical History Subgroup at Baseline from main data;
   data mh3;
      if 0 then set outlib.&OutDataName(keep=USUBJID Censor SURV%upcase(&TimeSCALE) &TrtName Trt_Num);
      if _n_ = 1 then do;
         declare hash h(dataset: "outlib.&OutDataName", hashexp: 6);
            h.defineKey("USUBJID");
            h.defineData("Censor","SURV%upcase(&TimeSCALE)","&TrtName","Trt_Num");
            h.defineDone();
      end;
      set mh2(keep=USUBJID &_MH_ByVar);
      if h.find() = 0;
      if length(trim(left(&_MH_ByVar))) & SURV%upcase(&TimeSCALE) ne .;
   run;
   
   %DM_SG_HR(mh3,&_MH_ByVar,MH_HR);
   %if &HR_NoGo %then %do;
      %let _Skip_MH = 1;
      %put NOTE: Skip the Hazard Ratio Estimat for Medical History Subgroup at Baseline because of not enough data for valid estimation.;
      %goto exit_MH;
   %end; 

   %let MKM_OutData_A6=;
   %PrepOutDS(MKM_OutData_A6,OutDataName_A6,overallsurvival,_a6);
   data &MKM_OutData_A6;
      set MH_HR;
      /*
      %do i = 1 %to %eval(&nTrtGroup-1);
         if HR_&i ne "NE" & WaldLower_&i ne . & WaldUpper_&i ne .;
      %end;
      */
   run;
   
   %if %sysfunc(exist(&MKM_OutData_A6)) %then %do;
      %let _OutputDatasets_ = &_OutputDatasets_,&OutDataName_A6..sas7bdat;
   %end;
   /*
   %nobsvars(&MKM_OutData_A6);
   %if ^&nobs %then %do;
      %put NOTE: Skip the Hazard Ratio Estimat for Medical History Subgroup at Baseline because of not enough data for valid estimation.;
      %let _Skip_MH2 = 1;
   %end;
   %else %do;
      %let _OutputDatasets_ = &_OutputDatasets_,&OutDataName_A6..sas7bdat;
   %end;
   */
%end;
%exit_MH:;


* generate tables;
%let MortalityKaplanMeierRtf = ;
%PrepOutFile(MortalityKaplanMeierRtf,MortalityKaplanMeierRtfName,&StudyName OverallSurvivalTables.rtf);

options orientation=landscape;
ods rtf file="&MortalityKaplanMeierRtf";
%Table_A1;
%let _BNum = 1;
%if &_DM_Flag_ %then %do;
   %if ^&_Skip_A2 %then %do;
      %let _BNum = %eval(&_BNum + 1);
      %Table_A2(2,DM_HR_All);
   %end;
%end;
%if &_LB_Flag_ %then %do;
   %if ^&_Skip_LB_H %then %do;
      %let _BNum = %eval(&_BNum + 1);
      %Table_A2(3,LB_HR_H);
   %end;
   %if ^&_Skip_LB_L %then %do;
      %let _BNum = %eval(&_BNum + 1);
      %Table_A2(4,LB_HR_L);
   %end;
%end;
%if &_CM_Flag_ & ^&_Skip_CM %then %do;
   %let _BNum = %eval(&_BNum + 1);
   %Table_A2(5,CM_HR);
%end;
%if &_MH_Flag_ & ^&_Skip_MH %then %do;
   %let _BNum = %eval(&_BNum + 1);
   %Table_A2(6,MH_HR);
%end;
ods rtf close;

%PrepOutDS(MortalityKaplanMeier_OutReport,OutReport,MortalityKaplanMeier,_Report);
%PrepHTML(
    MortalityKaplanMeier_Body, 
    MortalityKaplanMeier_Frame, 
    MortalityKaplanMeier_Contents, 
    &MortalityKaplanMeier_OutReport._body.html, 
    &MortalityKaplanMeier_OutReport._frame.html, 
    &MortalityKaplanMeier_OutReport._contents.html
);
options orientation=landscape;
ods html 
   body = "&MortalityKaplanMeier_Body" (url="&bodyfile")
   frame = "&MortalityKaplanMeier_Frame"  (url="&framefile")
   contents = "&MortalityKaplanMeier_Contents" (url="&contentsfile")
;
%Table_A1;
%let _BNum = 1;
%if &_DM_Flag_ %then %do;
   %if ^&_Skip_A2 %then %do;
      %let _BNum = %eval(&_BNum + 1);
      %Table_A2(2,DM_HR_All);
   %end;
%end;
%if &_LB_Flag_ %then %do;
   %if ^&_Skip_LB_H %then %do;
      %let _BNum = %eval(&_BNum + 1);
      %Table_A2(3,LB_HR_H);
   %end;
   %if ^&_Skip_LB_L %then %do;
      %let _BNum = %eval(&_BNum + 1);
      %Table_A2(4,LB_HR_L);
   %end;
%end;
%if &_CM_Flag_ & ^&_Skip_CM %then %do;
   %let _BNum = %eval(&_BNum + 1);
   %Table_A2(5,CM_HR);
%end;
%if &_MH_Flag_ & ^&_Skip_MH %then %do;
   %let _BNum = %eval(&_BNum + 1);
   %Table_A2(6,MH_HR);
%end;
ods HTML close;

%nodeathexit:;

%DMFlags;
%DMLabels(OutLib.&OutDataName);
%CheckStudyLevels;

%let OpenSASOutputTab=0;

* create JSL file;
%PrepOutFile(MortalityKaplanMeierScript,MortalityKaplanMeierScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &MortalityKaplanMeierScript;
%let FullAPName=%str(Mortality Time to Event);

* put JSL statements;
data _null_;
   file "&MortalityKaplanMeierScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:CreateReportListone={};";
   put "objRefNS:CreateReportListtwo= {};";
   put "objRefNS:NoteDomain = ""DS"";";
   %if &DeathResults %then %do;
		put "objRefNS:kmd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
		put "jg:ApplyValueOrderToTables(EvalList({objRefNS:kmd}), ""&StudyName"");";
		put "reportRefNS:linkAnalysisTableToADSLTable(objRefNS:kmd);";
		put "       Column(""&SubjectLabel"") << Label(1);";
		put "       Column(""&DeathLabel"") << Set Modeling Type(Nominal);";
		%if &TType = N %then %do;
			put "       Column(""&TrtLabel"") << Set Modeling Type(Nominal);";
		%end;
		put "       objRefNS:dbv = {};";
		%do i=1 %to &ndbv;
			put "       InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";    
		%end;   
		put "objRefNS:kmw=Function({},{default local},";
		put "   If(IsEmpty(objRefNS:kmd),";
		put "      objRefNS:kmd=open(""&ClientOutPath.OutDataName..sas7bdat"",invisible);";
		put "      jg:ApplyValueOrderToTables(EvalList({objRefNS:kmd}), ""&StudyName"");";
		put "   );";

		put "   VListBox(";
			%let trOpen = %nrbquote(%LocaleGet(Open_Text));
			put "TextBox(""""),";
			put "      objRefNS:Button1=ButtonBox(""&trOpen Overall Survival Report.rtf"",";
			put "          Open(""&OutPath.\&MortalityKaplanMeierRtfName"");";
			put "      ),";
			put "      objRefNS:Button1 << Underline Style(1),";  
			put "TextBox(""""),";


		put "   HListBox(";  
		put "   VListBox(";
		%if %length(&BeginTime) or %length(&EndTime) %then %do;
			put "   HListBox(";
			%if %length(&BeginTime) %then %do;
			%let trBeginTime = %nrbquote(%LocaleGet(BeginTime_Text));
				put "   textbox(""&trBeginTime = &trSingleTS &BeginTime   ""), "; 
			%end;
			%if %length(&EndTime) %then %do;
			%let trEndTime = %nrbquote(%LocaleGet(EndTime_Text));
				put "   textbox(""&trEndTime = &trSingleTS &EndTime""), "; 
			%end;
			put "   ),";
		%end;
		put "   HListBox(";
		put "   objRefNS:sp = VListBox(";
		put "		TextBox(locale:fill(locale:get(""NoteSubjectsWereCensoredUsingX_Text""), evalList({locale:ezx(""&CensorDateValue"")})),<<Set Wrap(500),<<Set Font Size(9)),";
		%if &DCUTDmissflag %then %do;
			put "		TextBox(locale:fill(locale:get(""NoteXSubjectsWereRemFromAnDueToMissCensDateInfo_Text""), {""&missingObs""}),<<Set Wrap(700),<<Set Font Size(9)),";
		%end;
		put "      Survival(Y(Column(""&trxsurv"")),Censor(:&trcensor), Grouping(Name(""&TrtLabel"")), ";
		%if %upcase(&SplitResults)=YES %then %do;
			put "         By(eval(objRefNS:dbv)),";
		%end;
		put "        	Automatic Recalc( 1 ),  Show Simultaneous CI(0),Show Points(1), Show Shaded Simultaneous CI(&shadedCI),";
		put "         Competing Causes(Name(""&TrtLabel"")),Weibull Lines( 1 ),Hazard Plot( 1 ),";
		put "         SendToReport(";
		put "            Dispatch(";
		put "               {""Survival Plot""},";
		put "                ""2"",";
		put "                ScaleBox,";
		put "                {Min( -0.05), Max( 1.05 ), Inc( 0.2 ),";
		put "                Minor Ticks( 1 )}";
		put "             )";
		put "          )";
		put "      )";
		put "   ),";
		put "   objRefNS:cp = VListBox(";
		put	" 	   TextBox(locale:get(""NoteOverallCtsBelowRefDeathOccRegOfCensDate_Text""),<<Set Wrap(600),<<Set Font Size(9)),";
		put "      Contingency(";
		put "            Automatic Recalc( 1), Contingency Table(1),";  
		%if %upcase(&SplitResults)=YES %then %do;
			put "         By(eval(objRefNS:dbv)),";
		%end;
		put "         Y(Column(""&DeathLabel"")),";
		put "         X(Column(""&TrtLabel"")),";
		put "				SendToReport(";
		put "					Dispatch(";
		put "						{},";
		put "	""%nrbquote(%sysfunc(transtrn(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(ContingencyAnalysisOfSubjectDiedByX_Text)), %nrbquote(&TrtLabel))), %str(%"), %str(\!%"))))"",";
		put "						OutlineBox,";
		put "						{Set Title( ""%nrbquote(%LocaleGet(ContingencyAnalysisForOverallDeathCounts_Text))"" )}";
		put "					)";
		put "				)";
		put "      )";
		put "   )";
		put ")),";            
		put "),";
		%if ^&_DM_Flag_ & ^&_LB_Flag_ & ^&_CM_Flag_ & ^&_MH_Flag_ %then %do;
			put "TextBox(locale:get(""NoteHazardRatiosCantBeCompBcOnlyOneTrtLev_Text""),<<Set Wrap(600),<<Set Font Size(9)),";
		%end;
		put "   )";
		put "   )";
		put ";";

		put "If(!reportRefNS:useGlobalSubjectFilterWithinReview(),";
		put "      objRefNS:AddDataFilterContextBox = 1;";
		put "      objRefNS:filterColumnsList = {};";     
		put "   objRefNS:ReportDataFilterReferenceList = reportRefNS:createReportFilter(objRefNS:kmd, objRefNS:filterColumnsList);";
		put ");";

   	
		put 'InsertInto(objRefNS:DataList,{{"objRefNS:kmd"}});';
		put "InsertInto(objRefNS:DataNameList,{{locale:get(""OverallSurvival_Text"")}});";
		put 'InsertInto(objRefNS:TabButtonList, expr(locale:get("OverallSurvival_Text")));';
		put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:kmw()));';
		put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
		put 'InsertInto(objRefNS:TabOpenList,1);';
		put "InsertInto(objRefNS:CreateReportList,{{objRefNS:sp,objRefNS:cp}});";

		put "include(""$LS_HOME\JSLFiles\HazardRatioPlot.JSL"");";
	
		%if &_DM_Flag_ %then %do;
      *check how many plots shall be shown;
      %if %upcase(&HR_Flag_AgeGroup) = YES %then %let HR1 = 1;
      %if %upcase(&HR_Flag_Gender) = YES %then %let HR2 = 1;
      %if %upcase(&HR_Flag_Race) = YES %then %let HR3 = 1;
      %if %upcase(&HR_Flag_Ethnicity) = YES %then %let HR4 = 1;
      %if %upcase(&HR_Flag_Country) = YES %then %let HR5 = 1;
      %if %upcase(&HR_Flag_StudySite) = YES %then %let HR6 = 1;
      
      %let _LastHR = 0;
      %if %upcase(&HR_Flag_StudySite) = YES %then %let _LastHR = 6;
      %else %if %upcase(&HR_Flag_Country) = YES %then %let _LastHR = 5;
      %else %if %upcase(&HR_Flag_Ethnicity) = YES %then %let _LastHR = 4;
      %else %if %upcase(&HR_Flag_Race) = YES %then %let _LastHR = 3;
      %else %if %upcase(&HR_Flag_Gender) = YES %then %let _LastHR = 2;
      %else %if %upcase(&HR_Flag_AgeGroup) = YES %then %let _LastHR = 1;
      
		* Analysis 2;
		put "objRefNS:Analysis2_code=Function({},{default local},";
		put "   if(IsEmpty(objRefNS:A2_dt1),";
		put "      objRefNS:A2_dt1 = open(""&ClientOutPath.&OutDataName_A2_P1..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "   );";
		put "   if(IsEmpty(objRefNS:A2_dt2),";
		put "      objRefNS:A2_dt2 = open(""&ClientOutPath.&OutDataName_A2_P2..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "   );";
		put "   if(IsEmpty(objRefNS:A2_dt3),";
		put "      objRefNS:A2_dt3 = open(""&ClientOutPath.&OutDataName_A2_P3..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "   );";
		put "   if(IsEmpty(objRefNS:A2_dt4),";
		put "      objRefNS:A2_dt4 = open(""&ClientOutPath.&OutDataName_A2_P4..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "   );";
		put "   if(IsEmpty(objRefNS:A2_dt5),";
		put "      objRefNS:A2_dt5 = open(""&ClientOutPath.&OutDataName_A2_P5..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "   );";
		put "   if(IsEmpty(objRefNS:A2_dt6),";
		put "      objRefNS:A2_dt6 = open(""&ClientOutPath.&OutDataName_A2_P6..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "   );";
		put "   jg:ApplyValueOrderToTables(EvalList({objRefNS:A2_dt1, objRefNS:A2_dt2, objRefNS:A2_dt3, objRefNS:A2_dt4, objRefNS:A2_dt5,";
		put "    objRefNS:A2_dt6}), ""&StudyName"");";

		%if &nTrtGroup = 2 %then %do;
		put "   objRefNS:A2_plot = vListBox(";
		put "      TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&ARM_1))"", ""%trim(%nrbquote(&ARM_2))""}));";
		put "      ,";    
      
      %if %upcase(&HR_Flag_AgeGroup) = YES %then %do;
		put "         objRefNS:HR_ForestPlot(objRefNS:A2_dt1,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
		put "            ""&trHazardRatioforAgeSG"",""&trHazardRatio"",""&trAgeGroup"",""objRefNS:DM_P1"");";
         %if &_LastHR > 1 %then put "      ,";;
      %end;
      %if %upcase(&HR_Flag_Gender) = YES %then %do;
		put "         objRefNS:HR_ForestPlot(objRefNS:A2_dt2,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
		put "            ""&trHazardRatioforGenderSG"",""&trHazardRatio"",""&trGender"",""objRefNS:DM_P2"");";
         %if &_LastHR > 2 %then put "      ,";;
         put "      ,";
      %end;
      %if %upcase(&HR_Flag_Race) = YES %then %do;
		put "         objRefNS:HR_ForestPlot(objRefNS:A2_dt3,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
		put "            ""&trHazardRatioforRaceSG"",""&trHazardRatio"",""&trRace"",""objRefNS:DM_P3"");";
         %if &_LastHR > 3 %then put "      ,";;
		put "      ,";
      %end;
      %if %upcase(&HR_Flag_Ethnicity) = YES %then %do;
		put "         objRefNS:HR_ForestPlot(objRefNS:A2_dt4,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
		put "            ""&trHazardRatioforEthnicitySG"",""&trHazardRatio"",""&trEthnicity"",""objRefNS:DM_P4"");";
         %if &_LastHR > 4 %then put "      ,";;
     		put "      ,";
      %end;
      
      %if %upcase(&HR_Flag_Country) = YES %then %do;
		put "         objRefNS:HR_ForestPlot(objRefNS:A2_dt5,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
		put "            ""&trHazardRatioforCountrySG"",""&trHazardRatio"",""&trCountry"",""objRefNS:DM_P5"");";
         %if &_LastHR > 5 %then put "      ,";;
		put "      ,";
      %end; 
      %if %upcase(&HR_Flag_StudySite) = YES & &HR_NoGo_A2_6 = 0 %then %do;
		put "         objRefNS:HR_ForestPlot(objRefNS:A2_dt6,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
		put "            ""&trHazardRatioforStudySiteSG"",""&trHazardRatio"",""&trStudySiteIdentifier"",""objRefNS:DM_P6"");";
      %end;
		put "   );";
		%end;
		%else %do;
		put "   objRefNS:A2_plot = hListBox(";
			%do i = 1 %to %eval(&nTrtGroup-1);
		put "      vListBox("; 
		put "         TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&&ARM_&i))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
		put "      ,";    
		put "         vListBox(";
      %if %upcase(&TrtName) ne AGE & %upcase(&HR_Flag_AgeGroup) = YES %then %do;
		put "            objRefNS:HR_ForestPlot(objRefNS:A2_dt1,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforAgeSG"",""&trHazardRatio"",""&trAgeGroup"",""objRefNS:DM_P1_&i"");";
      %if &_LastHR > 1 %then put "         ,";;
		%end;
      
      %if %upcase(&TrtName) ne SEX & %upcase(&HR_Flag_Gender) = YES %then %do;
		put "            objRefNS:HR_ForestPlot(objRefNS:A2_dt2,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforGenderSG"",""&trHazardRatio"",""&trGender"",""objRefNS:DM_P2_&i"");";
      %if &_LastHR > 2 %then put "         ,";;
		%end;
      
      %if %upcase(&TrtName) ne RACE & %upcase(&HR_Flag_Race) = YES %then %do;
		put "            objRefNS:HR_ForestPlot(objRefNS:A2_dt3,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforRaceSG"",""&trHazardRatio"",""&trRace"",""objRefNS:DM_P3_&i"");";
      %if &_LastHR > 3 %then put "         ,";;
		%end;
      %if %upcase(&TrtName) ne ETHNIC & %upcase(&HR_Flag_Ethnicity) = YES %then %do;
		put "            objRefNS:HR_ForestPlot(objRefNS:A2_dt4,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforEthnicitySG"",""&trHazardRatio"",""&trEthnicity"",""objRefNS:DM_P4_&i"");";
      %if &_LastHR > 4 %then put "         ,";;
		%end;
      %if %upcase(&TrtName) ne COUNTRY & %upcase(&HR_Flag_Country) = YES %then %do;
		put "            objRefNS:HR_ForestPlot(objRefNS:A2_dt5,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforCountrySG"",""&trHazardRatio"",""&trCountry"",""DM_P5_&i"");";
      %if &_LastHR > 5 %then put "         ,";;
		%end;
      %if %upcase(&TrtName) ne SITEID & %upcase(&HR_Flag_StudySite) = YES & &HR_NoGo_A2_6 = 0 %then %do;
		put "            objRefNS:HR_ForestPlot(objRefNS:A2_dt6,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforStudySiteSG"",""&trHazardRatio"",""&trStudySiteIdentifier"",""DM_P6_&i"");";
		%end;
		put "         );";
		put "      );";
				%if &i < %eval(&nTrtGroup-1) %then %do;
		put "      ,";
				%end;
			%end;
		put "   );";
		%end;
		put ");";
		put " ";
		%if ^&_Skip_A2 %then %do;
		put 'InsertInto(objRefNS:DataList,{{"objRefNS:A2_dt1","objRefNS:A2_dt2","objRefNS:A2_dt3","objRefNS:A2_dt4","objRefNS:A2_dt5","objRefNS:A2_dt6"}});';
		put "InsertInto(objRefNS:DataNameList,{{""&trAgeGroup"",""&trGender"",""&trRace"",""&trEthnicity"",""&trCountry"",""&trStudySiteIdentifier""}});";
		put 'InsertInto(objRefNS:TabButtonList, expr(locale:get("Demographics_Text")));'; *translation!;
		put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:Analysis2_code()));';
		put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
		put 'InsertInto(objRefNS:TabOpenList,1);';
		put "InsertInto(objRefNS:CreateReportList,{{objRefNS:A2_plot}});";
		put " ";
		%end;
		%end;

		%if &_LB_Flag_ %then %do;
		* Analysis 3;
		put "objRefNS:Analysis3_code=Function({},{default local},";
		%if ^&_Skip_LB_H %then %do;
		put "   if(IsEmpty(objRefNS:A3_dt1),";
		put "      objRefNS:A3_dt1 = open(""&ClientOutPath.&OutDataName_A3..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "      jg:ApplyValueOrderToTables(EvalList({objRefNS:A3_dt1}), ""&StudyName"");";
		put "   );";
		%end;

		put "   objRefNS:A3_plot = hListBox(";
		%if ^&_Skip_LB_H %then %do;
			%do i = 1 %to %eval(&nTrtGroup-1);
		put "      vListBox(";
		put "         TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&&ARM_&i))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
		put "         ,";
		put "         hListBox(";
		put "            objRefNS:HR_ForestPlot(objRefNS:A3_dt1,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforHighBLTSG"",""&trHazardRatio"",""&trLabTest"",""LB_H_P_&i"");";
		put "         )";
		put "      )";
				%if (&nTrtGroup > 1) & (&i < %eval(&nTrtGroup-1)) %then %do;
		put "      ,";
				%end;
			%end;
		%end;
		%else %do;
		put "      hListBox(";
		put "         TextBox(locale:fill(locale:get(""ThereisnotenoughdataforPerfValHazRatforXSubgroup_Text""), {locale:get(""HighBaselineLabTest_Text"")}));";
		put "      )";
		%end;
		put "   );";
		put ");";
		put " ";
		%if ^&_Skip_LB_H %then %do;
			put 'InsertInto(objRefNS:DataList,{{"objRefNS:A3_dt1"}});';
			put 'InsertInto(objRefNS:DataNameList,{{locale:get("HighBaselineLabTests_Text")}});';
			put 'InsertInto(objRefNS:TabButtonList, expr(locale:get("HighBaselineLabTests_Text")));'; *translation!;
			put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:Analysis3_code()));';
			put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
			put 'InsertInto(objRefNS:TabOpenList,1);';
			put "InsertInto(objRefNS:CreateReportList,{{objRefNS:A3_plot}});";
			put " ";
		%end;

		* Analysis 4;
		put "objRefNS:Analysis4_code=Function({},{default local},";
		%if ^&_Skip_LB_L %then %do;
		put "   if(IsEmpty(objRefNS:A4_dt1),";
		put "      objRefNS:A4_dt1 = open(""&ClientOutPath.&OutDataName_A4..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "      jg:ApplyValueOrderToTables(EvalList({objRefNS:A4_dt1}), ""&StudyName"");";
		put "   );";
		%end; 

		put "   objRefNS:A4_plot = hListBox(";
		%if ^&_Skip_LB_L %then %do;
			%do i = 1 %to %eval(&nTrtGroup-1);
		put "      vListBox(";
		put "         TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&&ARM_&i))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
		put "         ,";
		put "         hListBox(";
		put "            objRefNS:HR_ForestPlot(objRefNS:A4_dt1,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforLowBLTSG"",""&trHazardRatio"",""&trLabTest"",""objRefNS:LB_L_P"");";
		put "         )";
		put "      )";
				%if (&nTrtGroup > 1) & (&i < %eval(&nTrtGroup-1)) %then %do;
		put "      ,";
				%end;
			%end;
		%end;
		%else %do;
		put "      hListBox(";
		put "         TextBox(locale:fill(locale:get(""ThereisnotenoughdataforPerfValHazRatforXSubgroup_Text""), {locale:get(""LowBaselineLabTest_Text"")}));";
		put "      )";
		%end;
		put "   );";
		put ");";
		put " ";
		%if ^&_Skip_LB_L %then %do;
			put 'InsertInto(objRefNS:DataList,{{"objRefNS:A4_dt1"}});';
			put 'InsertInto(objRefNS:DataNameList,{{locale:get("LowBaselineLabTests_Text")}});';
			put 'InsertInto(objRefNS:TabButtonList, expr(locale:get("LowBaselineLabTests_Text")));'; *translation!;
			put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:Analysis4_code()));';
			put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
			put 'InsertInto(objRefNS:TabOpenList,1);';
			put "InsertInto(objRefNS:CreateReportList,{{objRefNS:A4_plot}});";
		%end;
		%end;

		%if &_CM_Flag_ %then %do;
		* Analysis 5;
		put "objRefNS:Analysis5_code=Function({},{default local},";
		%if ^&_Skip_CM %then %do;
		put "   if(IsEmpty(objRefNS:A5_dt1),";
		put "      objRefNS:A5_dt1 = open(""&ClientOutPath.&OutDataName_A5..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "      jg:ApplyValueOrderToTables(EvalList({objRefNS:A5_dt1}), ""&StudyName"");";
		put "   );";
		%end;

		put "   objRefNS:A5_plot = hListBox(";
		%if ^&_Skip_CM %then %do;
			%do i = 1 %to %eval(&nTrtGroup-1);
		put "      vListBox(";
		put "         TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&&ARM_&i))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
		put "         ,";
		put "         hListBox(";
		put "            objRefNS:HR_ForestPlot(objRefNS:A5_dt1,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforConMedSG"",""&trHazardRatio"",""&_CM_Label"",""objRefNS:CM_P"");";
		put "         )";
		put "      )";
				%if (&nTrtGroup > 1) & (&i < %eval(&nTrtGroup-1)) %then %do;
		put "      ,";
				%end;
			%end;
		%end;
		%else %do;
		put "      hListBox(";
		put "         TextBox(locale:fill(locale:get(""ThereisnotenoughdataforPerfValHazRatforXSubgroup_Text""), {locale:get(""ConcomitantMedication_Text"")}));";
		put "      )";
		%end;
		put "   );";
		put ");";
		put " ";
		%if ^&_Skip_CM %then %do;
			put 'InsertInto(objRefNS:DataList,{{"objRefNS:A5_dt1"}});';
			put 'InsertInto(objRefNS:DataNameList,{{locale:get("ConcomitantMedications_Text")}});';		
			put 'InsertInto(objRefNS:TabButtonList, expr(locale:get("ConcomitantMedications_Text")));'; *translation!;
			put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:Analysis5_code()));';
			put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
			put 'InsertInto(objRefNS:TabOpenList,1);';
			put "InsertInto(objRefNS:CreateReportList,{{objRefNS:A5_plot}});";
		%end;
		%end;

		%if &_MH_Flag_ %then %do;
		* Analysis 6;
		put "objRefNS:Analysis6_code=Function({},{default local},";
			%if ^&_Skip_MH %then %do;
		put "   if(IsEmpty(objRefNS:A6_dt1),";
		put "      objRefNS:A6_dt1 = open(""&ClientOutPath.&OutDataName_A6..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
		put "      jg:ApplyValueOrderToTables(EvalList({objRefNS:A6_dt1}), ""&StudyName"");";
		put "   );";
			%end;

		put "   objRefNS:A6_plot = hListBox(";
		%if ^&_Skip_MH & ^&_Skip_MH2 %then %do;
			%do i = 1 %to %eval(&nTrtGroup-1);
		put "      vListBox(";
		put "         TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&&ARM_&i))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
		put "         ,";
		put "         hListBox(";
		put "            objRefNS:HR_ForestPlot(objRefNS:A6_dt1,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
		put "               ""&trHazardRatioforMedHistorySG"",""&trHazardRatio"",""&_MH_Label"",""objRefNS:MH_P"");";
		put "         )";
		put "      )";
				%if (&nTrtGroup > 1) & (&i < %eval(&nTrtGroup-1)) %then %do;
		put "      ,";
				%end;
			%end; 
		%end;
		%else %do;
		put "      hListBox(";
		put "         TextBox(locale:fill(locale:get(""ThereisnotenoughdataforPerfValHazRatforXSubgroup_Text""), {locale:get(""MedicalHistory_Text"")}));";
		put "      )";
		%end;
		put "   );";
		put ");";
		put " ";
		%if ^&_Skip_MH %then %do;
			put 'InsertInto(objRefNS:DataList,{{"objRefNS:A6_dt1"}});';
			put 'InsertInto(objRefNS:DataNameList,{{locale:get("MedicalHistory_Text")}});';				
			put 'InsertInto(objRefNS:TabButtonList, expr(locale:get("MedicalHistory_Text")));'; *translation!;
			put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:Analysis6_code()));';
			put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
			put 'InsertInto(objRefNS:TabOpenList,1);';
			put "InsertInto(objRefNS:CreateReportList,{{objRefNS:A6_plot}});";
		%end;
		%end;

	/*  
		put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
		put "try(objRefNS:WhereState=""\[&WhereForReports]\"");";
		put "try(objRefNS:PopulationSet=""&PopulationSet"");";
		put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 


		put '//objRefNS:kmw << SetWindowIcon("$CLINICAL_HOME\Documentation\Icons\LifeTables.bmp");'; 

		 put "objRefNS:showReportFunc=function({},{default local},";
		 put "    objRefNS:reportWB = WebBrowserBox();";
		 put ");";

		 put "objRefNS:postReportFunc=Function({},{default local},";
		 put "    objRefNS:reportWB << navigate(""&outpath.&framefile"");";
		 put "    objRefNS:reportWB << setSize(800, 600);";
		 put ");";

		put "InsertInto(objRefNS:DataList,{{""""}});";
		put "InsertInto(objRefNS:DataNameList,{{""""}});";
		put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:showReportFunc()));";
		put "InsertInto(objRefNS:TabButtonList, expr(""Report""));";
		put "InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:postReportFunc()));";
		put "InsertInto(objRefNS:TabOpenList,1);";
	*/
		%SubjectDrillDowns;
		put "objRefNS:ActionDataTableList = {{{""objRefNS:kmd""},{""objRefNS:kmd""},{""objRefNS:kmd""},{""objRefNS:kmd""}}};";
	%end;
   %else %do;
      put "objRefNS:nodeathfunc=Function({},{default local},";
      put "    TextBox(locale:get(""ThereAreNoDeathstoReport_Text""),<<Set Font Size(14), <<Set Wrap(850));";
      put ");";
      
      put "objRefNS:ShowDataTables = 0;";
      put "objRefNS:ShowNotes = 0;";
      
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""OverallSurvival_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:nodeathfunc()));";
      put 'InsertInto(objRefNS:TabAfterCodeList, " ");';    
      put "InsertInto(objRefNS:TabOpenList,1);";   
   %end;
		
run;

%if &DeathResults %then %do;
	%PackageInsertHtml(HTML from &ProcessName, &OutPath, &bodyfile, &framefile, &contentsfile, yes, no);
	%PackageInsertFile(Hazard Ratio Tablets from &ProcessName, &OutPath, &MortalityKaplanMeierRtfName, , rtf, yes, no);
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &MortalityKaplanMeierScriptName, "&_OutputDatasets_", yes);
%end;
%else %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &MortalityKaplanMeierScriptName, &OutDataName..sas7bdat, yes);
%end;

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend MortalityKaplanMeier;

* run the macro;
%MortalityKaplanMeier;


