/*
JMP Clinical Macros

Name: CDISCParms.sas
Description: CDISCParms contains a collection
of useful SAS macros that can be called from
Clinical analytical processes.

Detailed:
   - INITIALIZE:         This macro is used to initialize data in JMPC. The majority of JMPC processes will run 
                           this macro in the beginning.   
   - LOADDATA:           This macro will apply subject filter and check for snap shot comparison and supplemental domains, 
                           and then merge in on the fly. It also calculates study day for each domain.
   - TREATMENTEMERGENT:  Calculate TEAE if TEAE flag Treatment Emergent Flag.
   - GROUPASSIGN:        This macro assigns domain grouping data set macro variable.
   - CDISCParms:         Assign CDISC parameters as macro variables. Run this macro since all processes need it.
   - AssignCOTreatment:  This macro is used in data step that will create a new treatment variable to incorporate crossover.
   - GroupVars:          Define new variables &domain.GRP, &domain.GRPN (numeric group index) and &domain.SGRPN (numeric group subindex) in order to meaningfully sort values.
   - CHKDATE:            This examines the validity of ISO8601 Dates.
   - CalculateStudyDays  This macro will calculate study day.
   - calcStudyDay:       Calculate Study Day Directly from the SDTM IG.
   - DATE9:              Create numeric days variable from iso 8601 variable.
   - TIME5:              Create numeric seconds variable from iso 8601 variable.
   - AddTimeWindows:     Add time window string to CatName and CatLabel if specified and/or create time window variable.
   - createCounts:       Create counts to be used in tree map.
   - SubjectDrillDowns:  macro for including drill-down buttons for Patient Profiler and Compare Subjects in tabbed report.
   - DMFlags:            Set flags for demographic variables.
   - DMLabels:           Set labels for demographic variables.
   - COMPAREDATES:       Compare start and stop date, month and year.
   - PROFILEDATEIMPUTE:  Impute profile date.
   - GETMEDDRAVAR:       Get Variable Based on Drop Down Label.
   - COMPAREMEDDRA:      Ensure that the Grouping Variable is at least the level of Term Variable.
   - AESerOnlyToAEWhereStmt:   check AESerOnly and modify AEWhereStmt accordingly, accounting for translation of "Y".
   - NONNULLVARS         For a dataset, determines the non-null variables
   - OUTNAMETRUNC        Truncates the name of a specified output data set name to keep name, suffixes and number within 32 characters
   - PARSEFINDINGS       Used with findings JSL widget to separate domain and test codes and create macro variables by domain 
   - CHECKOVERLAP        Used with findings JSL widget to check overlap of test codes between categorical and continuous lists 
   - CHECKPRES           Checks presence of variable in macro list and creates an indicator variable
   - REPLACEWORD         Removes a word from a text string and replaces with new word. Accounts for some arithmetic symbols
   - DELTAWHERE          Subsets data based on delta flag
   - TEAETEXTFLAG        Create footnote to communicate TEAE preferences
*/

*************************************************************************************;
*     Comment Blocks to parse and track variable specification in clinical system;
*     Create general Events, Interventions, Findings blocks for process inclusion;
*     Choose block to use based on requirements for calculating Study Day;
*************************************************************************************;
/*    
$CVS_START$
$ID: Events$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur","TreatmentUse","PopulationFlags","CalcStudyDay_Events"}}$
$CVS_STOP$

$CVS_START$
$ID: EventsO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur","TreatmentUse","PopulationFlags","CalcStudyDay_EventsO"}}$
$CVS_STOP$

$CVS_START$
$ID: Interventions$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","InterventionOccur","TreatmentUse","PopulationFlags","CalcStudyDay_Interventions"}}$
$CVS_STOP$

$CVS_START$
$ID: InterventionsO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","InterventionOccur","TreatmentUse","PopulationFlags","CalcStudyDay_InterventionsO"}}$
$CVS_STOP$


$CVS_START$
$ID: Findings$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_Findings"}}$
$CVS_STOP$

$CVS_START$
$ID: FindingsV$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_FindingsV"},{"Visits"}}$
$CVS_STOP$

$CVS_START$
$ID: FindingsO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_FindingsO"}}$
$CVS_STOP$

$CVS_START$
$ID: FindingsO2$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_FindingsO2"}}$
$CVS_STOP$

$CVS_START$
$ID: FindingsO3$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_FindingsO2","CalcStudyDay_FindingsO3"}}$
$CVS_STOP$
*/

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*************************************************************************************;
*      This function should always be used to load domains for every report;
*      This function may be called in %macro INITIALIZE or directly from each report;
*      It will check for snap shot comparison and supplemental domains
*      and merge in on the fly. 
*      Merge with delta folder only if ReviewNumber is greater than 0.
*      Calculate Study Day for each domain;
*      filter data set can be a file with path or just a file name;
*************************************************************************************;

/*
$CVS_START$
$ID: LoadData$
$CMP: RSYS.Load Data$ $UE: DM.USUBJID and xx.USUBJID$ 
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: xx.STUDYID$
$CMP: OPV.{"Merge supplemental domain","Yes"}$ $UE: xx.DOMAIN$
   $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$   
$CVS_STOP$

$CVS_START$
$ID: LoadDataForDM$
$CMP: RSYS.Load Data$ $UE: DM.USUBJID$ 
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: DM.STUDYID$
$CMP: OPV.{"Merge supplemental domain","Yes"}$ $UE: DM.DOMAIN$
   $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$   
$CVS_STOP$

$CVS_START$
$ID: LoadDataNxR$
$CMP: RSYS.Load Data$ $UE: DM.USUBJID$
$CMP: OSYS.Load Data$ $UE: xx.USUBJID$
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: xx.STUDYID$
$CMP: OPV.{"Merge supplemental domain","Yes"}$ $UE: xx.DOMAIN$
   $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$   
$CVS_STOP$

$CVS_START$
$ID: LoadDataO$
$CMP: OSYS.Load Data$ $UE: DM.USUBJID and xx.USUBJID$ 
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: xx.STUDYID$
$CMP: OPV.{"Merge supplemental domain","Yes"}$ $UE: xx.DOMAIN$
   $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$   
$CVS_STOP$

$CVS_START$
$ID: InterventionOccur$
$CMP: OSYS.Interventions Occurrence Check$ $UE: INTERVENTIONS.xxOCCUR$ 
   $NOTES: Removes records with xxOCCUR = N$
$CVS_STOP$

$CVS_START$
$ID: EventOccur$
$CMP: OSYS.Events Occurrence Check$ $UE: EVENTS.xxOCCUR$ 
   $NOTES: Removes records with xxOCCUR = N$
$CVS_STOP$
*/
 
%macro LOADDATA(domlist = ,mergesupp = &mergesuppdom, checkreview=1, checkForZeroObs = 0, _TempWhereStmt =, _TempDrop =, 
	_TempKeep =, _DoSort = 1, _Sortby =, _NarrFlag = 0, _StatFlag = 1, _DeltaFilter=);
   %put Entering LOADDATA for the following domains: &domlist;
   %ClockStart(LOADDATA);  
   %if ^%length(&domlist) %then %do;
      %put [MS_S_0188];
      %put ERROR: There is no domain to analyze with this report.;
      %let exiterror=1;
      %goto cexit;
   %end;

   %LocaleInit();
   
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _)); 
   %let domlist = %sysfunc(compbl(&domlist));

   %let trNew = %nrbquote(%LocaleGet(New_Text));
   %let trXReviewFlag = %nrbquote(%LocaleGet(XReviewFlag_Text));
   %let trDescriptionofPlannedArm = %nrbquote(%LocaleGet(DescriptionofPlannedArm_Text));
   %let trNameofActualTreatment = %nrbquote(%LocaleGet(NameofActualTreatment_Text));
   %let trDoseperAdministration = %nrbquote(%LocaleGet(DoseperAdministration_Text));
   %let trBlindedTreatment = %nrbquote(%LocaleGet(BlindedTreatment_Text));
   %let trDropped = %nrbquote(%LocaleGet(Dropped_Text));
   %let trNoOne = %nrbquote(%LocaleGet(N_Text));
   %let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));
       
   %if (%length(&adslpresent) and &ADSLBlind) or (^%length(&adslpresent) and &DMBlind) %then %let BlindedStudy = 1;
   %else %let BlindedStudy = 0;

   %let datekeep = "USUBJID";
   %if %sysfunc(findw(upcase(&ADSLVARS), RFSTDTC)) %then %let datekeep = &datekeep,"RFSTDTC";
   %else %if %sysfunc(findw(upcase(&ADSLVARS), TRTSDT)) %then %let datekeep = &datekeep,"TRTSDT";
   %if %sysfunc(findw(upcase(&ADSLVARS), RFENDTC)) %then %let datekeep = &datekeep,"RFENDTC"; 
   %else %if %sysfunc(findw(upcase(&ADSLVARS), TRTEDT)) %then %let datekeep = &datekeep,"TRTEDT";

   %let datekeep2 = USUBJID;
   %if %sysfunc(findw(upcase(&ADSLVARS), RFSTDTC)) %then %let datekeep2 = &datekeep2 RFSTDTC;
   %else %if %sysfunc(findw(upcase(&ADSLVARS), TRTSDT)) %then %let datekeep2 = &datekeep2 TRTSDT;
   %if %sysfunc(findw(upcase(&ADSLVARS), RFENDTC)) %then %let datekeep2 = &datekeep2 RFENDTC; 
   %else %if %sysfunc(findw(upcase(&ADSLVARS), TRTEDT)) %then %let datekeep2 = &datekeep2 TRTEDT; 
   
   %let _i_ = 1;
   %do %while(%length(%scan(&domlist,&_i_,' ')));
      %let temp = %scan(&domlist,&_i_,' ');  
      %let data = %scan(&&in_&temp, 2, ".");
      %let lib = %scan(&&in_&temp, 1, ".");  
      %put Domain = &temp;
      %let trxrf = %nrbquote(%LocaleFill(%nrbquote(&trXReviewFlag), f1=%nrbquote(&temp)));   
      %let dropblind =;
            
      %if %sysfunc(findw(%upcase(&domains), %upcase(&temp))) or %upcase(&temp) = ADSL %then %do;            
         *** transport xpt files when there is one ***;
         %if ^%sysfunc(exist(&&in_&temp)) %then %do;
            %let inDompath=%sysfunc(pathname(&lib));
            %let inDompath=&inDompath.\;
            %if %upcase(&lib) = ADAM %then %do;
            	%if %sysfunc(fileexist(&inDompath.AD&temp..xpt)) %then %do;
            		%XPTtoSASFile(AD&temp..xpt,&inDompath);
            	   %let in_&temp= AD&temp;
            	%end;
            %end;
          	%else %do;
            	%if %sysfunc(fileexist(&inDompath.&temp..xpt)) %then %do;
            	   %XPTtoSASFile(&temp..xpt,&inDompath);
            	   %let in_&temp= &temp;
					%end;
            %end;
         %end;
         
         %if ^%sysfunc(findw(&&&temp.VARS, %upcase(&temp)OCCUR)) %then %do;
            %let _TempWhereStmtApply = &_TempWhereStmt;           
         %end;
         %else %do;         
            %if %length(&_TempWhereStmt) %then %let _TempWhereStmtApply = %nrbquote((&_TempWhereStmt) and (&temp.occur not in ("N","&trNoOne")));
            %else %let _TempWhereStmtApply = %nrbquote(&temp.occur not in ("N","&trNoOne"));         
         %end;    
         
         *** remove unscheduled visits if requested ***;
         %if %symexist(RemoveUVs) %then %do;
            %if not(&ProcessName = MissingFindings and &temp = SV) %then %do;
               %if %upcase(&RemoveUVs) = YES and %sysfunc(findw(&&&temp.VARS, VISIT)) %then %do;               
                  %if ^%sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
                     %if %length(&_TempWhereStmtApply) %then %let _TempWhereStmtApply = %nrbquote((&_TempWhereStmtApply) and (upcase(VISIT) ^contains "UNSCHEDULED" and kupcase(VISIT) ^contains "%kupcase(&trUnscheduled)"));
                     %else %let _TempWhereStmtApply = %nrbquote(upcase(VISIT) ^contains "UNSCHEDULED" and kupcase(VISIT) ^contains "%kupcase(&trUnscheduled)");                         
                  %end;
                  %else %do;
                     %if %length(&_TempWhereStmtApply) %then %let _TempWhereStmtApply = %nrbquote((&_TempWhereStmtApply) and (upcase(AVISIT) ^contains "UNSCHEDULED" and kupcase(AVISIT) ^contains "%kupcase(&trUnscheduled)"));
                     %else %let _TempWhereStmtApply = %nrbquote(upcase(AVISIT) ^contains "UNSCHEDULED" and kupcase(AVISIT) ^contains "%kupcase(&trUnscheduled)");                                           
                  %end;
               %end;
            %end;
         %end;
         
         *** remove where status equals not done ***;         
         %if &_StatFlag %then %do;         
            %if %sysfunc(findw(&&&temp.VARS, &temp.STAT)) %then %do;
               %if %length(&_TempWhereStmtApply) %then %let _TempWhereStmtApply = %nrbquote((&_TempWhereStmtApply) and missing(&temp.STAT));
               %else %let _TempWhereStmtApply = %nrbquote(missing(&temp.STAT));
            %end;
         %end;
         
         %if &BlindedStudy %then %do;
            %if %upcase(&temp) = ADSL %then %do;
               %let _tid_ = %sysfunc(open(&in_adsl));
                  %if %sysfunc(varnum(&_tid_,ARM)) %then %let dropblind = &dropblind ARM;
                  %if %sysfunc(varnum(&_tid_,TRT01P)) %then %let dropblind = &dropblind TRT01P;
                  %if %sysfunc(varnum(&_tid_,TRT01A)) %then %let dropblind = &dropblind TRT01A;
                  %if %sysfunc(varnum(&_tid_,ACTARM)) %then %let dropblind = &dropblind ACTARM;
               %let _tid_ = %sysfunc(close(&_tid_));
            %end;
            %else %if %upcase(&temp) = DM %then %do;
               %let _tid_ = %sysfunc(open(&in_dm));
                  %if %sysfunc(varnum(&_tid_,ARM)) %then %let dropblind = &dropblind ARM;
                  %if %sysfunc(varnum(&_tid_,TRT01P)) %then %let dropblind = &dropblind TRT01P;
                  %if %sysfunc(varnum(&_tid_,TRT01A)) %then %let dropblind = &dropblind TRT01A;
                  %if %sysfunc(varnum(&_tid_,ACTARM)) %then %let dropblind = &dropblind ACTARM;
               %let _tid_ = %sysfunc(close(&_tid_));
            %end;            
            %else %if %upcase(&temp) = EX %then %do;
               %if &EXBlind %then %do;
                  %let _tid_ = %sysfunc(open(&in_ex));
                     %let doschk = %sysfunc(varnum(&_tid_,EXDOSE));
                     %let trtchk = %sysfunc(varnum(&_tid_,EXTRT));                  
                     %if &doschk and ^%sysfunc(findw(&&&temp.VARS, EXDOSE)) %then %let dropblind = &dropblind EXDOSE;
                     %if &trtchk and ^%sysfunc(findw(&&&temp.VARS, EXTRT)) %then %let dropblind = &dropblind EXTRT;
                  %let _tid_ = %sysfunc(close(&_tid_));
               %end;
            %end;            
         %end;
         
         %if %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
            %let _tid_ = %sysfunc(open(&&in_&temp));
            %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
               %let Datevarnum = %sysfunc(varnum(&_tid_,ADTM));
               %let DatevarLab = %sysfunc(varlabel(&_tid_,&Datevarnum));
            %end;
            %else %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADT)) %then %do;
               %let Datevarnum = %sysfunc(varnum(&_tid_,ADT));
            %end;
            %else %do;
               %let Datevarnum = 0;
            %end;
            %let DatevarLab = %sysfunc(varlabel(&_tid_,&Datevarnum));
            %let _tid_ = %sysfunc(close(&_tid_));
         %end;
         
         * get ASTDT and AENDT variable labels before renaming them;
         %if %length(&AInPath) & %sysfunc(findw(&_OccurDomains, &temp)) %then %do;
             %let _tid_ = %sysfunc(open(&&in_&temp));
             %if %sysfunc(findw(upcase(&&&temp.OccurVARS), ASTDT)) %then %do;
                %let ADatevarnum = %sysfunc(varnum(&_tid_,ASTDT));
                %let ASTDTvarLab = %sysfunc(varlabel(&_tid_,&ADatevarnum));
             %end;
             %if %sysfunc(findw(upcase(&&&temp.OccurVARS), AENDT)) %then %do;
                %let ADatevarnum = %sysfunc(varnum(&_tid_,AENDT));
                %let AENDTvarLab = %sysfunc(varlabel(&_tid_,&ADatevarnum));
             %end;
             %let _tid_ = %sysfunc(close(&_tid_));
         %end;
         
         *** _NarrFlag = 1 will first subset original domain to subjects with events of interest ***;
         *** Not used when bringing in AE domain in AENarrative in order to determine aesubs data set ***;
         %if &_NarrFlag %then %do;         
            data &temp;
               if 0 then set aesubs;
               if _n_ = 1 then do;
                  declare hash h(dataset:"aesubs");
                  h.defineKey("USUBJID");
                  h.defineData(ALL:"YES");
                  h.defineDone();
               end;
               set &&in_&temp;
               if h.find() = 0;
            run;            
            %let in_&temp = work.&temp;
            %put updated in_&temp = &&in_&temp;
         %end;

         *** keep records for ADSL subjects which applies filter data set, WhereStmt and gives references dates ***;
         *** apply _TempWhereStmt, _TempDrop and _TempKeep and apply blind ***;
         data &temp(compress=yes %if %length(&_TempKeep) %then keep=&_TempKeep; %if %length(&_TempDrop) %then drop=&_TempDrop;);
            if 0 then set &in_adsl(keep = &datekeep2);
            if _n_=1 then do;
               declare hash h(dataset:"&in_adsl");
               h.defineKey("USUBJID");
               h.defineData(&datekeep);
               h.defineDone();
            end;
            %if &BlindedStudy %then %do;
               %if %upcase(&temp) = DM or %upcase(&temp) = ADSL %then %do;
                  length ARM $ %length(&trBlindedTreatment);
               %end;
               %else %if %upcase(&temp) = EX and &EXBlind %then %do;
                  %if ^&trtchk or (&trtchk and ^%sysfunc(findw(&&&temp.VARS, EXTRT))) %then %do;
                     length EXTRT $ %length(&trBlindedTreatment);
                  %end;
               %end;
            %end;
            
            %if &ReviewNumber=0 & ^&ReviewsEnabled %then %do;
               %if %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
                  %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
                     length %sysfunc(tranwrd(xxDTC, xx, &temp)) $ 19;
                  %end;
                  %else %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADT)) %then %do;
                     length %sysfunc(tranwrd(xxDTC, xx, &temp)) $ 10;
                  %end;
               %end;
 					%if %length(&AInPath) & %sysfunc(findw(&_OccurDomains, &temp)) %then %do;
 						%if %sysfunc(findw(upcase(&&&temp.OccurVARS), ASTDT)) %then %do;
 							length %sysfunc(tranwrd(xxSTDTC, xx, &temp)) $ 10;
 						%end;
 						%if %sysfunc(findw(upcase(&&&temp.OccurVARS), AENDT)) %then %do;
 							length %sysfunc(tranwrd(xxENDTC, xx, &temp)) $ 10;
 						%end;
					%end;
            %end;                        
            %if %length(&_TempWhereStmtApply) %then %do;
               set &&in_&temp(where = (%unquote(&_TempWhereStmtApply)) %if %length(&dropblind) %then drop=&dropblind;) end = eof_e;
            %end;
            %else %do;
               set &&in_&temp(%if %length(&dropblind) %then drop=&dropblind;) end = eof_e;
            %end;
            if h.find() = 0;
            
            *** remove spaces from test codes ***;
            %if %sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
               paramcd = transtrn(strip(paramcd), " ", "_");
            %end;
            %else %if %sysfunc(findw(%upcase(&findingsdomains), &temp)) %then %do;
               &temp.testcd = transtrn(strip(&temp.testcd), " ", "_");
            %end;                  
            
            *** delta flag for reviewnumber 0 ***;
            %if &ReviewNumber=0 and &ReviewsEnabled %then %do;
               &temp.RFlg = "&trNew"; 
               label &temp.RFlg = "&trxrf";               
            %end;

            *** rename variables to accommodate BDS ***;
            %put &ReviewNumber;
            %put &ReviewsEnabled;
            %put &checkreview;
            
            %if (&ReviewNumber=0 and ^&ReviewsEnabled) or (&ReviewNumber>0 and (^&checkreview or ^%symexist(&&temp.keys))) %then %do;
               %if %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
                  %let _k_ = 1;
                  %do %while(%length(%scan(&_fromlist,&_k_,' ')));
                     %let tempf = %scan(&_fromlist,&_k_,' ');
                     %let tempt = %scan(&_tolist,&_k_,' ');
                     %let VarName = %sysfunc(tranwrd(&tempt, xx, &temp));
                     %if &tempf ne ADT %then %do;
                        %if %sysfunc(findw(upcase(&&&temp.BDSVARS), &tempf)) %then %do;
                           %if &tempf = ADTM %then %do;        
                              &VarName = put(&tempf,E8601DT19.); 
                              drop &tempf;
                              label &VarName = "&DatevarLab";
                           %end;
                           %else %do;	
                           	%if (%sysfunc(findw(upcase(&&&temp.BDSVARS), &Varname)) or %upcase(&tempf) = PARAM or %upcase(&tempf) = PARAMCD) %then %do;
                           		drop &Varname;
                           	%end;
                           	rename &tempf = &VarName; 
                           %end;                           
                        %end;             
                     %end;
                     %else %if &tempf = ADT %then %do;
                        %if ^%sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
                           &VarName = put(&tempf,E8601DA.);
                           drop &tempf;                         
                           label &VarName = "&DatevarLab";
                        %end;             
                     %end;                                        
                     %let _k_ = %eval(&_k_+1);      
                  %end;
               %end;
            %end;                       
                     
            %if %length(&AInPath) & %sysfunc(findw(&_OccurDomains, &temp)) %then %do;
            	*** for occurrence domains: rename following ADaM variables in domain datasets  ***;  
            	%let _k_ = 1;
					%do %while(%length(%scan(&_fromOccurlist,&_k_,' ')));
						%let tempf = %scan(&_fromOccurlist,&_k_,' ');
						%let tempt = %scan(&_toOccurlist,&_k_,' ');
						%let TempVarName = %sysfunc(tranwrd(&tempt, xx, &temp));
						%if %sysfunc(findw(upcase(&&&temp.OccurVARS), &tempf)) %then %do;
							%if &tempf = ASTDT or &tempf = AENDT %then %do;   
								if missing(&tempf) then do;
									&TempVarName ="";
								end;
								else do;
									&TempVarName = put(&tempf,E8601DA.); 
								end;
								drop &tempf;
								label &TempVarName = %if &tempf = ASTDT %then "&ASTDTvarLab"; %else "&AENDTvarLab";;
							%end;
							%else %do;	
								%if %sysfunc(findw(upcase(&&&temp.OccurVARS), &TempVarName)) %then %do;
									drop &TempVarName;
								%end;
								rename &tempf = &TempVarName; 
							%end;                           
						%end;             
                  %let _k_ = %eval(&_k_+1);      
					%end;
            %end;
            
            *** apply blind ***;
            %if &BlindedStudy %then %do;
               %if %upcase(&temp) = DM or %upcase(&temp) = ADSL %then %do;
                  %let &temp.VARS = &&&temp.VARS ARM;
                  ARM = "&trBlindedTreatment";
                  label arm = "&trDescriptionofPlannedArm";            
               %end;
               %else %if %upcase(&temp) = EX and &EXBlind %then %do;
                  %if ^&trtchk or (&trtchk and ^%sysfunc(findw(&&&temp.VARS, EXTRT))) %then %do;
                     %let &temp.VARS = &&&temp.VARS EXTRT;
                     EXTRT = "&trBlindedTreatment";
                     label EXTRT = "&trNameofActualTreatment";
                  %end;
                  %if ^&doschk or (&doschk and ^%sysfunc(findw(&&&temp.VARS, EXDOSE))) %then %do;
                     %let &temp.VARS = &&&temp.VARS EXDOSE;
                     EXDOSE = 1;
                     label EXDOSE = "&trDoseperAdministration";
                  %end;                              
               %end;
            %end;
            
            *** calculate study days ***;
            %calculateStudyDays(&temp);
         run;         

         %let in_&temp = work.&temp;
         %put updated in_&temp = &&in_&temp;

         *** merge delta ***;
         %if &ReviewNumber>0 and &checkreview and %symexist(&&temp.keys) %then %do;
            %let DeltaDir=%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder);
            libname DeltaLib "&DeltaDir";
            %if %sysfunc(fileexist(&DeltaDir.%lowcase(&temp).sas7bdat)) and %length(&&&temp.keys) %then %do;
               *** subset delta to ADSL subjects ***;
               data d&temp(compress=yes);
                  if 0 then set &in_adsl(keep = usubjid);
                  if _n_=1 then do;
                     declare hash h(dataset:"&in_adsl");
                     h.defineKey("USUBJID");
                     h.defineData("USUBJID");
                     h.defineDone();
                  end;
                  
                  %if %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
                     %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
                        length %sysfunc(tranwrd(xxDTC, xx, &temp)) $ 19;
                     %end;
                     %else %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADT)) %then %do;
                        length %sysfunc(tranwrd(xxDTC, xx, &temp)) $ 10;
                     %end;
                  %end;
                  %if %length(&AInPath) & %sysfunc(findw(&_OccurDomains, &temp)) %then %do;
                     %if %sysfunc(findw(upcase(&&&temp.OccurVARS), ASTDT)) %then %do;
                        length %sysfunc(tranwrd(xxSTDTC, xx, &temp)) $ 10;
                     %end;
                     %if %sysfunc(findw(upcase(&&&temp.OccurVARS), AENDT)) %then %do;
                        length %sysfunc(tranwrd(xxENDTC, xx, &temp)) $ 10;
                     %end;
                  %end;          
                  %if %kupcase(&_DeltaFilter) = %kupcase(%LocaleGet(New_Text)) or %kupcase(&_DeltaFilter) = %kupcase(%LocaleGet(Stable_Text)) %then %do;
                  	set DeltaLib.&temp(keep = &&&temp.keys &temp.RFlg where = ((%unquote(kupcase(&temp.RFlg) = "%kupcase(&_DeltaFilter)")) and (upcase(&temp.RFlg) ne "%upcase(&trDropped)"))) end = eof_list;
                  %end;
                  %else %do;
                  	set DeltaLib.&temp(keep = &&&temp.keys &temp.RFlg where = (upcase(&temp.RFlg) ne "%upcase(&trDropped)")) end = eof_list;
                  %end;
                  
                  *** remove spaces from test codes ***;
                  if h.find() = 0;
                  %if %sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
                     paramcd = transtrn(strip(paramcd), " ", "_");
                  %end;
                  %else %if %sysfunc(findw(%upcase(&findingsdomains), &temp)) %then %do;
                     &temp.testcd = transtrn(strip(&temp.testcd), " ", "_");
                  %end;
                  
                  *** rename variables to accommodate BDS ***;               
                  %if %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
                     %let _k_ = 1;
                     %do %while(%length(%scan(&_fromlist,&_k_,' ')));
                        %let tempf = %scan(&_fromlist,&_k_,' ');
                        %let tempt = %scan(&_tolist,&_k_,' ');
                        %let VarName = %sysfunc(tranwrd(&tempt, xx, &temp));
                        %if &tempf ne ADT %then %do;
                           %if %sysfunc(findw(upcase(&&&temp.BDSVARS), &tempf)) %then %do;
                              %if &tempf = ADTM %then %do;        
                                 &VarName = put(&tempf,E8601DT19.); 
                                 drop &tempf;
                                 label &VarName = "&DatevarLab";
                              %end;
                              %else %do;
											%if %sysfunc(findw(upcase(&&&temp.BDSVARS), &Varname)) %then %do;
												drop &Varname;
											%end;
											rename &tempf = &VarName; 
                              %end;                           
                           %end;             
                        %end;
                        %else %if &tempf = ADT %then %do;
                           %if ^%sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
                              &VarName = put(&tempf,E8601DA.);
                              drop &tempf;                         
                              label &VarName = "&DatevarLab";
                           %end;             
                        %end;                                        
                        %let _k_ = %eval(&_k_+1);      
                     %end;
                  %end;                           
                  
                  *** rename variables to accommodate Occurrence variables ***;  
						%if %length(&AInPath) & %sysfunc(findw(&_OccurDomains, &temp)) %then %do;
							%let _k_ = 1;
							%do %while(%length(%scan(&_fromOccurlist,&_k_,' ')));
								%let tempf = %scan(&_fromOccurlist,&_k_,' ');
								%let tempt = %scan(&_toOccurlist,&_k_,' ');
								%let TempVarName = %sysfunc(tranwrd(&tempt, xx, &temp));
								%if %sysfunc(findw(upcase(&&&temp.OccurVARS), &tempf)) %then %do;
									%if &tempf = ASTDT or &tempf = AENDT %then %do; 
										if missing(&tempf) then do;
											&TempVarName ="";
										end;
										else do;
											&TempVarName = put(&tempf,E8601DA.); 
										end;
										drop &tempf;
										label &TempVarName = %if &tempf = ASTDT %then "&ASTDTvarLab"; %else "&AENDTvarLab";;										
									%end;
									%else %do;	
										%if %sysfunc(findw(upcase(&&&temp.OccurVARS), &TempVarName)) %then %do;
											drop &TempVarName;
										%end;
										rename &tempf = &TempVarName; 
									%end;
								%end;
								%let _k_ = %eval(&_k_+1);
							%end;
						%end;
               run;  
               %let keyquote = %sysfunc(transtrn(&&&temp.keys, %str( ), %str(",")));
               data &temp.2(compress = yes);
                  if 0 then set &temp;
                  if _n_ = 1 then do;
                  declare hash h(dataset:"&temp");
                     h.defineKey("&keyquote");
                     h.defineData(ALL:"YES");
                     h.defineDone();
                  end;
                  set d&temp;
                  if h.find() = 0;
               run;
               
               proc datasets nolist;
                  delete &temp;
                  change &temp.2 = &temp;
               quit;               
            %end;
         %end;
         
         %if %upcase(&lib) = SDTM and &mergesupp = 1 and %sysfunc(findw(&suppdomains, %upcase(SUPP&temp))) %then %do;
            %put merging SUPP&temp;
            *** merge supp ***;         
            %let j = 1;
            %do %while(%length(%scan(&&&temp.SUPPIDVAR,&j,' ')));
               %let idvartemp = %scan(&&&temp.SUPPIDVAR,&j,' ');

               %if &j = 1 %then %let mergedata = &&in_&temp;
               %else %let mergedata = &temp;

               %if &idvartemp ne MISSING %then %do;
                  %let _tid_ = %sysfunc(open(&&in_&temp));
                     %let suppidchk = %sysfunc(varnum(&_tid_,&idvartemp));
                     %let SuppIDLabel = %sysfunc(varlabel(&_tid_,&suppidchk));
                  %let _tid_ = %sysfunc(close(&_tid_));
               %end;

               %if &idvartemp = MISSING %then %do;
                  proc sql noprint;
                     create table &temp.2(drop = USUBJID1 USUBJID2) as
                     select coalesce(aa.USUBJID1, bb.USUBJID2) as USUBJID label = "&SubjectLabel", aa.*, bb.*
                     from &mergedata(rename=(USUBJID = USUBJID1)) as aa
                     left join suppmeta.tsupp&temp._(rename=(USUBJID = USUBJID2)) as bb
                     on aa.USUBJID1 = bb.USUBJID2;
                  quit;
               %end;
               %else %do;
                  proc sql noprint;
                     create table &temp.2(drop = USUBJID1 USUBJID2 &idvartemp.1 &idvartemp.2) as
                     select coalesce(aa.USUBJID1, bb.USUBJID2) as USUBJID label = "&SubjectLabel", coalesce(aa.&idvartemp.1, bb.&idvartemp.2) as &idvartemp label = "&SuppIDLabel", aa.*, bb.* 
                     from &mergedata(rename=(USUBJID = USUBJID1 &idvartemp = &idvartemp.1)) as aa
                     left join suppmeta.tsupp&temp._&idvartemp(rename=(USUBJID = USUBJID2 &idvartemp = &idvartemp.2)) as bb
                     on aa.USUBJID1 = bb.USUBJID2 and aa.&idvartemp.1 = bb.&idvartemp.2;
                  quit;
               %end;

               %let &temp.VARS = &&&temp.VARS &&&temp.SUPPVARS;

               proc datasets nolist;
                  delete &temp;
                  change &temp.2 = &temp;
               quit;
               %let j = %eval(&j+1);
            %end;                             
         %end;
		 	%if &_DoSort %then %do;
				%if %length(&_Sortby) %then %do;
					proc sort data = &temp out = &temp;
						by &_Sortby;
					run;
				%end;
				%else %do;
					proc sort data = &temp out = &temp;
						by usubjid;
					run;         
				%end;
			%end;
                     
         *** kemicl: error check if loaded data has now valid rows for analysis ***;
         *** sasajf: make zero obs conditional (see argument list) ***;
         %if &checkForZeroObs %then %do;
            %nobsvars(&temp);
            %if &nobs = 0 %then %do;
               %put [MS_S_0190];
               %put ERROR: There are no valid observations in the &temp domain for analysis.  Please check that your study data sets or any applied subject filters are correct.;
               %let exiterror = 1;
            %end;
         %end;
      %end;
      
      %let _i_ = %eval(&_i_+1);        
   %end;   

   %cexit:;
   %put Exiting LOADDATA;
   %ClockStop(LOADDATA);     
%mend LOADDATA;   

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro REPLACEWORD(where = , word = , newword = );
   %let len = %length(&word);
   %let llen = %length(&&&where);
   %let first = %sysfunc(indexw(%upcase(&&&where), &word, %str( )+/-%str(*)=><));
   %let last = %eval(&first + &len - 1);

   %if &first = 1 %then %let &where = &newword%substr(&&&where, %eval(&last+1));
   %else %if &last = &llen %then %let &where = %substr(&&&where, 1, %eval(&first-1))&newword;
   %else %let &where = %substr(&&&where, 1, %eval(&first-1))&newword%substr(&&&where, %eval(&last+1)); 
%mend REPLACEWORD;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro INITIALIZE;
   %if &ProcessName ne DefineRiskThresholdDataSet %then %do;
      %put Entering INITIALIZE;
      %global _fromlist _tolist SpecialDomains _fromOccurlist _toOccurlist _OccurDomains;
      options nofmterr;
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %if &sysscp = WIN %then %do;
         %let _slash_ = \;
      %end;
      %else %do;
         %let _slash_ = /;
      %end;

      %LocaleInit();

      %if ^%symexist(Server) %then %let Server = Local;
      %include "&MacroPath./UtilityPathMacros.sas" / nosource;
      %let mbc = %jg_getPath(/user/clinical/metadata/base, CSTMacroVars.sas, 0, 0, 3, &studynameunder);
      %include "&mbc" / nosource;

      *** translate variable lists to accommodate BDS ***;
      %if %symexist(bdsdomains)=0 %then %let bdsdomains=;   
      %if %length(&bdsdomains) %then %do;
         %let i = 1;
         %let _fromlist = PARAM PARAMCD PARCAT1 AVISIT ADT ADTM ADY AVISITN ATPT ATPTN AVAL AVALC ANRLO ANRHI ABLFL ANRIND;
         %let _tolist = xxTEST xxTESTCD xxCAT VISIT xxDTC xxDTC xxDY VISITNUM xxTPT xxTPTNUM xxSTRESN xxSTRESC xxSTNRLO xxSTNRHI xxBLFL xxNRIND; 
         %do %while(%length(%scan(&bdsdomains,&i,' ')));
            %let temp = %scan(&bdsdomains,&i,' ');
            %global &temp.BDSVARS;
            %let &temp.BDSVARS = &&&temp.VARS;
               %let j = 1;
               %do %while(%length(%scan(&_fromlist,&j,' ')));
                  %let tempf = %scan(&_fromlist,&j,' ');
                  %let tempt = %scan(&_tolist,&j,' ');
                     %if &tempf ne ADT %then %do;
                        %if %sysfunc(findw(upcase(&&&temp.VARS), &tempf)) %then %do;                     
                           %CUTWORD(list = &&temp.VARS, word = &tempf);                      
                           %let &temp.VARS = &&&temp.VARS %sysfunc(tranwrd(&tempt, xx, &temp));             
                        %end;             
                     %end;
                     %else %if &tempf = ADT %then %do;
                        %if ^%sysfunc(findw(upcase(&&&temp.VARS), ADTM)) and %sysfunc(findw(upcase(&&&temp.VARS), &tempf)) %then %do;
                           %CUTWORD(list = &&temp.VARS, word = &tempf);
                           %let &temp.VARS = &&&temp.VARS %sysfunc(tranwrd(&tempt, xx, &temp));             
                        %end;             
                     %end;                                  
                  %let j = %eval(&j+1);      
               %end;
            %let i = %eval(&i+1);
         %end;

         *** modify where statements ***;
         %let wherestatelist = WhereTests FWhereStmtA FWhereStmtB FWhereStmtC;
         %let wheredomlist = FDomain FFDomA FFDomB FFDomC;

         %let i = 1;
         %do %while(%length(%scan(&wherestatelist,&i,' ')));
            %let tempwstate = %scan(&wherestatelist,&i,' '); 
            %let tempwdom = %scan(&wheredomlist,&i,' '); 

            %if %symexist(&tempwstate) %then %do;
               %if %length(&&&tempwstate) and %sysfunc(findw(upcase(&bdsdomains), &&&tempwdom))%then %do;
                  %put NOTE: Original &tempwstate &&&tempwstate;
                  *** add spaces around parentheses ***;
                  %let &tempwstate = %sysfunc(tranwrd(%nrbquote(&&&tempwstate), %str(%(), %str(%( )));
                  %let &tempwstate = %sysfunc(tranwrd(%nrbquote(&&&tempwstate), %str(%)), %str( %))));

                  %let j = 1;
                  %do %while(%length(%scan(&_fromlist,&j,' ')));
                     %let tempf = %scan(&_fromlist,&j,' ');
                      %let tempt = %scan(&_tolist,&j,' ');
                      %do %while(%sysfunc(indexw(%upcase(&&&tempwstate), &tempf,%str( )+/-%str(*)=)));
                         %let newvar = %sysfunc(tranwrd(&tempt, xx, &&&tempwdom)); 
                         %REPLACEWORD(where = &tempwstate, word = &tempf, newword = &newvar);                       
                      %end;             
                      %let j = %eval(&j+1);      
                  %end;
               %end;
               %put NOTE: Modified &tempwstate &&&tempwstate;
            %end;   
            %let i = %eval(&i+1);
         %end;      
      %end;  
      
      *** translate occurrence data variables and update variable varlist ***;		
      %let _fromOccurlist= ASTDT AENDT ASTDY AENDY ASEV AREL ATOXGR;
      %let _toOccurlist= xxSTDTC xxENDTC xxSTDY xxENDY xxSEV xxREL xxTOXGR; 
      %let _OccurDomains= AE CM MH CE;
      %let i = 1;
		%if %length(&AInPath) %then %do;		
			%do %while(%length(%scan(&_OccurDomains,&i,' ')));			
				%let temp = %scan(&_OccurDomains,&i,' ');
				%if %sysfunc(findw(&domains, &temp)) %then %do;
					%global &temp.OccurVARS;
            	%let &temp.OccurVARS = &&&temp.VARS;
					%let j = 1;
					%do %while(%length(%scan(&_fromOccurlist,&j,' ')));
						%let tempf = %scan(&_fromOccurlist,&j,' ');
						%let tempt = %scan(&_toOccurlist,&j,' ');
						%if %sysfunc(findw(upcase(&&&temp.VARS), &tempf)) %then %do;                     
							%CUTWORD(list = &&temp.VARS, word = &tempf);                      
							%let &temp.VARS = &&&temp.VARS %sysfunc(tranwrd(&tempt, xx, &temp));             
						%end;                                              
						%let j = %eval(&j+1);      
					%end;					
				%end;
				%let i = %eval(&i+1);
			%end;		
		%end;

      %if ^%sysfunc(findw(&domains, DM)) and %length(&adslpresent) %then %do;
         %let domains = &domains DM;
         %ALPHABETIZE(varlist = domains);
         %global in_DM;
         %let in_DM = &in_ADSL;
         %global DMVARS DMKEYS;
         %let DMVARS = &ADSLVARS;
         %let DMKEYS = ;
         /*
         %let DMKEYS = &ADSLKEYS;
         data deltalib.dm;
            set deltalib.adsl;
            rename ADSLRFlg = DMRFlg;
         run;
         */
      %end;

      %put NOTE: Available Domains: &domains;
      %put NOTE: Domains with Supplemental Qualifiers: &suppdomains;
      %put NOTE: Events Domains: &eventsdomains;
      %put NOTE: Findings Domains: &findingsdomains;
      %put NOTE: Interventions Domains: &interventionsdomains;
      %put NOTE: ADaM BDS Domains: &bdsdomains;
      %if %length(&adslpresent) %then %put NOTE: ADSL Available;
      %else %put NOTE: ADSL NOT Available;

      *** Unquote domain macros;
      %if %symexist(FDomain) %then %do;
         %let FDomain = %unquote(&FDomain);
      %end;
      %if %symexist(EventsDomain) %then %do;
         %let EventsDomain = %unquote(&EventsDomain);
      %end;
      %if %symexist(InterventionsDomain) %then %do;
         %let InterventionsDomain = %unquote(&InterventionsDomain);
      %end;

      %let i = 1;
      %do %while(%length(%scan(&domains,&i,' ')));
         %let temp = %scan(&domains,&i,' ');
         %put NOTE: in_&temp = &&in_&temp;
         %let i = %eval(&i+1);
      %end;
      %put NOTE: in_adsl = &in_adsl;

      *** Merge in Deltah Revue Flags ***;
      %if ^%symexist(ReviewsEnabled) %then %do;
         %global ReviewsEnabled;
         %let ReviewsEnabled=1;
      %end;

      title;
      footnote;
      options orientation = portrait;

      %global IconPath;
      %let IconPath = /install/lifesciences/documentation/icons/clinical/;
      
      %global IntDateFormat;
      %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(ja) or
         %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then
            %let IntDateFormat = DDMMYY10;
      %else %let IntDateFormat = DATE9;

      %put Exiting INITIALIZE;
   %end;
%mend INITIALIZE;
%macro RUNINITIALIZE;
   %if ^%symexist(runCDISCParmsFlag) %then %let runCDISCParmsFlag=0;
   %if ^&runCDISCParmsFlag %then %do;
      %INITIALIZE;
   %end;
%mend RUNINITIALIZE;
%RUNINITIALIZE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/*
$CVS_START$
$ID: TreatmentEmergentEvents$
$CMP: OPV.{"Event Type","Pre treatment events"}$
   $UE: EVENTS.xxTRTEM or EVENTS.TRTEMFL or (EVENTS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$
$CMP: CPV.{"Event Type","Treatment emergent events"}$
   $UE: EVENTS.xxTRTEM or EVENTS.TRTEMFL or (EVENTS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value of Y or Yes for flag variables. See parameter documentation for details of algorithm$
$CMP: CPV.{"Event Type","On treatment events"}$
   $UE: (EVENTS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Event start date and Treatment start date determine on treatment events, optionally use end dates.$
$CMP: OPV.{"Event Type","On treatment events"}$
   $UE: DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC$
   $NOTES: End dates used if available, otherwise everything after start date is assumed on treatment.$  
$CMP: CPV.{"Event Type","Off treatment follow up events"}$
   $UE: (EVENTS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC)
        and (DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC))$
   $NOTES: Treatment has to both start and stop to calculate off treatment events$
$CVS_STOP$


$CVS_START$
$ID: TreatmentEmergentInterventions$
$CMP: OPV.{"Intervention Type","Pre treatment interventions"}$
   $UE: INTERVENTIONS.xxTRTEM or INTERVENTIONS.TRTEMFL or (INTERVENTIONS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All interventions pre-treated if patient not treated$
$CMP: CPV.{"Intervention Type","Treatment emergent interventions"}$
   $UE: INTERVENTIONS.xxTRTEM or INTERVENTIONS.TRTEMFL or (INTERVENTIONS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value of Y or Yes for flag variables. See parameter documentation for details of algorithm$
$CMP: CPV.{"Intervention Type","On treatment interventions"}$
   $UE: (INTERVENTIONS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Intervention start date and Treatment start date determine on treatment events, optionally use end dates.$
$CMP: OPV.{"Intervention Type","On treatment interventions"}$
   $UE: DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC$
   $NOTES: End dates used if available, otherwise everything after start date is assumed on treatment.$  
$CMP: OPV.{"Intervention Type","Off treatment follow up interventions"}$
   $UE: (INTERVENTIONS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC)
        and (DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC))$
   $NOTES: Treatment has to both start and stop to calculate off treatment interventions$
$CVS_STOP$

$CVS_START$
$ID: OnTreatmentFindingsO$
$CMP: OSYS$ $UE: (FINDINGS.xxDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Findings start date and Treatment start date determine on treatment events, optionally use end dates.$
$CVS_STOP$

$CVS_START$
$ID: PreTreatmentFindingsO$
$CMP: OSYS$ $UE: FINDINGS.xxTRTEM or FINDINGS.TRTEMFL or (FINDINGS.xxDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$
$CVS_STOP$

$CVS_START$
$ID: PreTreatmentEventsO$
$CMP: OSYS$ $UE: EVENTS.xxTRTEM or EVENTS.TRTEMFL or (EVENTS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$$CVS_STOP$

$CVS_START$
$ID: PreTreatmentInterventionsO$
$CMP: OSYS$ $UE: INTERVENTIONS.xxTRTEM or INTERVENTIONS.TRTEMFL or (INTERVENTIONS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$$CVS_STOP$
*/

%macro TREATMENTEMERGENT(dataset = AE, domain = AE, date = , offset = 0, IgnoreFlag = NO, ExEndFlag = NO, ignoreRFdates = NO, useRFdates = NO);
   %let DateCalcVarSt = ;
   %let DateCalcDataSt =;
   %let DateCalcVarEn =;
   %let DateCalcDataEn = ;
   %let TEAEFlag = 0;   
   %global TEAEVARFLAG TEAEVARDOM TEAEVARFLAGs TEAEVARDOMs TEAEVARFLAGe TEAEVARDOMe;
   
   %let TEAEVARFLAG =;
   %let TEAEVARDOM =;
   %let TEAEVARFLAGs =;
   %let TEAEVARDOMs =;
   %let TEAEVARFLAGe =;
   %let TEAEVARDOMe =;
   
   %if %upcase(&IgnoreFlag) = NO %then %do;
      %if %sysfunc(findw(&&&domain.VARS, &domain.TRTEM)) %then %do;
         %let TEAEVARFLAG = &domain.TRTEM;
         %let TEAEVARDOM = &domain;
         data &dataset;
            set &dataset;
            if kupcase(&domain.TRTEM) in ("Y","YES",%unquote(%LocaleGetAll(Y_Text, _case = u)),%unquote(%LocaleGetAll(Yes_Text, _case = u))) then _TRTEMRG = 1;
            else _TRTEMRG = 0;          
         run;
         %let TEAEFlag = 1;
      %end;
      %else %if %sysfunc(findw(&&&domain.VARS, TRTEMFL)) %then %do;
         %let TEAEVARFLAG = TRTEMFL;
         %let TEAEVARDOM = &domain;
         data &dataset;
            set &dataset;
            if kupcase(TRTEMFL) in ("Y","YES",%unquote(%LocaleGetAll(Y_Text, _case = u)),%unquote(%LocaleGetAll(Yes_Text, _case = u))) then _TRTEMRG = 1;
            else _TRTEMRG = 0;          
         run;
         %let TEAEFlag = 1;
      %end;
   %end;
   %put TEAEFlag = &TEAEFlag;   
   
   *** Start Date ***;
   %if %upcase(&useRFdates) = NO %then %do; 
		%if %symexist(ADSLVARS) %then %do;
			%if %sysfunc(findw(&ADSLVARS, TRTSDTM)) %then %do;
				%let DateCalcVarSt = TRTSDTM;
				%let DateCalcDataSt = ADSL;
				%let TEAEVARDOMs = ADSL;
			%end;
			%else %if %sysfunc(findw(&ADSLVARS, TRTSDT)) %then %do;
				%let DateCalcVarSt = TRTSDT;
				%let DateCalcDataSt = ADSL;
				%let TEAEVARDOMs = ADSL;
			%end;
		%end;
		%if ^%length(&DateCalcVarSt) %then %do;
			%if %symexist(DMVARS) %then %do;
				%if %sysfunc(findw(&DMVARS, RFXSTDTC)) %then %do;
					%let DateCalcVarSt = RFXSTDTC;
					%let DateCalcDataSt = ADSL;
					%let TEAEVARDOMs = DM;					
				%end;
			%end;
		%end;
		%if ^%length(&DateCalcVarSt) %then %do;
			%if %symexist(EXVARS) %then %do;
				%if %sysfunc(findw(&EXVARS, EXSTDTC)) %then %do;
					%let DateCalcVarSt = EXSTDTC;
					%let DateCalcDataSt = EX;
					%let TEAEVARDOMs = EX;
				%end;
			%end; 
		%end; 
		%if (^%length(&DateCalcVarSt) and %upcase(&ignoreRFdates)= NO) %then %do;
			%if %symexist(DMVARS) %then %do;
				%if %sysfunc(findw(&DMVARS, RFSTDTC)) %then %do;
					%let DateCalcVarSt = RFSTDTC;
					%let DateCalcDataSt = ADSL;
					%let TEAEVARDOMs = DM;
				%end;
			%end;
		%end;
   %end;
   %else %do;
		%let DateCalcVarSt = RFSTDTC;
		%let DateCalcDataSt = ADSL;
		%let TEAEVARDOMs = DM;
   %end;		

	%if %upcase(&ignoreRFdates)= YES and ^%length(&DateCalcVarSt) %then %do;
		%put ERROR: Required treatment start date variables missing for determining events on Treatment.;
		%put ERROR: TRTSDTM, TRTSDT, RFXSTDTC, EXSTDTC, are not available.;
		%let exiterror = 1;
	%end;   

	%let TEAEVARFLAGs = &DateCalcVarSt;
	
	%if ^&TEAEFlag %then %do;
		%let TEAEVARFLAG = &DateCalcVarSt;
		%let TEAEVARDOM = &TEAEVARDOMs;
	%end;

	%if ^&TEAEFlag and ^%length(&DateCalcVarSt) %then %do;
		%put [MS_S_0191];
		%put ERROR: Treatment emergent flags or date variables missing for calculating event type.;
		%put ERROR: &domain.TRTEM, TRTEMFL, TRTSDTM, TRTSDT, RFXSTDTC, EXSTDTC, RFSTDTC are not available.;
		%let exiterror = 1;
	%end;

   
   %if %length(&DateCalcVarSt) %then %do;
      %put DateCalcVarSt = &DateCalcVarSt;
      %put DateCalcDataSt = &DateCalcDataSt;
   
      %if &DateCalcVarSt = EXSTDTC %then %do;
         /* check if it's a xpt file;*/
         %if %upcase(%scan(&in_ex, 1, ".")) = SDTM or %upcase(%scan(&in_ex, 1, ".")) = ADAM  %then %do; 
            %let inDompath=%sysfunc(pathname(%scan(&in_ex, 1, ".")));
            %let inDompath=&inDompath.\;
            %let inDomFile = %scan(&in_ex, 2, ".");
            %if %sysfunc(fileexist(&inDompath.&inDomFile..xpt)) %then %do; 
               %XPTtoSASFile(&inDomFile..xpt,&inDompath);
               %let in_ex=&inDomFile;
            %end;
         %end;

         proc sort data = &in_ex out = exst(keep = usubjid EXSTDTC);
            by usubjid EXSTDTC;
         run;

         data exst;
            set exst;
            by usubjid EXSTDTC;
            if first.usubjid;
         run;        
      %end;

      proc sql noprint;
         create table &dataset.2(drop = usubjid1 usubjid2) as
            select coalesce(aa.usubjid1, bb.usubjid2) as usubjid, aa.*, bb.*
            %if &DateCalcVarSt = EXSTDTC %then %do;
               from &dataset(rename=(usubjid=usubjid1)) as aa left join exst(keep = usubjid &DateCalcVarSt rename=(usubjid=usubjid2)) as bb
            %end;
            %else %do;
               from &dataset(rename=(usubjid=usubjid1)) as aa left join &&in_&DateCalcDataSt(keep = usubjid &DateCalcVarSt rename=(usubjid=usubjid2)) as bb
            %end;
            on aa.usubjid1 = bb.usubjid2 
            order by usubjid, &date;
      quit;

      proc datasets nolist;
         delete &dataset;
         change &dataset.2 = &dataset;
      quit;

      *** End Date ***;
   %if %upcase(&useRFdates) = NO %then %do;       
      %if %sysfunc(findw(&ADSLVARS, TRTEDTM)) %then %do;
         %let DateCalcVarEn = TRTEDTM;
         %let DateCalcDataEn = ADSL;
			%let TEAEVARDOMe = ADSL;                  
      %end;
      %else %if %sysfunc(findw(&ADSLVARS, TRTEDT)) %then %do;
         %let DateCalcVarEn = TRTEDT;
         %let DateCalcDataEn = ADSL;
			%let TEAEVARDOMe = ADSL;                  
      %end;
      %else %if %sysfunc(findw(&DMVARS, RFXENDTC)) %then %do;
         %let DateCalcVarEn = RFXENDTC;
         %let DateCalcDataEn = ADSL;
			%let TEAEVARDOMe = DM;                  
      %end;
      %else %if %sysfunc(findw(&EXVARS, EXENDTC)) %then %do;
         %let DateCalcVarEn = EXENDTC;
         %let DateCalcDataEn = EX;
			%let TEAEVARDOMe = EX;            
      %end;    
      %else %if %upcase(&ExEndFlag) = YES and %sysfunc(findw(&EXVARS, EXSTDTC)) %then %do;
         %let DateCalcVarEn = EXSTDTC;
         %let DateCalcDataEn = EX;
         %let TEAEVARDOMe = EX;         
      %end;
      %else %if (%sysfunc(findw(&DMVARS, RFENDTC)) and %upcase(&ignoreRFdates)= NO)  %then %do;
         %let DateCalcVarEn = RFENDTC;
         %let DateCalcDataEn = ADSL;
		 %let TEAEVARDOMe = DM;         
      %end;
   %end;
   %else %if %upcase(&ignoreRFdates)= NO %then %do;
      %if %sysfunc(findw(&ADSLVARS, RFENDTC)) %then %do;
         %let DateCalcVarEn = RFENDTC;
         %let DateCalcDataEn = ADSL;
		 %let TEAEVARDOMe = DM;         
      %end;
   %end;		      

      %put DateCalcVarEn = &DateCalcVarEn;
      %put DateCalcDataEn = &DateCalcDataEn;

      %let TEAEVARFLAGe = &DateCalcVarEn;
		
      %if %length(&DateCalcVarEn) %then %do;
         %if &DateCalcVarEn = EXENDTC or &DateCalcVarEn = EXSTDTC %then %do;
				/* check if it's a xpt file;*/
				%if %upcase(%scan(&in_ex, 1, ".")) = SDTM or %upcase(%scan(&in_ex, 1, ".")) = ADAM  %then %do; 
					%let inDompath=%sysfunc(pathname(%scan(&in_ex, 1, ".")));
					%let inDompath=&inDompath.\;
					%let inDomFile = %scan(&in_ex, 2, ".");
					%if %sysfunc(fileexist(&inDompath.&inDomFile..xpt)) %then %do; 
						%XPTtoSASFile(&inDomFile..xpt,&inDompath);
						%let in_ex=&inDomFile;
					%end;
				%end; 
				
            proc sort data = &in_ex(keep = usubjid &DateCalcVarEn) out = exen;
               by usubjid &DateCalcVarEn;
            run;

            %if &DateCalcVarEn = EXSTDTC %then %do;
               data exen(rename = (EXSTDTC = EXENDTC));
                  set exen;
                  by usubjid &DateCalcVarEn;
                  if last.usubjid;
               run;
            %end;
            %else %do;
               data exen;
                  set exen;
                  by usubjid &DateCalcVarEn;
                  if last.usubjid;
               run;                                    
            %end;
            %if &DateCalcVarEn = EXSTDTC %then %let DateCalcVarEn = EXENDTC;
         %end; 

         proc sql noprint;
            create table &dataset.2(drop = usubjid1 usubjid2) as
               select coalesce(aa.usubjid1, bb.usubjid2) as usubjid, aa.*, bb.*
               %if &DateCalcVarEn = EXENDTC %then %do;
                  from &dataset(rename=(usubjid=usubjid1)) as aa left join exen(keep = usubjid &DateCalcVarEn rename=(usubjid=usubjid2)) as bb
               %end;
               %else %do;
                  from &dataset(rename=(usubjid=usubjid1)) as aa left join &&in_&DateCalcDataEn(keep = usubjid &DateCalcVarEn rename=(usubjid=usubjid2)) as bb
               %end;
               on aa.usubjid1 = bb.usubjid2 
               order by usubjid, &date;
         quit;

         proc datasets nolist;
            delete &dataset;
            change &dataset.2 = &dataset;
         quit;
      %end;

      *** TEAE ***;
      *** only calculate TEAE if TEAE flag was not present ***;
      %if ^&TEAEFlag %then %do;
         data &dataset;
            set &dataset;  
            reftimefl = 0;
            datetimefl = 0;
            _TRTEMRG = 0;
            %if %length(&DateCalcVarSt) %then %do;
               *** if both dates do not have time component use only date part for treatment emergence ***;
               %if &DateCalcVarSt = TRTSDTM %then %do;
                  reftimefl = 1;
               %end;
               %else %if &DateCalcVarSt ne TRTSDT %then %do;
                  if index(&DateCalcVarSt,"T") then reftimefl = 1;
               %end;
               if index(&date,"T") then datetimefl = 1;

               if reftimefl = 1 and datetimefl = 1 then do;
                  %if &DateCalcVarSt = TRTSDTM %then %do;
                     refdate = TRTSDTM;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarSt) = 19 then 
   refdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),substr(&DateCalcVarSt,18,2));
                     else if lengthn(&DateCalcVarSt) = 16 then 
   refdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),"00");
                     else if lengthn(&DateCalcVarSt) = 13 then 
   refdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),"00","00");
                  %end;
                  if lengthn(&Date) = 19 then valdate = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),substr(&Date,18,2));
                  else if lengthn(&Date) = 16 then valdate = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),"00");
                  else if lengthn(&Date) = 13 then valdate = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),"00","00");         
               end;
               else do;
                  %if &DateCalcVarSt = TRTSDTM %then %do;
                     refdate = datepart(TRTSDTM);
                  %end;
                  %else %if &DateCalcVarSt = TRTSDT %then %do;
                     refdate = TRTSDT;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarSt) >= 10 then refdate = mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4));
                     else if lengthn(&DateCalcVarSt) = 7 then refdate = mdy(substr(&DateCalcVarSt, 6, 2), "01", substr(&DateCalcVarSt, 1, 4));
                     else if lengthn(&DateCalcVarSt) = 4 then refdate = mdy("01", "01", substr(&DateCalcVarSt, 1, 4));
                  %end;

                  if lengthn(&Date) >= 10 then valdate = mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4));
                  else if lengthn(&Date) = 7 then valdate = mdy(substr(&Date, 6, 2), "01", substr(&Date, 1, 4));
                  else if lengthn(&Date) = 4 then valdate = mdy("01", "01", substr(&Date, 1, 4));         
               end;

               if refdate > . then do;
                  *** AE and CE will be TE if equal to dose datetime or if date is missing for conservativeness ***;
                  %if &domain = AE or &domain = CE %then %do;
                     if (valdate >= refdate) or missing(valdate) then _TRTEMRG = 1;
                  %end;
                  %else %if &domain = CM %then %do;
                     if (valdate >= refdate) then _TRTEMRG = 1;
                  %end;
                  %else %do;
                     *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                     if (valdate > refdate) then _TRTEMRG = 1;
                  %end;
               end;
               drop refdate valdate;      
            %end;
            drop reftimefl datetimefl;
         run;
      %end;   

      *** OT and OTFU ***;
      data &dataset;
         set &dataset;  
         sttimefl = 0;
         entimefl = 0;
         datetimefl = 0;
         endate = .;
         valdateen = .;
         _OTFLAG = 0;
         _OTFUFLAG = 0;
         *** calculate if start date present ***;
         %if %length(&DateCalcVarSt) %then %do;
            *** will take most informative date for start and for stop separately incase time is present for stop or start ***;
            %if &DateCalcVarSt = TRTSDTM %then %do;
               sttimefl = 1;
            %end;
            %else %if &DateCalcVarSt ne TRTSDT %then %do;
               if index(&DateCalcVarSt,"T") then sttimefl = 1;
            %end;
				%if %length(&DateCalcVarEn) %then %do;
					%if &DateCalcVarEn = TRTEDTM %then %do;
						entimefl = 1;
					%end;
					%else %if &DateCalcVarEn ne TRTEDT %then %do;
						if index(&DateCalcVarEn,"T") then entimefl = 1;
					%end; 
				%end;
            if index(&date,"T") then datetimefl = 1;

            *** Start ***;
            if sttimefl = 1 and datetimefl = 1 then do;
               %if &DateCalcVarSt = TRTSDTM %then %do;
                  stdate = TRTSDTM;
               %end;
               %else %do;
                  if lengthn(&DateCalcVarSt) = 19 then 
   stdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),substr(&DateCalcVarSt,18,2));
                  else if lengthn(&DateCalcVarSt) = 16 then 
   stdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),"00");
                  else if lengthn(&DateCalcVarSt) = 13 then 
   stdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),"00","00");
               %end;
               if lengthn(&Date) = 19 then valdatest = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),substr(&Date,18,2));
               else if lengthn(&Date) = 16 then valdatest = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),"00");
               else if lengthn(&Date) = 13 then valdatest = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),"00","00");         
            end;
            else do;
               %if &DateCalcVarSt = TRTSDTM %then %do;
                  stdate = datepart(TRTSDTM);
               %end;
               %else %if &DateCalcVarSt = TRTSDT %then %do;
                  stdate = TRTSDT;
               %end;
               %else %do;
                  if lengthn(&DateCalcVarSt) >= 10 then stdate = mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4));
                  else if lengthn(&DateCalcVarSt) = 7 then stdate = mdy(substr(&DateCalcVarSt, 6, 2), "01", substr(&DateCalcVarSt, 1, 4));
                  else if lengthn(&DateCalcVarSt) = 4 then stdate = mdy("01", "01", substr(&DateCalcVarSt, 1, 4));
               %end;
               if lengthn(&Date) >= 10 then valdatest = mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4));
               else if lengthn(&Date) = 7 then valdatest = mdy(substr(&Date, 6, 2), "01", substr(&Date, 1, 4));
               else if lengthn(&Date) = 4 then valdatest = mdy("01", "01", substr(&Date, 1, 4));         
            end;

            %if %length(&DateCalcVarEn) %then %do;
               *** End ***;
               if entimefl = 1 and datetimefl = 1 then do;
                  %if &DateCalcVarEn = TRTEDTM %then %do;
                     endate = TRTEDTM;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarEn) = 19 then 
   endate = dhms(mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4)),substr(&DateCalcVarEn,12,2),substr(&DateCalcVarEn,15,2),substr(&DateCalcVarEn,18,2));
                     else if lengthn(&DateCalcVarEn) = 16 then 
   endate = dhms(mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4)),substr(&DateCalcVarEn,12,2),substr(&DateCalcVarEn,15,2),"00");
                     else if lengthn(&DateCalcVarEn) = 13 then 
   endate = dhms(mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4)),substr(&DateCalcVarEn,12,2),"00","00");
                  %end;
                  if lengthn(&Date) = 19 then valdateen = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),substr(&Date,18,2));
                  else if lengthn(&Date) = 16 then valdateen = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),"00");
                  else if lengthn(&Date) = 13 then valdateen = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),"00","00");         
               end;
               else do;
                  %if &DateCalcVarEn = TRTEDTM %then %do;
                     endate = datepart(TRTEDTM);
                  %end;
                  %else %if &DateCalcVarEn = TRTEDT %then %do;
                     endate = TRTEDT;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarEn) >= 10 then endate = mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4));
                     else if lengthn(&DateCalcVarEn) = 7 then endate = mdy(substr(&DateCalcVarEn, 6, 2), "01", substr(&DateCalcVarEn, 1, 4));
                     else if lengthn(&DateCalcVarEn) = 4 then endate = mdy("01", "01", substr(&DateCalcVarEn, 1, 4));
                  %end;
                  if lengthn(&Date) >= 10 then valdateen = mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4));
                  else if lengthn(&Date) = 7 then valdateen = mdy(substr(&Date, 6, 2), "01", substr(&Date, 1, 4));
                  else if lengthn(&Date) = 4 then valdateen = mdy("01", "01", substr(&Date, 1, 4));         
               end;
            %end;

            if stdate > . and missing(endate) then do;
               *** AE and CE will be OT if equal to dose datetime or if date is missing for conservativeness ***;
               %if &domain = AE or &domain = CE %then %do;
                  if (valdatest >= stdate) or missing(valdatest) then _OTFLAG = 1;
               %end;
               %else %if &domain = CM %then %do;
                  if (valdatest >= stdate) then _OTFLAG = 1;
               %end;
               %else %do;
                  *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                  if (valdatest > stdate) then _OTFLAG = 1;
               %end;
            end;
            else if stdate > . and endate > . then do;
               *** if enddate has time then need to convert offset ***; 
               if entimefl = 1 and datetimefl = 1 then do;
                  *** AE and CE will be OT if equal to dose datetime or if date is missing for conservativeness ***;
                  %if &domain = AE or &domain = CE %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+%eval(&offset*86400))) or missing(valdatest) then _OTFLAG = 1;
                     if (valdateen > endate+%eval(&offset*86400)) then _OTFUFLAG = 1;
                  %end;
                  %else %if &domain = CM %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+%eval(&offset*86400))) then _OTFLAG = 1;
                     if (valdateen > endate+%eval(&offset*86400)) then _OTFUFLAG = 1;
                  %end;
                  %else %do;
                     *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                     if ((valdatest > stdate) and (valdateen <= endate+%eval(&offset*86400))) then _OTFLAG = 1;
                     if (valdateen > endate+%eval(&offset*86400)) then _OTFUFLAG = 1;
                  %end;
               end;
               else do;
                  *** AE and CE will be OT if equal to dose datetime or if date is missing for conservativeness ***;
                  %if &domain = AE or &domain = CE %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+&offset)) or missing(valdatest) then _OTFLAG = 1;
                     if (valdateen > endate+&offset) then _OTFUFLAG = 1;
                  %end;
                  %else %if &domain = CM %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+&offset)) then _OTFLAG = 1;
                     if (valdateen > endate+&offset) then _OTFUFLAG = 1;
                  %end;
                  %else %do;
                     *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                     if ((valdatest > stdate) and (valdateen <= endate+&offset)) then _OTFLAG = 1;
                     if (valdateen > endate+&offset) then _OTFUFLAG = 1;
                  %end;          
               end;           
            end;
            drop stdate valdatest endate valdateen;
         %end;
         drop entimefl sttimefl datetimefl;
      run;     
   %end;
%mend TREATMENTEMERGENT;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* Assign domain grouping data set macro variable;
%macro GroupAssign(domain);
   %let domain = %lowcase(&domain);
   %global ing_&domain;
   %let ing_&domain = ;
   %if %length(&AInPath) %then %do;
      %if %sysfunc(fileexist("&AInPath./ad&domain.grp.sas7bdat")) %then %do;
         %let ing_&domain = adam.ad&domain.grp;
      %end;
      %else %if %sysfunc(fileexist("&AInPath./ad&domain.grp.xpt")) %then %do;
         %XPTtoSASFile(ad&domain.grp.xpt,&AInPath);
         %let ing_&domain = work.ad&domain.grp;
      %end;
   %end;
   %if ^%length(&&ing_&domain) and %length(&InPath) %then %do;
      %if %sysfunc(fileexist("&InPath./&domain.grp.sas7bdat")) %then %do;
         %let ing_&domain = sdtm.&domain.grp;
      %end;
      %else %if %sysfunc(fileexist("&InPath./&domain.grp.xpt")) %then %do;
         %XPTtoSASFile(&domain.grp.xpt,&InPath);
         %let ing_&domain = work.&domain.grp;
      %end;
   %end;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/*
$CVS_START$
$ID: TreatmentUse$
$CMP: CPV.{"Treatment or Comparison Variable to Use","Actual"}$
   $UE: DM.TRT01A or DM.ACTARM or DM.TRT01P or DM.ARM$
   $NOTES: Will attempt to use Planned treatment variables if Actual treatment variables not found.
      If ARM is missing, it will be created and set to "Blinded Treatment" so that certain reports
      can still be run.$
$CMP: CPV.{"Treatment or Comparison Variable to Use","Planned"}$
   $UE:DM.TRT01P or DM.ARM$
   $NOTES: If ARM is missing, it will be created and set to "Blinded Treatment" so that certain reports
      can still be run.$
$CVS_STOP$

$CVS_START$
$ID: PopulationFlags$
$CMP: CPV.{"Select the analysis population","Safety"}$
   $UE: DM.SAFFL or DM.SAFETY$
   $NOTES: SAFETY may be used in DM Supp.$
$CMP: OPV.{"Select the analysis population","All Subjects Excluding Screen Failures"}$
   $UE: DM.ARM$
   $NOTES: Records with Screen Failure or SCRNFAIL values dropped.  All subjects used if ARM not available$
$CMP: CPV.{"Select the analysis population","Full Analysis Set"}$
   $UE: DM.FASFL or DM.FULLSET$
   $NOTES: FULLSET may be used in DM Supp.$
$CMP: CPV.{"Select the analysis population","Intent-to-Treat"}$
   $UE: ADSL.ITTFL or ADSL.ITT$   
$CMP: CPV.{"Select the analysis population","Per-Protocol"}$
   $UE: DM.PPROTFL or DM.PPROT$
   $NOTES: PPROT may be used in DM Supp.$
$CMP: CPV.{"Select the analysis population","Completers"}$
   $UE: DM.COMPFL or DM.COMPLETED or DM.COMPLT$
   $NOTES: COMPLETED or COMPLT may be used in DM Supp.$
$CMP: CPV.{"Select the analysis population","Randomized"}$
   $UE: DM.RANDFL or DM.RANDOMIZED or DM.RANDOM$
   $NOTES: RANDOMIZED or RANDOM may be used in DM Supp.$
$CMP: CPV.{"Select the analysis population","Enrolled"}$
   $UE: DM.ENRLFL or DM.ENROLLED or DM.ENRL$
   $NOTES: ENROLLED or ENRL may be used in DM Supp.$
$CVS_STOP$
*/

%macro PREPADSL(_ProfileFlag = 0, _ConvertEndDateFlag =1);
   %global studynameunder RFStartDateConverted RFEndDateConverted;
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));

   %LocaleInit();

   %let trFull = %nrbquote(%LocaleGet(Full_Text));
   %let trFullAnalysisSet = %nrbquote(%LocaleGet(FullAnalysisSet_Text));
   %let trSafety = %nrbquote(%LocaleGet(Safety_Text));
   %let trIntent = %nrbquote(%LocaleGet(Intent_Text));
   %let trIntenttoTreat = %nrbquote(%LocaleGet(IntenttoTreat_Text));
   %let trProtocol = %nrbquote(%LocaleGet(Protocol_Text));
   %let trPerProtocol = %nrbquote(%LocaleGet(PerProtocol_Text));
   %let trComplete = %nrbquote(%LocaleGet(Complete_Finished_Text));
   %let trCompleters = %nrbquote(%LocaleGet(Completers_Text));
   %let trRandom = %nrbquote(%LocaleGet(Random_Text));
   %let trRandomized = %nrbquote(%LocaleGet(Randomized_Text));
   %let trEnroll = %nrbquote(%LocaleGet(Enroll_Text));
   %let trEnrolled = %nrbquote(%LocaleGet(Enrolled_Text));
   %let trAllSubjects = %nrbquote(%LocaleGet(AllSubjects_Text));
   %let trScreenFailure = %nrbquote(%LocaleGet(ScreenFailure_Text));
     
   * check if it is xpt file;
   %if ^%sysfunc(exist(&in_adsl)) %then %do;
      %let inDompath=%sysfunc(pathname(%scan(&in_adsl, 1, ".")));
      %let inDompath=&inDompath.\;
      %let inDomFile = %scan(&in_adsl, 2, ".");
      %if %sysfunc(fileexist(&inDompath.&inDomFile..xpt)) %then %do; 
         %XPTtoSASFile(&inDomFile..xpt,&inDompath);
         %let in_adsl = &inDomFile;
      %end;
   %end;
   %let _tid_ = %sysfunc(open(&in_adsl));
      %let usubjvn = %sysfunc(varnum(&_tid_,USUBJID));
      %if ^&usubjvn %then %do;
         %put [MS_S_0193];
         %put ERROR: Variable USUBJID not found in adsl data set.;
         %let exiterror = 1;
         %goto cexit;
      %end;
      %else %do;
         %global SubjectLabel;
         %let SubjectLabel = %sysfunc(varlabel(&_tid_,&usubjvn));
         %if ^%length(%trim(&SubjectLabel)) %then %let SubjectLabel = USUBJID;
         %put SubjectLabel = &SubjectLabel;
      %end;      
   %let _tid_ = %sysfunc(close(&_tid_));

	* merge with derived population flags;
	%global POPFLLIB;
	%let POPFLLIB=%jg_getPath(/user/clinical/populationflags,,1,0,1,&studynameunder); 
	libname popfllib "&POPFLLIB";
     
	%if %sysfunc(fileexist(&POPFLLIB.derivedPopFlags.sas7bdat)) %then %do;
		proc sql noprint;
			create table adsl as
			select * from &in_adsl a1 left join popfllib.derivedpopflags a2
			on a1.USUBJID=a2.USUBJID;
		quit;
		%let in_adsl = work.adsl;
		%put merged in prop flags;         
	%end;
		
   *** assign group data sets, to be used later for sorting ***;
   %let _i_ = 1;
   %do %while(%length(%scan(&findingsdomains,&_i_,' ')));
      %let temp = %scan(&findingsdomains,&_i_,' ');
      %GroupAssign(&temp);
      %let _i_ = %eval(&_i_+1);
   %end;

   %let _tid_ = %sysfunc(open(&in_adsl));
   %if ^&_tid_ %then %do;
      %put [MS_S_0192];
      %put ERROR: Subject data not found. The study may not exist on the active server.;
      %let exiterror = 1;
      %goto cexit;
   %end;
   %else %do;
      *** determine treatment variable to use ***;
      %global TrtName TrtLabel;
      %let TrtName = ;
      %let TrtLabel = ;
      %let tvn = 0;
      %if ^%symexist(TrtVar) %then %let TrtVar = ;
      %if %length(&TrtVar) %then %do;
         %let TrtName = &TrtVar;
         %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
      %end;
      %else %do;
         %if ^%symexist(TrtUse) %then %do;
            %global TrtUse;
            %let TrtUse = Actual;
         %end;
         %if (&TrtUse = Actual) %then %do;
            %let TrtName = TRT01A;
         %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
            %if ^&tvn %then %do;
               %let TrtName = ACTARM;              
               %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
            %end;
         %end;
         %if ^&tvn %then %do;
            %if (&TrtUse = Actual) %then %do;
               %put WARNING: Actual treatment variable not found, so using planned treatment variable.;
            %end;
            %let TrtName = TRT01P;
         %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
            %if ^&tvn %then %do;
               %let TrtName = ARM;
               %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
            %end;
         %end;
      %end;

      %let TrtLabel = &TrtName;
      %CheckLabel(&in_adsl,TrtLabel);
      %put TrtName = &TrtName;
      %put TrtLabel = &TrtLabel;

      %let tflag = ;
      %if &TrtName = TRT01A %then %do;
         %let tflag = A;
      %end;
      %else %if &TrtName = TRT01P %then %do;
         %let tflag = P;
      %end;

      /*
      $CVS_START$
      $ID: CrossoverPeriods$
      $CMP: OSYS.Treatment Crossover Detection$
         $UE: (DM.TRT02P or DM.TRT02A) and (DM.TRT02SDTM or DM.TRT02SDT)$
         $NOTES: Creates Treatment and Period variables to use in analysis to support crossover trial. 
         Handles more than 2 periods by presence of TRTxxP or TRTxxA and corresponding period date variables.$
      $CVS_STOP$
      */

      *** Macros to check for Crossover treatment periods ***;
      %global NTrtPeriods TrtVarList TrtStartDateList MaxTrtVarLen COflag PMBase;
      %let NTrtPeriods = 1;
      %let COflag = ;
      %let PMBase = ;
      %if %length(&tflag) %then %do;
      %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
         %let TrtVarList = ;
         %do %while(&tvn);
            %let TrtVarList = &TrtVarList TRT0&NTrtPeriods.&tflag;
            %global TrtVarLabel&NTrtPeriods;
            %let TrtVarLabel&NTrtPeriods = TRT0&NTrtPeriods.&tflag;
      	    %CheckLabel(&in_adsl,TrtVarLabel&NTrtPeriods);
      	    %put Treatment Label = &&TrtVarLabel&NTrtPeriods;
            %let NTrtPeriods = %eval(&NTrtPeriods +1);            
            %let tvn = %sysfunc(varnum(&_tid_,TRT0&NTrtPeriods.&tflag));
         %end;
         %let NTrtPeriods = %eval(&NtrtPeriods-1);
      %end;
      %else %do;
      	%global TrtVarLabel1;
         %let TrtVarList = &TrtName;
         %let TrtVarLabel1 = &TrtLabel;
      %end;
      %put Number of Treatment Period Variables = &NTrtPeriods;
      %put Treatment Variables = &TrtVarList;

      %if &NTrtPeriods > 1 %then %do;
         %let NTrtPeriods = 1;
         %Global dtsuffix;
         *** try TR01SDTM ***;
         %let dtsuffix = ;
         %let trtsdtmnum = %sysfunc(findw(%upcase(&ADSLVARS), TR01SDTM));
         %if &trtsdtmnum=0 %then %do;
         *** try TR01SDT ***;
         %let trtsdtnum = %sysfunc(findw(%upcase(&ADSLVARS), TR01SDT));
            %if &trtsdtnum>0 %then %do;
               %let dtsuffix = sdt;
            %end;
         %end;
         %else %do;
            %let dtsuffix = sdtm;
         %end;

         %if %length(&dtsuffix) %then %do;
            %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), %upcase(TR01&dtsuffix)));
            %let TrtStartDateList = ;
            %do %while(&tvn);
               %let TrtStartDateList = &TrtStartDateList TR0&NTrtPeriods.&dtsuffix;
               %let NTrtPeriods = %eval(&NTrtPeriods +1);
               %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), %upcase(TR0&NTrtPeriods.&dtsuffix)));
            %end;
            %let NTrtPeriods = %eval(&NtrtPeriods-1);
         %end;
         %put Number of CrossOver Start Dates = &NTrtPeriods;
         %put Treatment Start Date Variables = &TrtStartDateList;
      %end;
      %put Number of Treatment Period Variables to be Used = &NTrtPeriods;
      /**********************
      If NTrtPeriods is greater than 1 above, we detect a crossover trial and use the 
      start date list and treatment list to create a new conditional treatment variable in 
      stacked time data.  See the %AssignCOTreatment macro below.
      ***********************/

      %let MaxTrtVarlen = 0;
      %do m = 1 %to &NTrtPeriods;
         %let curt = %scan(&TrtVarList,&m," ");
         %let tnum = %sysfunc(varnum(&_tid_,&curt));
         %let curlen = %sysfunc(varlen(&_tid_,&tnum));
         %if &curlen > &MaxTrtVarlen %then %let MaxTrtVarlen = &curlen;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
   %end;
   
 	* Map TRTSDT to RFSTDTC and TRTEDT to RFENDTC if RFSTDTC/RFENDTC does not exist in adsl;
	/* add RFSTDTC to var list;*/
	%let RFStartDateConverted = 0;
	%let RFEndDateConverted = 0;
	%if ^%sysfunc(findw(upcase(&ADSLVARS), RFSTDTC)) & %sysfunc(findw(upcase(&ADSLVARS), TRTSDT)) %then %do;
		%let ADSLVARS = &ADSLVARS RFSTDTC; 
		%let RFStartDateConverted=1;
		data &in_adsl;
			set &in_adsl;
			RFSTDTC = put(TRTSDT,E8601DA.);
		run;
	%end;
	%if &_ConvertEndDateFlag %then %do;
		* add RFENDTC to var list;
		%if ^%sysfunc(findw(upcase(&ADSLVARS), RFENDTC)) & %sysfunc(findw(upcase(&ADSLVARS), TRTEDT)) %then %do;
			%let ADSLVARS = &ADSLVARS RFENDTC; 
			%let RFEndDateConverted=1;
			data &in_adsl;
				set &in_adsl;
				RFENDTC = put(TRTEDT,E8601DA.);
			run; 
		%end;
	%end;
   *** determine population ***;
   %let IncSF = No;
   %if %kindex(%quote(%kupcase(&PopulationSet)),FULL) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trFull))) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trFullAnalysisSet))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,FASFL)) %then %let PopulationVar = FASFL;
         %else %if %sysfunc(findw(&ADSLVARS,FULLSET)) %then %let PopulationVar = FULLSET;
         %else %do;       
            %put ERROR: The variables FASFL and FULLSET were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end;
   %else %if %kindex(%quote(%kupcase(&PopulationSet)),SAFETY) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trSafety))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,SAFFL)) %then %let PopulationVar = SAFFL;
         %else %if %sysfunc(findw(&ADSLVARS,SAFETY)) %then %let PopulationVar = SAFETY;
         %else %do;       
            %put ERROR: The variables SAFFL and SAFETY were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end;
   %else %if %kindex(%quote(%kupcase(&PopulationSet)),INTENT) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trIntent))) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trIntenttoTreat))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,ITTFL)) %then %let PopulationVar = ITTFL;
         %else %if %sysfunc(findw(&ADSLVARS,ITT)) %then %let PopulationVar = ITT;
         %else %do;       
            %put ERROR: The variables ITTFL and ITT were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end;
   %else %if %kindex(%quote(%kupcase(&PopulationSet)),PROTOCOL) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trProtocol))) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trPerProtocol))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,PPROTFL)) %then %let PopulationVar = PPROTFL;
         %else %if %sysfunc(findw(&ADSLVARS,PPROT)) %then %let PopulationVar = PPROT;
         %else %do;       
            %put ERROR: The variables PPROTFL and PPROT were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end;
   %else %if %kindex(%quote(%kupcase(&PopulationSet)),COMPLETE) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trComplete))) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trCompleters))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,COMPLFL)) %then %let PopulationVar = COMPLFL;
         %else %if %sysfunc(findw(&ADSLVARS,COMPFL)) %then %let PopulationVar = COMPFL;
         %else %if %sysfunc(findw(&ADSLVARS,COMPLT)) %then %let PopulationVar = COMPLT;
         %else %if %sysfunc(findw(&ADSLVARS,COMPLETED)) %then %let PopulationVar = COMPLETED;
         %else %do;       
            %put ERROR: The variables COMPLFL, COMPFL, COMPLT and COMPLETED were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end; 
   %else %if %kindex(%quote(%kupcase(&PopulationSet)),RANDOM) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trRandom))) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trRandomized))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,RANDFL)) %then %let PopulationVar = RANDFL;
         %else %if %sysfunc(findw(&ADSLVARS,RAND)) %then %let PopulationVar = RAND;
	 %else %if %sysfunc(findw(&ADSLVARS,RANDOMIZED)) %then %let PopulationVar = RANDOMIZED;
         %else %if %sysfunc(findw(&ADSLVARS,RANDOM)) %then %let PopulationVar = RANDOM;
         %else %do;       
            %put ERROR: The variables RANDFL, RAND, RANDOMIZED, and RANDOM were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end; 
   %else %if %kindex(%quote(%kupcase(&PopulationSet)),ENROLL) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trEnroll))) or
      %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trEnrolled))) %then %do;
         %if %sysfunc(findw(&ADSLVARS,ENRLFL)) %then %let PopulationVar = ENRLFL;
	 %else %if %sysfunc(findw(&ADSLVARS,ENRL)) %then %let PopulationVar = ENRL;
	 %else %if %sysfunc(findw(&ADSLVARS,ENROLLED)) %then %let PopulationVar = ENROLLED;
         %else %do;       
            %put ERROR: The variables ENRLFL, ENRL, and ENROLLED were not found in the adsl data set.;
            %let exiterror = 1;
            %goto cexit;
         %end;       
   %end; 
   %else %if %quote(%kupcase(&PopulationSet)) = %quote(%kupcase(All Subjects)) or
      %quote(%kupcase(&PopulationSet)) = %quote(%kupcase(&trAllSubjects)) %then %do;
         %let IncSF = Yes;
   %end;

   %let popval =;
   %if ^%symexist(PopulationVar) %then %let PopulationVar=;
   %if %length(&PopulationVar) %then %do;
      %let _tid_ = %sysfunc(open(&in_adsl));
      %let pvnum = %sysfunc(varnum(&_tid_,&PopulationVar));
      %let pvtype = %sysfunc(vartype(&_tid_,&pvnum));     
      %let _tid_ = %sysfunc(close(&_tid_));      
      %if &pvtype = N %then %let popval = %bquote(1);
      %else %if &pvtype = C %then %let popval = %bquote("Y");            
   %end;    

   %if ^%symexist(WhereStmt) %then %let WhereStmt=;
   %if ^%symexist(WhereStmtDisplay) %then %let WhereStmtDisplay=;
   %if ^%symexist(SubjectFilter) %then %let SubjectFilter=;
   %put IncSF=&IncSF PopulationVar=&PopulationVar WhereStmt=&WhereStmt SubjectFilter=&SubjectFilter;    

   %if ^%symexist(PopulationVar) or ^%length(&PopulationVar) %then %do;
      %if %length(&WhereStmtDisplay) %then %let WhereStmtDisplay = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XandX_Text)),
        f1=%nrbquote(&WhereStmtDisplay), f2=%nrbquote(%LocaleEZX(%nrbquote(&PopulationSet), EIA=1))));
      %else %let WhereStmtDisplay = %nrbquote(%LocaleEZX(%nrbquote(&PopulationSet), EIA=1));      
   %end;

   %if (&IncSF = No) or %length(&PopulationVar) or %length(&WhereStmt) or %length(&SubjectFilter) or &_ProfileFlag %then %do;      
      *** add population to where statement ***;
      %if %length(&PopulationVar) %then %do;
         %if %length(&WhereStmt) %then %let WhereStmt = %nrbquote((&WhereStmt) and (&PopulationVar = &popval));
         %else %let WhereStmt = %nrbquote(&PopulationVar = &popval);
         %if %length(&WhereStmtDisplay) %then %let WhereStmtDisplay = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XandX_Text)),
           f1=%nrbquote(&WhereStmtDisplay), f2=%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XPopulation_Text)),
           f1=%nrbquote(%LocaleEZX(%nrbquote(&PopulationSet), EIA=1))))));
         %else %let WhereStmtDisplay = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XPopulation_Text)), f1=%nrbquote(%LocaleEZX(%nrbquote(&PopulationSet), EIA=1))));
      %end;    
      *** add screen failure to where statement ***;
      %if (&IncSF = No) %then %do;
         %if %sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
            %if %length(&WhereStmt) %then %let WhereStmt = %nrbquote((&WhereStmt) and
             (not(kupcase(ARM) in (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) or kupcase(ARM) = "SCRNFAIL" or missing(ARM))));
            %else %let WhereStmt = %nrbquote(not(kupcase(ARM) in
             (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) or kupcase(ARM) = "SCRNFAIL" or missing(ARM)));
            /*%if %length(&WhereStmtDisplay) %then %let WhereStmtDisplay = %nrbquote(&WhereStmtDisplay and (ARM is neither screen failure nor missing));
            %else %let WhereStmtDisplay = %nrbquote((ARM is neither screen failure nor missing));*/
         %end;
         %if %length(&TrtName) %then %do;
            %if &TrtName ne ARM %then %do;
               /* S1452741 begin */
               %let _tid_ = %sysfunc(open(&in_adsl));
               %let tvnum = %sysfunc(varnum(&_tid_,&TrtName));
               %let tvtype = %sysfunc(vartype(&_tid_,&tvnum));     
               %let _tid_ = %sysfunc(close(&_tid_));                 
               %if &tvtype ^= N %then %do; 
                  %if %length(&WhereStmt) %then %let WhereStmt = %nrbquote((&WhereStmt) and
                   (not(kupcase(&TrtName) in (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) or kupcase(&TrtName) = "SCRNFAIL" or missing(&TrtName))));
                  %else %let WhereStmt = %nrbquote(not(kupcase(&TrtName) in
                   (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) or kupcase(&TrtName) = "SCRNFAIL" or missing(&TrtName)));
                  /*%if %length(&WhereStmtDisplay) %then %let WhereStmtDisplay = %nrbquote(&WhereStmtDisplay and (&TrtName is neither screen failure nor missing));
                  %else %let WhereStmtDisplay = %nrbquote((&TrtName is neither screen failure nor missing));*/
               %end;
               /* kemicl added in review of S1452741*/
               %else %do;
                  %if %length(&WhereStmt) %then %let WhereStmt = %nrbquote((&WhereStmt) and not missing(&TrtName));
                  %else %let WhereStmt = %nrbquote(not missing(&TrtName));
                %end;
               /* S1452741 end */
            %end;    
          %end;    
      %end;    

      %if %length(&SubjectFilter) %then %do;
         %if &Server = Local %then %do;
            %let sfpath = %jg_getPath(/user/clinical/filters,,0,0,1,&studynameunder);
         %end;
         %else %do;
            %let sfpath = %jg_getPath(/user/clinical,,0,0,2,&studynameunder);
         %end;
         *** look up filter data set name ***;
         %if %sysfunc(fileexist(&sfpath./subjectfilters.sas7bdat)) %then %do;
            libname sf "&sfpath";

            %let filterds = ;
            data _null_;
               set sf.subjectfilters;
               if FilterName = "&SubjectFilter" then call symputx("filterds",FilterDataSet);
            run;

            *** apply the filter, population and where statement ***;
            %if %length(&filterds) and %sysfunc(fileexist(&sfpath./&filterds..sas7bdat)) %then %do;
               %if ^%symexist(sfLib) %then %do;
                  %let sfLib = sf;
               %end;
               data adsl;
                  if 0 then set &sfLib..&filterds;
                  if _n_ = 1 then do;
                  declare hash h(dataset:"&sfLib..&filterds");
                     h.defineKey("USUBJID");
                     h.defineData(ALL:"YES");
                     h.defineDone();
                  end;
                  %if %length(&WhereStmt) %then %do;
                     set &in_adsl(where = (%unquote(&WhereStmt)));
                  %end;
                  %else %do;
                     set &in_adsl;
                  %end;
                  if h.find() = 0;
                  label usubjid="&SubjectLabel";
               run;

               %let in_adsl = work.adsl;
               %put updated in_adsl = &in_adsl;
            %end;
            %else %do;
               %put [MS_S_0194];
               %put ERROR: Subject filter &SubjectFilter does not exist for study &StudyName.. Either recreate it or apply a different filter.;
               %let exiterror=1;
               %goto cexit;
            %end;
         %end;
         %else %do;
            %put [MS_S_0195];
            %put ERROR: Study &StudyName does not have any subject filters defined. Use the Create Subject Filter drill-down button to create them.;
            %let exiterror=1;
            %goto cexit;
         %end;       
      %end;
      %else %if &_ProfileFlag %then %do;
         data adsl;
            if 0 then set subjects;
            if _n_ = 1 then do;
            declare hash h(dataset:"subjects");
               h.defineKey("USUBJID");
               h.defineData(ALL:"YES");
               h.defineDone();
            end;
            %if %length(&WhereStmt) %then %do;
               set &in_adsl(where = (%unquote(&WhereStmt)));
            %end;
            %else %do;
               set &in_adsl;
            %end;
            if h.find() = 0;
            label usubjid="&SubjectLabel";
         run;

         %let in_adsl = work.adsl;
         %put updated in_adsl = &in_adsl;
      %end;
      %else %do;
         *** apply the population and where statement ***; 
         data adsl;
            %if %length(%quote(&WhereStmt)) %then %do;
               set &in_adsl(where = (%unquote(&WhereStmt)));
            %end;
            %else %do;
               set &in_adsl;
            %end;
         run;
         %let in_adsl = work.adsl;
         %put updated in_adsl = &in_adsl;   
      %end;
            
      proc sort data = &in_adsl out = adsl;
         by usubjid;
      run;

      %let in_adsl = work.adsl;
      %put updated in_adsl = &in_adsl;  
          
      %if &syserr>4 %then %let exiterror=1;
      %Nobsvars(adsl);
      %if &nobs=0 %then %do;
         %put [MS_S_0196];
         %put ERROR: No subjects satisfy the Filter to Include Subjects or are in all populations selected on the Population tab.;
         %let exiterror=1;
         %goto cexit;
      %end;
   %end;
   %cexit:;
%mend PREPADSL;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* Macro to use in data step that will create a new treatment variable to incorporate crossover */
%macro AssignCOTreatment(Indat, OutDat, DTvar, Return = PMBase);
   %put Assigning crossover treatment;
   %put Indat=&Indat Outdat=&Outdat DTvar=&DTvar TrtStartDateList=&TrtStartDateList TrtVarList=&TrtVarList dtsuffix=&dtsuffix;
   %Global TrtNameP TrtLabelP &return;
   %let TrtNameP = ;
   %let TrtLabelP = ;
   %let COFlag = 1;
   %if %sysfunc(findw(%upcase( &&&Indat.VARS), ABLFL)) %then %let baseflag = ABLFL;
   %else %if %sysfunc(findw(%upcase( &&&Indat.VARS), &Indat.BLFL)) %then %let baseflag = &Indat.BLFL;
   %else %let baseflag = ;
   
   %let _tid_ = %sysfunc(open(&Indat));
     %let trtpfl = %sysfunc(varnum(&_tid_,TRTP));
     %let trtafl = %sysfunc(varnum(&_tid_,TRTA));
     %let perfl = %sysfunc(varnum(&_tid_,APERIOD));
   %let _tid_ = %sysfunc(close(&_tid_));
   
   data &OutDat;
      merge &Indat(in=a) &in_adsl(keep=USUBJID &TrtVarList &TrtStartDateList);
      by USUBJID;
      %CHKDATE(date = &DTVar,suffix = CO);
      array trtdtc{*} &TrtStartDateList;
      array trts{*} &TrtVarList;
      length Trt $ %sysfunc(max(&MaxTrtVarLen,13)) TrtPeriod $ %sysfunc(max(%eval(&MaxTrtVarLen + 5),18));
      label Trt = "Treatment";
      label TrtPeriod = "Treatment (Period)";
      *Trt = "None";
      %put &TrtStartDateList;
      
      %if &perfl and (&trtpfl or &trtafl) %then %do;
               %if %sysfunc(findw(&TrtVarList,TRT01A)) %then %do; Trt = TRTA; %end;
               %else %if %sysfunc(findw(&TrtVarList,TRT01P)) %then %do; Trt = TRTP; %end;               
               Period = APERIOD;
               TrtPeriod = trim(left(Trt))||" ("||trim(left(Period))||")";         
      %end;
      
      %else %do;
	      if not missing(&DTVar) and (missing(errdtCO) or lengthn(trim(left(errdtCO))) = 0) then do;
		 if lengthn(&DTVar) = 19 then do;
		    DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),substr(&DTvar,12,2),substr(&DTvar,15,2),substr(&DTvar,18,2));
		 end;
		 else if lengthn(&DTVar) = 16 then do;
		    DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),substr(&DTvar,12,2),substr(&DTvar,15,2),"00");
		 end;
		 else if lengthn(&DTVar) = 13 then do;
		    DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),substr(&DTvar,12,2),"00","00");
		 end;
		 else if lengthn(&DTVar) = 10 then do;
		    DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),"00","00","00");
		 end;
		 else if lengthn(&DTVar) = 7 then do;
		    DTCvar = dhms(mdy(substr(&DTvar, 6, 2), "01", substr(&DTvar, 1, 4)),"00","00","00");
		 end;  
		 else if lengthn(&DTVar) = 4 then do;
		    DTCvar = dhms(mdy("01", "01", substr(&DTvar, 1, 4)),"00","00","00");
		 end;        
		 do i = &NTrtPeriods to 1 by -1;
		    %if &dtsuffix = sdtm %then %do;
		       if not missing(trtdtc[i]) and DTCvar >= trtdtc[i] then do;
		    %end;
		    %else %do;
		       if not missing(trtdtc[i]) and DTCvar >= dhms(trtdtc[i],"00","00","00") then do;
		    %end;
		       Trt = trts[i];
		       Period = i;
		       i = 0;
		    end;
		 end;
		 *** If DTCvar occurs before any treatment starts set to Pre-treatment period ***;
		 %if &dtsuffix = sdtm %then %do;
		    if not missing(trtdtc[1]) and not missing(DTCvar) and DTCvar < trtdtc[1] then do;
		 %end;
		 %else %do;
		    if not missing(DTCvar) and DTCvar < dhms(trtdtc[1],"00","00","00") then do;
		 %end;
		    Trt = "Pre-Treatment";
		    Period = 0;
		  end;
		 *** Baseline loop ***;
		 %if %length(&baseflag) %then %do;
		    if index(upcase(&baseflag),"Y") then do;
		 	do i = 1 to &NTrtPeriods by 1;
		   	 %if &dtsuffix = sdtm %then %do;
		   	    if not missing(trtdtc[i]) and DTCvar <= trtdtc[i] then do;
		  	  %end;
		   	 %else %do;
		   	    if not missing(trtdtc[i]) and DTCvar <= dhms(trtdtc[i],"00","00","00") then do;
		   	 %end;
		   	    Trt = trts[i];
		   	    Period = i;
		   	    i = &NtrtPeriods + 1;
		   	 end;	   	 
		 	end;
		    end;
		 %end;
		  *** If ADaM variables already exist for TRT or PERIOD then replace ***;
                 %if &trtafl and %sysfunc(findw(&TrtVarList,TRT01A)) %then %do; Trt = TRTA; %end;
                 %else %if &trtpfl and %sysfunc(findw(&TrtVarList,TRT01P)) %then %do; Trt = TRTP; %end;               
                 %if &perfl %then %do; Period = APERIOD; %end;	
                 
                 TrtPeriod = trim(left(Trt))||" ("||trim(left(Period))||")";
	      end;
      %end;
      
      if a;
      drop i;
   run;
   
   /*
   data outlib.co;
      set &outdat;
   run;
   */
   
   /* Check for period-matched baselines */
   %if &NTrtPeriods > 1 and %length(&baseflag) %then %do;
      proc sort data=&OutDat out=temp_base(keep=period) nodupkey;
        by period;
        where &baseflag ne '';
      run;
      %NObsVars(temp_base);
      %if &nobs > 1 %then %let &return = 1;
   %end;
   
   %let TrtName = Trt;
   %let TrtLabel = Treatment;
   %let TrtNameP = TrtPeriod;
   %let TrtLabelP = Treatment (Period);
   %put TrtName = &TrtName;
   %put TrtLabel = &TrtLabel;
%mend AssignCOTreatment;
   
*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* Define new variables &domain.GRP, &domain.GRPN (numeric group index) and &domain.SGRPN (numeric group subindex) in order to meaningfully sort values;
* Useful for Findings Domains, such as LB, VS, ECG;
%macro GroupVars(domain,indat,outdat);

   %if %length(&&ing_&domain) %then %do;

      %put Creating group variables for ing_&domain = &&ing_&domain;
   
      * max number of tags;
      %NObsVars(&&ing_&domain);
      %let nsg = %eval(&nvars-1);
      %let ng = &nobs;
      
      * check for vspos;
      %if %upcase(&domain)=VS %then %do;
         %let gposv = &domain.POS;
         %let _ds = %sysfunc(open(&indat));
         %let gpos = %sysfunc(varnum(&_ds,&gposv));
         %let _ds =%sysfunc(close(&_ds));
         %let nsg = %eval(&nsg/2);
         %if ^&gpos %then %let gposv = ;
      %end;
      %else %do;
         %let gposv = ;
         %let gpos = 0;
      %end;
      
      * max number of subgroups;
      %if ^%symexist(maxnsg) %then %do;
         %global maxnsg;
         %let maxnsg = 0;
      %end;

      * assign macros from the group definition table;
      data _null_;
         set &&ing_&domain end=_last;
         retain maxglen 0;
         call symput("g"||compress(_n_),%upcase(&domain.GRP));
         maxglen = max(maxglen,length(&domain.GRP));
         %do i=1 %to &nsg;
            if lengthn(&domain.TESTC&i) then do;
               call symput("g"||compress(_n_)||"_&i",%upcase(&domain.TESTC&i));
               call symput("ng"||compress(_n_),compress(&i));
               %if &gpos %then %do;
                  call symput("gp"||compress(_n_)||"_&i",%upcase(&domain.POS&i));
               %end;
               if &i > &maxnsg then call symput("maxnsg",compress(&i));
            end;
         %end;
         if _last then do;
            call symput("maxglen",compress(maxglen));
         end;
      run;
      
      * assign nonmatching values to their own group;
      data temp;
         set &indat;
         keep &domain.TESTCD &gposv;
      run;
      proc sort data=temp nodupkey;
         by &domain.TESTCD &gposv;
      run;
      
      data _null_;
         set temp end=_last;
         retain gx &ng;
         * determine if the current value of &domain.TESTCD matches any of the group tags;
         match = 0;
         %do g=1 %to &ng;
            %do i=1 %to &&ng&g;
               %if %length(%str(&&g&g._&i)) %then %do;
                  %if &gpos %then %do;
                     if not match and upcase(&domain.TESTCD)="&&g&g._&i" and upcase(&domain.POS)="&&gp&g._&i" then do;
                        match = 1;
                     end;                 
                  %end;
                  %else %do;
                     if not match and upcase(&domain.TESTCD)="&&g&g._&i" then do;
                        match = 1;
                     end;
                  %end;
               %end;
            %end;
         %end;
         * if no match then put the value into its own group at the end of the list;
         if not match then do;
            gx+1;
            call symput("ng"||compress(gx),compress(1));
            %if &gpos %then %do;
               call symput("g"||compress(gx),
                  cat(trim(left(&domain.TESTCD))," ",trim(left(&domain.POS)))
               );
               call symput("gp"||compress(gx)||"_1",trim(left(&domain.POS)));
            %end;
            %else %do;
               call symput("g"||compress(gx),trim(left(&domain.TESTCD)));
            %end;
            call symput("g"||compress(gx)||"_1",trim(left(&domain.TESTCD)));
         end;
         if _last then do;
            call symput("ng",compress(gx));
         end;
      run;
                  
      %put ng = &ng;
      %do g=1 %to &ng;
         %put g&g = &&g&g;
         %do i=1 %to &&ng&g;
            %put g&g._&i = &&g&g._&i;
            %if &gpos %then %do;
               %put gp&g._&i = &&g&g._&i;
            %end;
         %end;
      %end;
      %put maxglen = &maxglen;
      
      * define new variables based on the macro variables created above;
      data &outdat;
         set &indat;
         retain gx &ng;
         length %upcase(&domain.GRP) $ &maxglen;
         * determine matching tag;
         match = 0;
         %do g=1 %to &ng;
            %do i=1 %to &&ng&g;
               %if &gpos %then %do;
                  if not match and upcase(&domain.TESTCD)="&&g&g._&i" and upcase(&domain.POS)="&&gp&g._&i" then do;
                     %upcase(&domain.GRP) = "&&g&g";
                     %upcase(&domain.GRPN) = &g;
                     %upcase(&domain.SGRPN) = &i;
                     match = 1;
                  end;
               %end;
               %else %do;
                  if not match and upcase(&domain.TESTCD)="&&g&g._&i" then do;
                     %upcase(&domain.GRP) = "&&g&g";
                     %upcase(&domain.GRPN) = &g;
                     %upcase(&domain.SGRPN) = &i;
                     match = 1;
                  end;              
               %end;
            %end;
         %end;
         drop match gx;
      run;
         
   %end;
   %else %do;
      %put No grouping data set found for domain &domain.;
      data &outdat;
         set &indat;
      run;
   %end;
   %gexit:;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*** CHKDATE examines the validity of ISO8601 Dates;
%macro CHKDATE(date = , suffix = );
   %global DateErrFlg;
   errdt&suffix = " ";
   call symput("DateErrFlg", "N");
   if lengthn(compress(&date)) = 0 then errdt&suffix = " ";
   else if lengthn(&date) in (4,7,10,13,16,19) then do;
      if substr(&date,1,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if substr(&date,2,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if substr(&date,3,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if substr(&date,4,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if lengthn(&date) >= 7 then do;
         if substr(&date,5,1) ne "-" then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,6,2) not in ("01","02","03","04","05","06","07","08","09","10","11","12") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
      end;
      else if lengthn(&date) >= 10 then do;
         if substr(&date,8,1) ne "-" then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,9,2) not in ("01","02","03","04","05","06","07","08","09","10","11","12",
                                           "13","14","15","16","17","18","19","20","21","22","23","24",
                                           "25","26","28","29","30","31") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
      end;
      else if lengthn(&date) >= 13 then do;
         if substr(&date,11,1) ne "T" then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,12,2) not in ("00","01","02","03","04","05","06","07","08","09","10","11",
                                            "12","13","14","15","16","17","18","19","20","21","22","23") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
      end;
      else if lengthn(&date) >= 16 then do;
         if substr(&date,14,1) ne ":" then do;
            call symput("DateErrFlg", "Y");
            errdt&suffix = "Y";
         end;
         else if substr(&date,15,2) not in ("00","01","02","03","04","05","06","07","08","09","10","11",
                                            "12","13","14","15","16","17","18","19","20","21","22","23",
                                            "24","25","26","27","28","29","30","31","32","33","34","35",
                                            "36","37","38","39","40","41","42","43","44","45","46","47",
                                            "48","49","50","51","52","53","54","55","56","57","58","59") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
     end;
     else if lengthn(&date) = 19 then do;
         if substr(&date,17,1) ne ":" then do;
           errdt&suffix = "Y";
           call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,18,2) not in ("00","01","02","03","04","05","06","07","08","09","10","11",
                                            "12","13","14","15","16","17","18","19","20","21","22","23",
                                            "24","25","26","27","28","29","30","31","32","33","34","35",
                                            "36","37","38","39","40","41","42","43","44","45","46","47",
                                            "48","49","50","51","52","53","54","55","56","57","58","59") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
     end;
   end;
   else do;
      errdt&suffix = "Y"; 
      call symput("DateErrFlg", "Y");
   end;
%if &DateErrFlg = Y %then %put WARNING: %upcase(&dtcdate) has at least one instance that does not comply to ISO 8601 standards.;
%mend CHKDATE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;
/*Blocks to specify variables used for relative study day calulcations */
/*
$CVS_START$
$ID: CalcStudyDay_Findings$
$CMP: RSYS.Study Day Calculation$ $UE: FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_SV$
$CMP: RSYS.Study Day Calculation$ $UE: SV.SVSTDY or (SV.SVSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_SVO$
$CMP: OSYS.Study Day Calculation$ $UE: SV.SVSTDY or (SV.SVSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_FindingsV$
$CMP: RSYS.Findings Timing Calculation$ $UE: FINDINGS.VISIT or FINDINGS.VISITNUM or FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Findings Timing: Use of Visits or Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_FindingsO$
$CMP: OSYS.Findings Timing Calculation$ $UE: FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Findings Timing: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_FindingsO2$
$CMP: OSYS.Study Day Calculation$ $UE: FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_FindingsO3$
$CMP: CPV.{"Only include on-trial measurements","Yes"}$ $UE: FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_Interventions$
$CMP: RSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxSTDY or (INTERVENTIONS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment.$
$CMP: OSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxENDY or INTERVENTIONS.xxDUR or (INTERVENTIONS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_InterventionsO$
$CMP: OSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxSTDY or (INTERVENTIONS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment. Missing dates set STDY = 0$
$CMP: OSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxENDY or INTERVENTIONS.xxDUR or (INTERVENTIONS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_Events$
$CMP: RSYS.Study Day Calculation$ $UE: EVENTS.xxSTDY or (EVENTS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment.$
$CMP: OSYS.Study Day Calculation$ $UE: EVENTS.xxENDY or EVENTS.xxDUR or (EVENTS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_EventsO$
$CMP: OSYS.Study Day Calculation$ $UE: EVENTS.xxSTDY or (EVENTS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment.$
$CMP: OSYS.Study Day Calculation$ $UE: EVENTS.xxENDY or EVENTS.xxDUR or (EVENTS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

*/
%macro calculateStudyDays(domain);
   %let domain = %upcase(&domain);
   %let useboth = 0;
   %if &domain=DS %then %do;
      %let daysuffix = STDY;
      %let timesuffix = STDTC;
      %let useboth = 1;
   %end;
   %else %if &domain=PP %then %do;
      %let daysuffix = DY;
      %let timesuffix = RFTDTC;
   %end;
   %else %if %sysfunc(findw(&findingsdomains, &domain)) %then %do;
      %let daysuffix = DY;
      %let timesuffix = DTC;
      %if &domain=MH %then %do;
         %let useboth = 1;
      %end;
   %end;
   %else %do;
      %let daysuffix = STDY;
      %let timesuffix = STDTC;
   %end;
   
   STDY = 0;
   if not missing(&domain.&daysuffix) then STDY = &domain.&daysuffix;
   else do;
      %calcStudyDay(&domain.&timesuffix,STDY);
      %if &useboth %then %do;
         if missing(STDY) then do;
            %calcStudyDay(&domain.STDTC,STDY);
         end;
      %end;
      &domain.&daysuffix = STDY;
   end;
   
   %if ^%sysfunc(findw(&findingsdomains, &domain)) or ^%sysfunc(findw(&&&domain.VARS, &domain.STRESN)) %then %do;
      ENDY = 0;
      if not missing(&domain.ENDY) then ENDY = &domain.ENDY;
      /* else if not missing(&domain.DUR) then ENDY = STDY + &domain.DUR;*/
      else do;
         %if %sysfunc(findw(&&&domain.VARS, &domain.ENDTC)) %then %do;
            %calcStudyDay(&domain.ENDTC,ENDY);
         %end;
         %else %do;
            endy = .;
         %end;
      end;
      * if missing(ENDY) then ENDY = STDY;
      if missing(&domain.DUR) and not missing(ENDY) then &domain.DUR = ENDY-STDY;
      LOG2_&domain.DUR = log2(max(1,&domain.DUR));
      if not missing(ENDY) and STDY > ENDY then STDY = ENDY - 1;
   %end;
 %mend; 

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;
/**
Calculate Study Day
Directly from the SDTM IG
--DY = (date portion of --DTC) - (date portion of RFSTDTC) + 1 
if --DTC is on or after RFSTDTC --DY = (date portion of --DTC) - (date portion of RFSTDTC) 
if --DTC precedes RFSTDTC This algorithm should be used across all domains.
**/
%macro calcStudyDay(dtcdate,result,referenceDate = RFSTDTC );
	/* update to accommodate numeric dtcdate specification*/
	if vtype(&dtcdate) = 'N' then do;
		errdt&result = " ";
		call symput("DateErrFlg", "N");		
	end;
	else do;
		/*if not numeric date, check that its correct ISO format and return record level errdt&result as Y if incorrect */
   	%CHKDATE(date = &dtcdate, suffix = &result);
   end;
   
   if not missing(&dtcdate) and not missing(&referenceDate) and errdt&result = " " then do;
   	/* Calculate days for comparison variable into _cdy*/
   	if vtype(&dtcdate) = 'N' then do;
   		_cdy&dtcdate=input(vvalue(&dtcdate),anydtdte22.);
		end;
		else do;
			if find(&dtcdate,'T') then do;
				_cdy&dtcdate = mdy(substr(&dtcdate, 6, 2), substr(&dtcdate, 9, 2), substr(&dtcdate, 1, 4));
				partial_&dtcdate = 0;
			end;
			else if lengthn(scan(&dtcdate,3,'-')) then do;
				_cdy&dtcdate = mdy(substr(&dtcdate, 6, 2), substr(&dtcdate, 9, 2), substr(&dtcdate, 1, 4));
				partial_&dtcdate = 0;
			end;
			else if lengthn(scan(&dtcdate,2,'-')) then do;
				_cdy&dtcdate = mdy(substr(&dtcdate, 6, 2), 1, substr(&dtcdate, 1, 4));
				partial_&dtcdate = 1;
			end;
			else do;
				_cdy&dtcdate = mdy(1, 1, substr(&dtcdate, 1, 4));
				partial_&dtcdate = 1;
			end;
		end;
		/* Calculate days for reference variable into _rdy.  */
   	if vtype(&referenceDate) = 'N' then do;
   		_rdy&dtcdate=input(vvalue(&referenceDate),anydtdte22.);
		end;		
      else if find(&referenceDate,'T') then do;
         _rdy&dtcdate = mdy(substr(&referenceDate, 6, 2), substr(&referenceDate, 9, 2), substr(&referenceDate, 1, 4));
      end;
      else if lengthn(scan(&referenceDate,3,'-')) then do;
         _rdy&dtcdate = mdy(substr(&referenceDate, 6, 2), substr(&referenceDate, 9, 2), substr(&referenceDate, 1, 4));
      end;
      else if lengthn(scan(&referenceDate,2,'-')) then do;
         _rdy&dtcdate = mdy(substr(&referenceDate, 6, 2), 1, substr(&referenceDate, 1, 4));
      end;
      else do;
         _rdy&dtcdate = mdy(1, 1, substr(&referenceDate, 1, 4));
      end;
      if _cdy&dtcdate >= _rdy&dtcdate then &result = _cdy&dtcdate - _rdy&dtcdate + 1;
      else &result = _cdy&dtcdate - _rdy&dtcdate;
      drop _cdy&dtcdate _rdy&dtcdate;
   end;
   else do;
      %if %index(%upcase(&result),STDY) %then %do;
         &result = 0;
      %end;
      %else %do;
         &result = .;
      %end;
      partial_&dtcdate = .;
   end;
***if errdt&result = "Y" then delete;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* create numeric days variable from iso 8601 variable;
%macro DATE9(var=,return=);
   if length(&var) = 4 then do;
      &return = mdy(1, 1, substr(&var, 1, 4));
      d9partial_&var = 1;
   end;
   else if length(&var) = 7 then do;
      &return = mdy(substr(&var, 6, 2), 1, substr(&var, 1, 4));
      d9partial_&var = 1;
   end;
   else if missing(&var) then do;
      &return = .;
      d9partial_&var = 1;
   end;
   else do;
      &return = mdy(substr(&var, 6, 2), substr(&var, 9, 2), substr(&var, 1, 4));
      d9partial_&var = 0;
   end;
   format &return &IntDateFormat..;
%mend DATE9;


*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* create numeric seconds variable from iso 6801 variable;
%macro TIME5(var=,return=);
   if index(&var,"T") then do;
      if lengthn(&var) = 19 then do;
         &return = hms(substr(&var, 12, 2), substr(&var, 15, 2), substr(&var, 18, 2));
      end;
      else if lengthn(&var) = 16 then do;
         &return = hms(substr(&var, 12, 2), substr(&var, 15, 2), 0);
      end;
      else if lengthn(&var) = 13 then do;
         &return = hms(substr(&var, 12, 2), 0, 0);
      end;
   end;
   else do;
      &return = .;
   end;
   format &return time5.;
%mend TIME5;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* add time window string to CatName and CatLabel if specified and/or create time window variable;
%macro AddTimeWindows(domain=,indat=,outdat=,twb=,tvar=,findingscall=0,blflag=0,errflag=1);

   %if &indat ne &outdat %then %do; 
      data &outdat;
         set &indat;
      run;
   %end;
   
   %if ^%symexist(CatName) %then %do;
      %global CatName;
      %let CatName = ;
   %end;
   %if ^%symexist(CatLabel) %then %do;
      %global CatLabel;
      %let CatLabel = &CatName;
   %end;
   %if ^%symexist(GroupVar) %then %do;
      %global GroupVar;
      %let GroupVar = ;
   %end;
   
      
   %global ntw maxtw CatKeepVars;
   %if %length(&twb) %then %do;
      %if ^&findingscall %then %do;
         data dm;
            set &in_dm;
            keep USUBJID rfstdtc;
         run;

         proc sort data=&outdat;
            by USUBJID;
         run;

         proc sort data=dm;
            by USUBJID;
         run;

         data &outdat;
            merge &outdat dm;
            by USUBJID;
            %if %length(&CatName) %then %do;
               if missing(&CatName) then delete;
            %end;
            %calculateStudyDays(&domain);
            if missing(ENDY) then ENDY = STDY;
            %if %Upcase(&TimeScale) = WEEKS %then %do;
               STWK = ceil(STDY/7);
               ENWK = ceil(ENDY/7);
               %let STvar = STWK;
               %let ENvar = ENWK;
            %end;
            %else %do;  
               %let STvar = STDY;
               %let ENvar = ENDY;
            %end;
         run;
      %end;

      * get time window boundary levels;
      %let i=1;
      %let maxtw = 0;
      %if %index(&twb,%str(])) %then %do;
         %do %while(%length(%scan(&twb,&i,']')));
            %global tw&i tws&i twe&i;
            %let tw&i = %qscan(&twb,&i,']')%str(]);
            %if %index(&&tw&i,%str(,)) %then %do;
               %let tws&i = %scan(&&tw&i,1,'[, ]');
               %let twe&i = %scan(&&tw&i,2,'[, ]');
               %let twv&i = 2;
            %end;
            %else %do;
               %let tws&i = %scan(&&tw&i,1,'[ ]');
               %let twe&i = %scan(&&tw&i,1,'[ ]');
               %let twv&i = 1;
            %end;
            %if %length(&&tw&i) > &maxtw %then %let maxtw = %length(&&tw&i);
            %let i=%eval(&i+1);
         %end;
      %end;
      %else %do;
         %do %while(%length(%scan(&twb,&i,' ')));
            %if %index(&twb,%str(,)) %then %do;
               %let exiterror=1;
               %put [MS_S_0089];
               %put ERROR: Commas must be used with brackets when specifying time windows.;
               %goto twexit;
            %end;
            %global tw&i tws&i twe&i;
            %let tw&i = %qscan(&twb,&i,' ');
            %let tws&i =%scan(&twb,&i,' ');
            %let twe&i =%scan(&twb,&i,' ');
            %let twv&i = 1;
            %if %length(&&tw&i) > &maxtw %then %let maxtw = %length(&&tw&i);
            %let i=%eval(&i+1);
         %end;
      %end; 
      %let ntw = %eval(&i-1);
      %put ntw = &ntw;
      %do i=1 %to &ntw;
         %let tw&i = %trim(&&tw&i);
         %put tw&i = &&tw&i;
         /*
         %put tws&i = &&tws&i;
         %put twe&i = &&twe&i;
         %put twv&i = &&twv&i;
         */
      %end;

      * get lengths;
      %if %length(&CatName) %then %do;
         %let _tid_ = %sysfunc(open(&outdat));
         %let vn = %sysfunc(varnum(&_tid_,&CatName));
         %let CatLen = %sysfunc(varlength(&_tid_,&vn));
         %let _tid_ = %sysfunc(close(&_tid_));
         %let CatLabelTW = &CatLabel with Time Window;
      %end;
      %if %length(&GroupVar) %then %do;
         %let _tid_ = %sysfunc(open(&outdat));
         %let vn = %sysfunc(varnum(&_tid_,&GroupVar));
         %let GroupLen = %sysfunc(varlength(&_tid_,&vn));
         %let GroupLabel = %sysfunc(varlabel(&_tid_,&vn));
         %let _tid_ = %sysfunc(close(&_tid_));
         %if ^%length(&GroupLabel) %then %let GroupLabel = &GroupVar;
         %let GroupLabelTW = &GroupLabel with Time Window;
      %end;

      * create a time variable and/or add time window strings to catname;
      data &outdat;
         %if %length(&CatName) %then %do;
            length &CatName.TW $ %eval(&CatLen+&maxtw+1);
         %end;
         %if %length(&GroupVar) %then %do;
            length &GroupVar.TW $ %eval(&GroupLen+&maxtw+1);
         %end;
         %if %length(&tvar) %then %do;
            length &tvar $ &maxtw;
         %end;
         set &outdat;
         gotoutput=0;
         %do i=1 %to &ntw;
            Domain = "&domain";
            TimeWindowNum = &i;
         
            if (&ENvar >= &&tws&i) and (&STvar <= &&twe&i) then do;
               %if %length(&CatName) %then %do;
                  &CatName.TW = cat(trim(left(&CatName))," &&tw&i");
               %end;             
               %if %length(&GroupVar) %then %do;
                  &GroupVar.TW = cat(trim(left(&GroupVar))," &&tw&i");
               %end;             
               %if %length(&tvar) %then %do;
                  &tvar = "&&tw&i";
               %end;
               output;
               gotoutput=1;               
            end;
         %end;
         %if %symexist(BaselineTimeWindow) %then %do;
            %if &blflag and ^%length(&BaselineTimeWindow) %then %do;
               if &domain.BLFL in ('Y',%unquote(%LocaleGetAll(Y_Text, _case = u))) and gotoutput=0 then output;
            %end;
         %end;
         
         %if %length(&CatName) %then %do;
            label &CatName.TW = &CatLabelTW;
         %end;
         %if %length(&GroupVar) %then %do;
            label &GroupVar.TW = &GroupLabelTW;
         %end;
      run;

      %if %length(&CatName) %then %do;
         %global CatName0 CatLabel0;
         %let CatName0 = &CatName;
         %let CatLabel0 = &CatLabel;
         %let CatName = &CatName.TW;
         %let CatLabel = &CatLabelTW;
         %let CatKeepVars = &CatName0 &Tvar TimeWindowNum Domain;
      %end;
      %if %length(&GroupVar) %then %do;
         %global GroupVar0 GroupLabel0;
         %let GroupVar0 = &GroupVar;
         %let GroupLabel0 = &GroupLabel;
         %let GroupVar = &GroupVar.TW;
         %let GroupLabel = &GroupLabelTW;
         %if %length(&CatName) %then %do;
            %let CatKeepVars = &CatKeepVars &GroupVar0;
         %end;
      %end;
            
      %NObsVars(&outdat);
      %if ^&nobs and &errflag %then %do;
         %put [MS_S_0197];
         %put ERROR: No data found within the specified time windows. Please change the windows.;
         %let exiterror = 1;
      %end;
      
      /*
      data outlib.addtime;
         set &outdat;
      run;
      */
   %end;
   %else %do;
      %let ntw = 1;
      %let CatKeepVars = ;
   %end;
   %twexit:;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* create counts to be used in tree map;
%macro createCounts(domain,KeepVars=);

   %if ^%symexist(GroupVar) %then %do;
      %global GroupVar;
      %let GroupVar = ;
   %end;

   data c1;
      set &domain;
      keep USUBJID &GroupVar &CatName &KeepVars;
   run;

   proc sort data=c1 out=c2 nodupkey;
      by USUBJID &GroupVar &CatName &KeepVars;
   run;

   proc sort data=c2;
      by &GroupVar &CatName &KeepVars;
   run;

   data c2;
      set c2;
      one = 1;
   run;

   proc means data=c2 noprint;
      by &GroupVar &CatName &KeepVars;
      var one;
      output out=OutLib.&domain._counts(drop=_TYPE_ _FREQ_) n=Count;
   run;

   proc sort data=OutLib.&domain._counts;
      by &CatName;
   run;

%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* SAS macro for including drill-down buttons for Patient Profiler and Compare Subjects in tabbed report */
/*
   set _fCustomizedFunctions to 1 and define functions objRefNS:ppdd and objRefNS:cpdd prior to calling this macro
   if customized objRefNS:ppdd and objRefNS:cpdd functions are needed. Exposure Summary is one example that needs customized functions;
*/
%macro SubjectDrillDowns(_fCustomizedFunctions=0);

   %if ^&_fCustomizedFunctions %then %do;
      put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   /*   put "objRefNS:cpdd =  evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); clusterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""); clusterFunc(this:getParent());));"; */
      put "objRefNS:fpdd = evalexpr(Function({},{default local},  this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); reviewRefNS = namespace(reportRefNS:getParent()); createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< newContext); createFilterFunc(reviewRefNS << getName);));";  
      put "objRefNS:aendd = evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); aenFunc = Include(""$CLINICAL_HOME\ProcessLibrary\AENarrativeDrilldownFunction.jsl"",<< New Context,<< ""Names Default To Here""); aenFunc(this:getParent(), 1);));";
      put "objRefNS:spdd =  Function({}, {}, Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";
   %end;
   /*Kemicl add for new UI action button toolbar */
   put "    objRefNS:ActionButtonList= {{Expr(locale:get(""ProfileSubjects_Text"")), Expr(locale:get(""ShowSubjects_Text"")), expr(locale:get(""AENarrativeGeneration_Text"")), Expr(locale:get(""CreateSubjectFilter_Text""))}};";
   put "    objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"", ""&IconPath.DataSetUser.gif"",  ""&IconPath.CreateNarratives.gif"",  ""&IconPath.AddFilter.gif""}};";
   put "    objRefNS:ActionSelectionCountList = {{1, 1, 1, 1}};";
   put "    objRefNS:ActionRequirementFlagList = {{3, 0, 3, 2}};";
   put "    objRefNS:ActionCodeList= {{objRefNS:ppdd(), objRefNS:spdd(), objRefNS:aendd(), objRefNS:fpdd()}};";
   /* Kemicl likely will go away */
   put "    objRefNS:ActionButtonDesc = {Expr(locale:get(""SelectSubjectsthenClick_Text""))};";

%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* set flags for demographic variables */
%macro DMFlags;

   %global cflg eflg rflg aflg sflg siflg tflg uflg wflg xflg yflg zflg;
   %global doAge doAgeTxt doSex doRace doTrt doStart doEnd doSite doStudy doCtry doEth doSpec doStrain doSbstrain;

   %let aflg = %sysfunc(indexw(&ADSLVARS, AGE, ' '));
   %let cflg = %sysfunc(indexw(&ADSLVARS, COUNTRY, ' '));
   %let eflg = %sysfunc(indexw(&ADSLVARS, ETHNIC, ' '));
   %let rflg = %sysfunc(indexw(&ADSLVARS, RACE, ' '));
   %let sflg = %sysfunc(indexw(&ADSLVARS, SITEID, ' '));
   %let siflg = %sysfunc(indexw(&ADSLVARS, STUDYID, ' '));
   %let tflg = %sysfunc(indexw(&ADSLVARS, RFSTDTC, ' '));
   %let uflg = %sysfunc(indexw(&ADSLVARS, RFENDTC, ' '));
   %let wflg = %sysfunc(indexw(&ADSLVARS, AGETXT, ' '));
   %let xflg = %sysfunc(indexw(&ADSLVARS, SPECIES, ' '));
   %let yflg = %sysfunc(indexw(&ADSLVARS, STRAIN, ' '));
   %let zflg = %sysfunc(indexw(&ADSLVARS, SBSTRAIN, ' '));
   
   %let doAge = 1;
   %let doAgeTxt = 1;
   %let doSex = 1;
   %let doRace = 1;
   %let doTrt = 1;
   %let doStart = 1;
   %let doEnd = 1;
   %let doSite = 1;
   %let doStudy = 1;
   %let doCtry = 1;
   %let doEth = 1;
   %let doSpec = 1;
   %let doStrain = 1;
   %let doSbstrain = 1;
   %let Country = %str(COUNTRY);
   %let Ethnic = %str(ETHNIC);
   %let Race = %str(RACE);
   %let Age = %str(AGE);
   %let StartDate = %str(RFSTDTC);
   %let EndDate = %str(RFENDTC);
   %let Site = %str(SITE);
   %let Species = %str(SPECIES);
   %let Strain = %str(STRAIN);
   %let Sbstrain = %str(SBSTRAIN);
   %let AgeTxt = %str(AGETXT);

   %if ^&aflg %then %do;
      %let doAge = 0;
      %let Age = ;
   %end;
   %if ^&sflg %then %do;
      %let doSite = 0;
      %let Site = ;
   %end;
   %if ^&siflg %then %do;
      %let doStudy = 0;
      %let Study = ;
   %end;   
   %if ^&cflg %then %do;
      %let doCtry = 0;
      %let Country = ;
   %end;
   %if ^&eflg %then %do;
      %let doEth = 0;
      %let Ethnic = ;
   %end;
   %if ^&rflg %then %do;
      %let doRace = 0;
      %let Race = ;
   %end;
   %if ^&tflg %then %do;
      %let doStart = 0;
      %let StartDate = ;
   %end;
   %if ^&uflg %then %do;
      %let doEnd = 0;
      %let EndDate = ;
   %end;
   %if ^&xflg %then %do;
      %let doSpec = 0;
      %let Species = ;
   %end;
   %if ^&yflg %then %do;
      %let doStrain = 0;
      %let Strain = ;
   %end;
   %if ^&zflg %then %do;
      %let doSbstrain = 0;
      %let Sbstrain = ;
   %end;
   %if ^&wflg %then %do;
      %let doAgeTxt = 0;
      %let AgeTxt = ;
   %end;

   %if %length(&DesignByVars) %then %do;
      %do i=1 %to %sysfunc(countw(&DesignByVars));
         %let bvi = %upcase(%scan(&DesignByVars,&i,' '));
         %if &bvi=AGE %then %let doAge = 0;
         %else %if &bvi=SEX %then %let doSex = 0;
         %else %if &bvi=RACE %then %let doRace = 0;
         %else %if &bvi=COUNTRY %then %let doCtry = 0;      
         %else %if &bvi=SITEID %then %let doSite = 0;
         %else %if &bvi=STUDYID %then %let doStudy = 0;
         %else %if &bvi=ETHNIC %then %let doEth = 0;     
         %else %if &bvi=SPECIES %then %let doSpec = 0;      
         %else %if &bvi=STRAIN %then %let doStrain = 0;
         %else %if &bvi=SBSTRAIN %then %let doSbstrain = 0;    
         %else %if &bvi=AGETXT %then %let doAgeTxt = 0;     
         %else %if &bvi=RFSTDTC %then %let doStart = 0;     
         %else %if &bvi=RFENDTC %then %let doEnd = 0;     
         %else %if &bvi=%upcase(&TrtName) %then %let doTrt = 0;   
      %end;
   %end;

   %if ^&doAge and ^&doSex and ^&doRace and ^&doTrt and ^&doCtry and ^&doSite and ^&doEth and 
      ^&doSpec and ^&doStrain and ^&doSbstrain and ^&doAgeTxt and ^&doStart and ^&doEnd %then %do;
      %put [MS_S_0198];
      %put ERROR: At least one of &Age SEX &Race &Site &Country &Ethnic &Species &Strain &Sbstrain &AgeTxt &StartDate &EndDate or &TrtName must not be specified as a By Variable.;
      %let exiterror = 1;
   %end;

   %if %upcase(&TrtVar) = SEX %then %let doSex = 0;
   %else %if %upcase(&TrtVar) = RACE %then %let doRace = 0;
   %else %if %upcase(&TrtVar) = AGE %then %let doAge = 0;
   %else %if %upcase(&TrtVar) = COUNTRY %then %let doCtry = 0;
   %else %if %upcase(&TrtVar) = SITEID %then %let doSite = 0;
   %else %if %upcase(&TrtVar) = SITEID %then %let doStudy = 0;
   %else %if %upcase(&TrtVar) = ETHNIC %then %let doEth = 0;
   %else %if %upcase(&TrtVar) = SPECIES %then %let doSpec = 0;    
   %else %if %upcase(&TrtVar) = STRAIN %then %let doStrain = 0;
   %else %if %upcase(&TrtVar) = SBSTRAIN %then %let doSbstrain = 0;     
   %else %if %upcase(&TrtVar) = RFSTDTC %then %let doSbstrain = 0;     
   %else %if %upcase(&TrtVar) = RFENDTC %then %let doSbstrain = 0;     
   %else %if %upcase(&TrtVar) = AGETXT %then %let doAgeTxt = 0;
   
   %global DMKeepVars;
   %let DMKeepVars = ;
   %if &doSex %then %let DMKeepVars = &DMKeepVars SEX;
   %if &doAge %then %let DMKeepVars = &DMKeepVars AGE;
   %if &doAgeTxt %then %let DMKeepVars = &DMKeepVars AGETXT;
   %if &doRace %then %let DMKeepVars = &DMKeepVars RACE;
   %if &doCtry %then %let DMKeepVars = &DMKeepVars COUNTRY;
   %if &doSite %then %let DMKeepVars = &DMKeepVars SITEID;
   %if &doStudy %then %let DMKeepVars = &DMKeepVars STUDYID;
   %if &doEth %then %let DMKeepVars = &DMKeepVars ETHNIC;
   %if &doSpec %then %let DMKeepVars = &DMKeepVars SPECIES;
   %if &doStrain %then %let DMKeepVars = &DMKeepVars STRAIN;
   %if &doSbstrain %then %let DMKeepVars = &DMKeepVars SBSTRAIN;
   %if &doStart %then %let DMKeepVars = &DMKeepVars RFSTDTC;
   %if &doEnd %then %let DMKeepVars = &DMKeepVars RFENDTC;
   
%mend;

/* Macro to check DM or ADSL to see if there is more than 1 StudyID value */
%macro CheckStudyLevels;
   %global StudyLevs;
   %if ^%symexist(TrtName) %then %let TrtName = ;
   %if ^%symexist(DesignByVars) %then %let DesignByVars = ;
   %let StudyLevs = 0;

   %if %sysfunc(findw(%upcase(&ADSLVARS), STUDYID)) > 0 %then %do;
      proc sql noprint; 
         select distinct STUDYID 
         into :uniquelevels separated by ' '
         from &in_adsl;
         %let StudyLevs = &sqlobs;
      quit;
   %end;

   %if (&StudyLevs > 1 and %upcase(STUDYID) ne %upcase(&TrtName) and ^%index(%Upcase(&DesignByVars),STUDYID)) %then %do;
      %put Data contains records from &StudyLevs Studies;
      %let doStudy = 1;
   %end;
   %else %let doStudy = 0;
%mend;


*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* set labels for demographic variables */
%macro DMLabels(OutData);

   %global AgeLabel AgeTxtLabel SexLabel  RaceLabel  SiteLabel  StudyLabel CtryLabel  EthLabel SpecLabel StrainLabel SbstrainLabel;

   %if &aflg %then %do;
      %let AgeLabel = AGE;
      %CheckLabel(&OutData,AgeLabel);
   %end;
   %if &wflg %then %do;
      %let AgeTxtLabel = AGETXT;
      %CheckLabel(&OutData,AgeTxtLabel);
   %end;
   %let SexLabel = SEX;
   %CheckLabel(&OutData,SexLabel);
   %if &rflg %then %do;
      %let RaceLabel = RACE;
      %CheckLabel(&OutData,RaceLabel);
   %end;
   %if &sflg %then %do;
      %let SiteLabel = SITEID;
      %CheckLabel(&OutData,SiteLabel);
   %end;
   %if &siflg %then %do;
      %let STUDYLabel = STUDYID;
      %CheckLabel(&OutData,STUDYLabel);
   %end;   
   %if &cflg %then %do;
      %let CtryLabel = COUNTRY;
      %CheckLabel(&OutData,CtryLabel);
   %end;
   %if &eflg %then %do;
      %let EthLabel = ETHNIC;
      %CheckLabel(&OutData,EthLabel);
   %end;
   %if &xflg %then %do;
      %let SpecLabel = SPECIES;
      %CheckLabel(&OutData,SpecLabel);
   %end;
   %if &yflg %then %do;
      %let StrainLabel = STRAIN;
      %CheckLabel(&OutData,StrainLabel);
   %end;
   %if &zflg %then %do;
      %let SbstrainLabel = SBSTRAIN;
      %CheckLabel(&OutData,SbstrainLabel);
   %end;

%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro COMPAREDATES(start = , stop = );
   _stmo = substr(&start, 6, 2) + 0;
   _stda = substr(&start, 9, 2) + 0;
   _stye = substr(&start, 1, 4) + 0;
   
   _enmo = substr(&stop, 6, 2) + 0;
   _enda = substr(&stop, 9, 2) + 0;
   _enye = substr(&stop, 1, 4) + 0;
   
   _start = mdy(_stmo, _stda, _stye);
   _end = mdy(_enmo, _enda, _enye);
   _orderr = "";
   
   if _stda > . and _enda > . then do;
      if _start > _end then _orderr = "Y";
   end;
   else if _stmo > . and _enmo > . then do;
      if _stye = _enye and _stmo > _enmo then _orderr = "Y";
   end;
   else if _stye > . and _enye > . then do;
      if _stye > _enye then _orderr = "Y";
   end;
%mend COMPAREDATES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro PROFILEDATEIMPUTE;
   *** Leaving out Medical History since it isnt in main profiler;
   /*
   %let implist = AE CE CM SU;
   %if %sysfunc(findw(&implist, &temp)) %then %do;
   */
   %if (%sysfunc(findw(&eventsdomains, %upcase(&temp))) and %upcase(&temp)^=MH) or %sysfunc(findw(&interventionsdomains, %upcase(&temp)))
      or %upcase(&temp)=CE or %upcase(&temp)=SU %then %do;
      
      *** Start Date ***;
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) %then %do;
         _impstart = "";
         %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STRF, ' ')) %then %do;
            if &temp.STDTC = "" and kupcase(&temp.STRF) not in ("U","UNKNOWN",
             %unquote(%LocaleGetAll(U_Text, _case = u)),%unquote(%LocaleGetAll(Unknown_Text, _case = u))) then do;
               &temp.STDTC = RFSTDTC;
               _impstart = "Y";
            end;
         %end;
         %else %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STRTPT, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STTPT, ' ')) %then %do;
            if &temp.STDTC = "" and kupcase(&temp.STRTPT) not in ("U","UNKNOWN",
             %unquote(%LocaleGetAll(U_Text, _case = u)),%unquote(%LocaleGetAll(Unknown_Text, _case = u))) then do;
               &temp.STDTC = &temp.STTPT;
               _impstart = "Y";
            end;           
         %end;
      %end;
      
      *** End Date ***;
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do;
         _impend = "";
         %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENRF, ' ')) %then %do;
            if &temp.ENDTC = "" and kupcase(&temp.ENRF) not in ("U","UNKNOWN",
             %unquote(%LocaleGetAll(U_Text, _case = u)),%unquote(%LocaleGetAll(Unknown_Text, _case = u))) then do;
               &temp.ENDTC = RFENDTC;
               _impend = "Y";
            end;
         %end;
         %else %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENRTPT, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENTPT, ' ')) %then %do;
            if &temp.ENDTC = "" and kupcase(&temp.ENRTPT) not in ("U","UNKNOWN",
             %unquote(%LocaleGetAll(U_Text, _case = u)),%unquote(%LocaleGetAll(Unknown_Text, _case = u))) then do;
               &temp.ENDTC = &temp.ENTPT;
               _impend = "Y";
            end;           
         %end;   
         %else %if %upcase(&temp) = AE and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)OUT, ' ')) %then %do;
            if &temp.ENDTC = "" and (kupcase(AEOUT) in ("NOT RECOVERED/NOT RESOLVED", "NOT RECOVERED", "NOT RESOLVED",
             "RECOVERING/RESOLVING", "RECOVERING", "RESOLVING", "CONTINUING", "ONGOING", 
             * Why are so few of these strings translated?;
             /*%unquote(%LocaleGetAll(_Text, _case = u)),
             %unquote(%LocaleGetAll(_Text, _case = u)),
             %unquote(%LocaleGetAll(_Text, _case = u)),
             %unquote(%LocaleGetAll(_Text, _case = u)),
             %unquote(%LocaleGetAll(_Text, _case = u)),
             %unquote(%LocaleGetAll(_Text, _case = u)),
             %unquote(%LocaleGetAll(_Text, _case = u)),*/
             %unquote(%LocaleGetAll(Ongoing_Text, _case = u))) 
               or kindex(kupcase(AEOUT), "CONTINUING")>0) then do;
               &temp.ENDTC = RFENDTC;
               _impend = "Y";
            end;
         %end;
      %end;
    
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do; 
         %COMPAREDATES(start = &temp.STDTC, stop = &temp.ENDTC);
         if _orderr = "Y" then do;
           if _impstart = "Y" then &temp.STDTC = "";
           if _impend = "Y" then &temp.ENDTC = "";
         end;
      %end;
      
      if _impstart = "Y" or _impend = "Y" then do;
         &temp.ENDY = .;
         &temp.DUR = .;
         %calculateStudyDays(&temp);
      end;
      
      *** Clean Up ***;
      /* %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) %then %do;
         drop _impstart;
      %end;
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do;
         drop _impend;
      %end; 
      */
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do;
         drop _stmo _stda _stye _enmo _enda _enye _start _end _orderr;
      %end;     
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)STRF)) %then %do;
         drop &temp.STRF;
      %end;
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)ENRF)) %then %do;
         drop &temp.ENRF;
      %end;
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)STRTPT)) %then %do;
         drop &temp.STRTPT;
      %end;       
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)STTPT)) %then %do;
         drop &temp.STTPT;            
      %end;       
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)ENRTPT)) %then %do;
         drop &temp.ENRTPT;
      %end;       
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)ENTPT)) %then %do;
         drop &temp.ENTPT;
      %end;                   
   %end;
%mend PROFILEDATEIMPUTE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*** Get Variable Based on Drop Down Label ***;
%macro GETMEDDRAVAR(domain = AE, label = , column = );
   %if &sysscp = WIN %then %do;
      %let _slash_ = \;
   %end;
   %else %do;
      %let _slash_ = /;
   %end;

   %let studynameunder = %sysfunc(transtrn(&StudyName, %str( ), _));
   %global &column;
   
   %let flabel =;
   
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
   
   data _null_;
      length column $ 20 label $ 100;
      retain foundit 0;
      infile "%jg_getPath(/user/clinical/metadata/base, &domain.MEDDRA.txt, 0, 0, 3, &studynameunder)" delimiter = "~";
      input column $ label $;
      if (foundit < 1) then do;
         if (label = "&label") then foundit = 1;
         /* Disabling translated labels per S1043149
         else do;
            call symput("flabel", label);
            if resolve('%LocaleAreEquiv(&flabel, &label)') = 1 then foundit = 1;
         end;
         */
         if foundit = 1 then call symput("&column", upcase(trim(column)));
      end;
   run;     
%mend GETMEDDRAVAR;

/*kemicl Notes:  do we really want to categorize variables twice? Once because at least one is needed from the list supplied to the parameter and 
twice to be specific to the conditional parameter choice? */
/*
$CVS_START$
$ID: AETermValues$
$CMP: RSYS.Term Variable$ $UE: AE.AEDECOD or AE.AETERM or AE.AEBODSYS or AE.AELLT or AE.AEHLT or AE.AEHLGT or AE.AESOC$ 
   $NOTES: At least one variable that provides the adverse event term value must be present.$ 
$CMP: CPV.{"Term Level","Reported Term for the Adverse Event"}$
   $UE: AE.AETERM$
   $NOTES: Also supplies Group Level Parameter.$   
$CMP: CPV.{"Term Level","Lowest Level Term"}$
   $UE: AE.AELLT$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Term Level","Dictionary-Derived Term"}$
   $UE: AE.AEDECOD$
   $NOTES: Also supplies Group Level Parameter. Default value for Term Level if found in data.$
$CMP: CPV.{"Term Level","Body System or Organ Class"}$
   $UE: AE.AEBODSYS$
   $NOTES: Also supplies Group Level Parameter. Default value for Group Level if found in data$
$CMP: CPV.{"Term Level","High Level Term"}$
   $UE: AE.AEHLT$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Term Level","High Level Group Term"}$
   $UE: AE.AEHLGT$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$   
$CMP: CPV.{"Term Level","Primary System Organ Class"}$
   $UE: AE.AESOC$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CVS_STOP$
*/

*** Ensure that the Grouping Variable is at least the level of Term Variable ***;
%macro COMPAREMEDDRA(domain = AE, term = , group = );
   %if ^%symexist(DoubleFDR) %then %do;
      %Global DoubleFDR;
      %let DoubleFDR = no;
   %end;
   %if %upcase(&DoubleFDR) = YES %then %do;
      %if &term = &domain.TERM or &term = &domain.LLT %then %do;        
      %end;
      %else %if &term =  &domain.DECOD and (&group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;    
      %else %if (&term = &domain.BODSYS or &term = &domain.SOC) and (&group = &domain.DECOD or &group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;
      %else %if &term = &domain.HLT and (&group = &domain.BODSYS or &group = &domain.SOC or &group = &domain.DECOD or &group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &&domain.TermLevel cannot have &&domain.GroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;    
      %else %if &term = &domain.HLGT and (&group = &domain.HLT or &group = &domain.BODSYS or &group = &domain.SOC or &group = &domain.DECOD or &group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;
   %end;
   %else %if %upcase(&DoubleFDR) = NO %then %do;
      %if &term = &domain.TERM or &term = &domain.LLT %then %do;        
      %end;
      %else %if &term =  &domain.DECOD and (&group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put NOTE: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable. Grouping variable set to &&&domain.TermLevel.;
         %let GroupVar = &CatName;
      %end;    
      %else %if (&term = &domain.BODSYS or &term = &domain.SOC) and (&group = &domain.DECOD or &group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put NOTE: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable. Grouping variable set to &&&domain.TermLevel.;
         %let GroupVar = &CatName;
      %end;
      %else %if &term = &domain.HLT and (&group = &domain.BODSYS or &group = &domain.SOC or &group = &domain.DECOD or &group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put NOTE: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable. Grouping variable set to &&&domain.TermLevel.;
         %let GroupVar = &CatName;
      %end;    
      %else %if &term = &domain.HLGT and (&group = &domain.HLT or &group = &domain.BODSYS or &group = &domain.SOC or &group = &domain.DECOD or &group = &domain.TERM or &group = &domain.LLT) %then %do;
         %put NOTE: &&&domain.TermLevel cannot have &&&domain.GroupLevel as a grouping variable. Grouping variable set to &&&domain.TermLevel.;
         %let GroupVar = &CatName;        
      %end; 
   %end;
%mend COMPAREMEDDRA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* check AESerOnly and modify AEWhereStmt accordingly, accounting for translation of "Y";

%macro AESerOnlyToAEWhereStmt;
   %let trY = %nrbquote(%LocaleGet(Y_Text));
   %if %upcase(&AESerOnly)=YES %then %do;
      %if %length(&AEWhereStmt) %then %let AEWhereStmt = %nrbquote((&AEWhereStmt) and (AESER="Y" or AESER in (%unquote(%LocaleGetAll(Y_Text, _case = u)))));
      %else %let AEWhereStmt = %nrbquote(AESER="Y" or (AESER="Y" or AESER in (%unquote(%LocaleGetAll(Y_Text, _case = u)))));
      %if ^%symexist(AEWhereStmtDisplay) %then %let AEWhereStmtDisplay=;
      %if %length(&AEWhereStmtDisplay) %then %let AEWhereStmtDisplay = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XandX_Text)),
        f1=%nrbquote(&AEWhereStmtDisplay), f2=%nrbquote(%LocaleGet(SeriousAdverseEventsOnly_Text))));
      %else %let AEWhereStmtDisplay = %nrbquote(%LocaleGet(SeriousAdverseEventsOnly_Text));
   %end;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro NONNULLVARS(dataset = rbm_jmpc , varlistname = RBMVARS);
   options nonotes;
   ods exclude all;
   ods noresults;
   
   proc freq data = &dataset nlevels;
      ods output nlevels=temp;
   run;

   %let _tid_ = %sysfunc(open(temp));
   %let _vn_ = %sysfunc(varnum(&_tid_,nnonmisslevels));
   %let _tid_ = %sysfunc(close(&_tid_));  
   
   %global &varlistname;
   %let &varlistname = ;

   %if &_vn_ %then %do;   
      proc sql noprint;      
         select distinct upcase(tablevar)
            into :&varlistname separated by ' '
            from temp(where=(nnonmisslevels > 0))
            order by upcase(tablevar);
      quit;
   %end;
   %else %do;
      proc sql noprint;      
         select distinct upcase(tablevar)
            into :&varlistname separated by ' '
            from temp
            order by upcase(tablevar);
      quit;   
   %end;

   proc datasets nolist;
      delete temp;
   quit;
        
   options notes;
   ods exclude none;
   ods results;
%mend NONNULLVARS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro OUTNAMETRUNC(outdat =, truncnum = );    
   %if %length(&&&outdat)>&truncnum %then %let &outdat = %substr(&&&outdat,1,&truncnum);
   %if %length(&&&outdat) %then %let Outname = &&&outdat.._;
   %else %let Outname = &&&outdat;
   %let &outdat =;
%mend OUTNAMETRUNC;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro ADSLMergeDM (temp=adsl);
   %if %length(&SDTMMETA) & %length(&adslpresent) %then %do;
      proc sql noprint;
         create table &temp as
         select * from &temp a1 left join &in_DM a2
         on a1.USUBJID=a2.USUBJID;
         quit;
   %end;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro PARSEFINDINGS(dlist = , flist = );
   *** initialize variable lists ***;
   %let i = 1;
   %do %while(%length(%scan(&dlist,&i,' ')));
      %let temp = %scan(&dlist,&i,' ');
     %global &temp.findlist;
     %let &temp.findlist =;
      %let i = %eval(&i+1);
   %end;

   *** fill variable lists ***;
   %let i = 1;
   %do %while(%length(%scan(&flist,&i,' ')));
      %let temp = %scan(&flist,&i,' ');
     %let prefix = %scan(&temp,1,':'); 
     %let suffix = %scan(&temp,2,':'); 
      %if ^%length(&&&prefix.findlist) %then %let &prefix.findlist = &&&prefix.findlist %nrbquote("&suffix");
     %else %let &prefix.findlist = &&&prefix.findlist,%nrbquote("&suffix");
      %let i = %eval(&i+1);
   %end;
%mend PARSEFINDINGS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CHECKOVERLAP(flist = , slist = );
   %global overlaplist;
   %let overlaplist =;
   %let i = 1;
   %do %while(%length(%scan(&flist,&i,' ')));
      %let temp = %scan(&flist,&i,' ');
     %if %sysfunc(findw(&slist, &temp)) %then %let overlaplist = &overlaplist &temp;
      %let i = %eval(&i+1);
   %end;
   %if %length(&overlaplist) %then %do;
      %put [MS_S_0200];
      %put ERROR: The following findings tests are listed as both categorical and continuous: &overlaplist..;
     %put ERROR: Ensure each test is listed as either categorical or continuous in the dialog and re-run.;
      %let exiterror = 1;
   %end;
%mend CHECKOVERLAP;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CHECKPRES(domain = , var = , adddomain = 0);
   %if ^&adddomain %then %do;
      %global %upcase(&var);
      %if %sysfunc(findw(%upcase(&&&domain.VARS), %upcase(&var))) %then %let %upcase(&var) = %upcase(&var);
      %else %let %upcase(&var) = ;
   %end;
   %else %do;
      %global %upcase(&domain&var);
      %if %sysfunc(findw(%upcase(&&&domain.VARS), %upcase(&var))) %then %let %upcase(&domain&var) = %upcase(&var);
      %else %let %upcase(&domain&var) = ;
   %end;
%mend CHECKPRES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro WHEREDELTA(deltastate =, wherestate =, deltaflag = );
   %if %kupcase(&&&deltastate) = %kupcase(%LocaleGet(New_Text)) or %kupcase(&&&deltastate) = %kupcase(%LocaleGet(Stable_Text)) %then %do;
      %if %length(&&&wherestate) %then %do;
         %let &wherestate = (&&&wherestate) and kupcase(&deltaflag) = "%kupcase(&&&deltastate)";
      %end;
      %else %do;
         %let &wherestate =  kupcase(&deltaflag) = "%kupcase(&&&deltastate)";     
      %end; 
   %end;
   /* %else %if %upcase(&&&deltastate) = OTHER THAN STABLE %then %do;
      %if %length(&&&wherestate) %then %do;
         %let &wherestate = (&&&wherestate) and upcase(&deltaflag) ne "STABLE";
      %end;
      %else %do;
         %let &wherestate = upcase(&deltaflag) ne "STABLE";     
      %end;    
   %end;*/     
%mend WHEREDELTA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro TEAETEXTFLAG(TEAEstring = &TEAEOnly);
   %global TEAEText;   
   %let trsent = %nrbquote(%LocaleGet(XdeterminedusingY_Text));   
   %let TEAEText = ;   
   %if %index(%upcase(&TEAEstring),EMERGENT) %then %do;
      %let trterm = %nrbquote(%LocaleGet(Treatmentemergence_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOM..&TEAEVARFLAG));
   %end;
   %else %if %index(%upcase(&TEAEstring),PRE) %then %do;
      %let trterm = %nrbquote(%LocaleGet(Pretreatment_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOMs..&TEAEVARFLAGs));
   %end;
   %else %if %index(%upcase(&TEAEstring),OFF) %then %do;
      %let trterm = %nrbquote(%LocaleGet(Offtreatmentfollowup_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOMe..&TEAEVARFLAGe));
   %end;
   %else %if %index(%upcase(&TEAEstring),ON) %then %do;
      %let trsent = %nrbquote(%LocaleGet(XdeterminedusingYandZ_Text));
      %let trterm = %nrbquote(%LocaleGet(Ontreatment_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOMs..&TEAEVARFLAGs, &TEAEVARDOMe..&TEAEVARFLAGe));
   %end;
%mend TEAETEXTFLAG;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;